// 'use strict';
/* jshint esversion: 6 */
/* jshint multistr: true */
/* globals Ext, _, capitalizeFirstLetter, addDataIdToElement, joinAndCapitalize */

Ext.define('DSS.component.ConditionConfigurator', {
    extend: 'DSS.component.Base',
    height: '100%',
    layout: 'border',
    conditionSelectorName: null,
    selectedElement: null,
    fEditMode: false,
    gridStore: null,
    grids: {},
    macPanel: null,
    mainGrid: null,
    lblSelectorAnzeige: null,
    oButtonNew: null,
    oButtonEdit: null,
    oButtonUp: null,
    oButtonDown: null,
    oButtonDelete: null,
    iDefaultWidth: 150,
    iDefaultHeight: 293,
    oActualValue: null,
    fIgnorePresence: false,
    fHeatingOperationMode: false,
    stateGridText: [],
    updatePanelDescription: function () {
        if (this.macPanel.collapsed) {
            this.macPanel.setTitle('');
            return;
        }

        var sConditionAction = _(this.getSelectedMainRecord() == null ? 'new condition: ' : 'edit condition: ');

        window.selection = this.grids.lvl1.getSelectionModel().getSelection();
        if (!selection.length) {
            this.macPanel.setTitle(sConditionAction + _('incomplete condition'));
            return;
        }

        if (this.oActualValue == null || this.oActualValue.data == null || this.oActualValue.data.condition == null) {
            sConditionAction += _('incomplete condition');
        } else {
            sConditionAction += selection[0].get('name');   // it's already translated
        }
        this.macPanel.setTitle(sConditionAction);
    },

    getDeviceStore: function (ZoneID) {
        if (Ext.data.StoreManager.lookup('conditionConfigurator_deviceStore_' + ZoneID) == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'conditionConfigurator_deviceStore_' + ZoneID,
                fields: [ 'dSID', 'dSUID', 'name', 'icon', 'present' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            var oDeviceArray = dss.staticDataModel.getDevicesForZone(ZoneID, 0);
            if (oDeviceArray) {
                for (var iIndex = 0; iIndex < oDeviceArray.length; iIndex++) {
                    if ((oDeviceArray[iIndex].tasterMode == 0)||(oDeviceArray[iIndex].tasterMode == 15))
                        Ext.data.StoreManager.lookup('conditionConfigurator_deviceStore_' + ZoneID).add(oDeviceArray[iIndex]);
                }
            }
        }
        return Ext.data.StoreManager.lookup('conditionConfigurator_deviceStore_' + ZoneID);
    },

    defMainGridRenderer: function (val, metadata, record, colIdx, store, view) {    // renders list of existing conditions in Conditions tab of the editor window
        var me = this.parent;
        var oAction = record.get('condition');

        const rowTableTag = '<table data-item="' + (this.parent.conditionSelectorName ? this.parent.conditionSelectorName : '') + 'Item">';
        let sError = '';

        switch (oAction.type) {
            case 'date':
                var startDate = oAction.value.start;
                var startDay = parseInt(startDate.substr(6, 2), 10);
                var startMonth;
                switch(parseInt(startDate.substr(4, 2), 10)) {
                    case 1:
                        startMonth = _('january');
                    break;
                    case 2:
                        startMonth = _('february');
                    break;
                    case 3:
                        startMonth = _('march');
                    break;
                    case 4:
                        startMonth = _('april');
                    break;
                    case 5:
                        startMonth = _('may');
                    break;
                    case 6:
                        startMonth = _('june');
                    break;
                    case 7:
                        startMonth = _('july');
                    break;
                    case 8:
                        startMonth = _('august');
                    break;
                    case 9:
                        startMonth = _('september');
                    break;
                    case 10:
                        startMonth = _('october');
                    break;
                    case 11:
                        startMonth = _('november');
                    break;
                    case 12:
                        startMonth = _('december');
                    break;
                }

                var endDate = oAction.value.end;
                var endDay = parseInt(endDate.substr(6, 2), 10);
                var endMonth;
                switch(parseInt(endDate.substr(4, 2), 10)) {
                    case 1:
                        endMonth = _('january');
                    break;
                    case 2:
                        endMonth = _('february');
                    break;
                    case 3:
                        endMonth = _('march');
                    break;
                    case 4:
                        endMonth = _('april');
                    break;
                    case 5:
                        endMonth = _('may');
                    break;
                    case 6:
                        endMonth = _('june');
                    break;
                    case 7:
                        endMonth = _('july');
                    break;
                    case 8:
                        endMonth = _('august');
                    break;
                    case 9:
                        endMonth = _('september');
                    break;
                    case 10:
                        endMonth = _('october');
                    break;
                    case 11:
                        endMonth = _('november');
                    break;
                    case 12:
                        endMonth = _('december');
                    break;
                }


                return rowTableTag + '<tr><td>' + _('Dateframe from') + ' ' + startDay + '. '+ startMonth + ' ' + _('to') + ' ' + endDay + '. '+ endMonth + '</td></tr></table>';
            break;
            case 'addon-state':
                if (oAction.addonid == 'system-addon-user-defined-states') {
                    if (dss.staticDataModel.userStates) {
                        for (let iIndex = 0; iIndex < dss.staticDataModel.userStates.length; iIndex++) {
                            if (dss.staticDataModel.userStates[iIndex].name == oAction.stateName) {
                                var valueText;
                                var image;
                                if (oAction.stateValue == 1) {
                                    valueText = dss.staticDataModel.userStates[iIndex].setName;
                                    image = dss.staticDataModel.baseImagePath + 'state/uds_state_active.png';
                                } else {
                                    valueText = dss.staticDataModel.userStates[iIndex].resetName;
                                    image = dss.staticDataModel.baseImagePath + 'state/uds_state_inactive.png';
                                }
                                return rowTableTag + '<tr><td class="icon-small"><img src="' + image + '"></td><td style="' + sError + '">' +
                                    Ext.String.format2(_('Customstate {state} is {value}'), { state: dss.staticDataModel.userStates[iIndex].displayName, value: valueText }) +
                                '</td></tr></table>';
                            }
                        }
                    }
                    return rowTableTag + '<tr>' +
                        '<td style="color:#ff0000;">' + Ext.String.format2(_('Customstate {state} is {value}'), { state: oAction.stateName, value: oAction.stateValue }) + '</td>' +
                        '</tr></table>';
                }
                if (oAction.addonid === 'system-addon-heating-controller') {
                    var textString;
                    var image;
                    switch (oAction.stateValue) {
                        case 0:
                            textString = 'grid_climate_off';        // 'only if heating is inactive';
                            image = dss.staticDataModel.baseImagePath + 'state/heating_mode_off.png';
                            break;
                        case 1:
                            textString = 'grid_climate_heating';    // 'only if heating is active';
                            image = dss.staticDataModel.baseImagePath + 'state/heating_mode_heating.png';
                            break;
                        case 2:
                            textString = 'grid_climate_cooling';    // 'only if cooling is active';
                            image = dss.staticDataModel.baseImagePath + 'state/heating_mode_cooling.png';
                            break;
                    }
                    if (image) {
                        textString = '<td class="icon-small"><img src="' + image + '"></td><td>' + _(textString) + '</td>';
                    } else {
                        textString = '<td colspan="2">' + _(textString) + '</td>';
                    }
                    return rowTableTag + '<tr>' + textString + '</tr></table>';
                }
                return oAction.name;
            break;
            case 'timeframe':
                var oTimeObjectStart;
                var oTimeObjectEnd;
                oTimeObjectStart = oAction.value.start;
                oTimeObjectEnd = oAction.value.end;
                var pad = function (i) { return (i < 10 ? '0' + i : i); };

                var sStart = '';
                if (oTimeObjectStart.timeBase === 'daily') {
                    var sStart = pad(Math.floor(Math.abs(oTimeObjectStart.offset/3600))) + ':' +
                        pad(Math.floor(Math.abs(oTimeObjectStart.offset/60)%60)) + ' ' + _('o\'clock');
                    if (oTimeObjectStart.random != null) {
                        sStart = '&plusmn;' + sText;
                    }
                } else {
                    var sStart = pad(Math.floor(Math.abs(oTimeObjectStart.offset / 3600))) + ':' +
                        pad(Math.floor(Math.abs(oTimeObjectStart.offset / 60) % 60));

                    switch (oTimeObjectStart.timeBase) {
                        case 'sunset':
                            if (oTimeObjectStart.offset == 0) {
                                sStart = _('at sunset');
                            } else if (oTimeObjectStart.offset < 0) {
                                sStart = Ext.String.format2(_('{time} before sunset'), { time: sText });
                            } else {
                                sStart = Ext.String.format2(_('{time} after sunset'), { time: sText });
                            }
                        break;
                        case 'sunrise':
                            if (oTimeObjectStart.offset == 0)
                                sStart = _('at sunrise');
                            else if (oTimeObjectStart.offset<0)
                                sStart = Ext.String.format2(_('{time} before sunrise'), { time: sText });
                            else
                                sStart = Ext.String.format2(_('{time} after sunrise'), { time: sText });
                        break;
                        case 'dawn':
                            if (oTimeObjectStart.offset == 0)
                                sStart = _('at dawn');
                            else if (oTimeObjectStart.offset<0)
                                sStart = Ext.String.format2(_('{time} before dawn'), { time: sText });
                            else
                                sStart = Ext.String.format2(_('{time} after dawn'), { time: sText });
                        break;
                        case 'dusk':
                            if (oTimeObjectStart.offset == 0)
                                sStart = _('at dusk');
                            else if (oTimeObjectStart.offset<0)
                                sStart = Ext.String.format2(_("{time} before dusk"), { time: sText });
                            else
                                sStart = Ext.String.format2(_("{time} after dusk"), { time: sText });
                         break;
                    }
                    if (oTimeObjectStart.limitBefore != null)  {

                        sStart += Ext.String.format2(_("<br>but not past {hour}:{minute} o'clock"), {hour:pad(Math.floor(Math.abs(oTimeObjectStart.limitBefore/3600))), minute:pad(Math.floor(Math.abs(oTimeObjectStart.limitBefore/60)%60))});
                    }
                    if (oTimeObjectStart.limitAfter != null)  {
                        sStart += Ext.String.format2(_("<br>but not before {hour}:{minute} o'clock"), {hour:pad(Math.floor(Math.abs(oTimeObjectStart.limitAfter/3600))), minute:pad(Math.floor(Math.abs(oTimeObjectStart.limitAfter/60)%60))});
                    }
                    if (oTimeObjectStart.random != null) {
                        sStart = '&plusmn;' + sText;
                    }
                }

                var sEnd = '';
                if (oTimeObjectEnd.timeBase == "daily") {
                    var sEnd=  pad(Math.floor(Math.abs(oTimeObjectEnd.offset/3600))) + ":" +
                                pad(Math.floor(Math.abs(oTimeObjectEnd.offset/60)%60)) + " " + _("o'clock");
                    if (oTimeObjectEnd.random != null) {
                        sEnd = "&plusmn;" + sText;
                    }
                } else {

                    var sEnd=  pad(Math.floor(Math.abs(oTimeObjectEnd.offset/3600))) + ":" +
                                pad(Math.floor(Math.abs(oTimeObjectEnd.offset/60)%60));

                    switch (oTimeObjectEnd.timeBase)
                    {
                        case 'sunset':
                            if (oTimeObjectEnd.offset == 0)
                                sEnd = _('at sunset');
                            else if (oTimeObjectEnd.offset<0)
                                sEnd = Ext.String.format2(_('{time} before sunset'), { time: sText });
                            else
                                sEnd = Ext.String.format2(_('{time} after sunset'), { time: sText });
                        break;
                        case 'sunrise':
                            if (oTimeObjectEnd.offset == 0)
                                sEnd = _('at sunrise');
                            else if (oTimeObjectEnd.offset<0)
                                sEnd = Ext.String.format2(_('{time} before sunrise'), { time: sText });
                            else
                                sEnd = Ext.String.format2(_('{time} after sunrise'), { time: sText });
                        break;
                        case 'dawn':
                            if (oTimeObjectEnd.offset == 0)
                                sEnd = _('at dawn');
                            else if (oTimeObjectEnd.offset<0)
                                sEnd = Ext.String.format2(_('{time} before dawn'), { time: sText });
                            else
                                sEnd = Ext.String.format2(_('{time} after dawn'), { time: sText });
                        break;
                        case 'dusk':
                            if (oTimeObjectEnd.offset == 0)
                                sEnd = _('at dusk');
                            else if (oTimeObjectEnd.offset<0)
                                sEnd = Ext.String.format2(_('{time} before dusk'), { time: sText });
                            else
                                sEnd = Ext.String.format2(_('{time} after dusk'), { time: sText });
                         break;
                    }
                    if (oTimeObjectEnd.limitBefore != null)  {
                        sEnd += Ext.String.format2(_("<br>but not past {hour}:{minute} o'clock"), {hour:pad(Math.floor(Math.abs(oTimeObjectEnd.limitBefore/3600))), minute:pad(Math.floor(Math.abs(oTimeObjectEnd.limitBefore/60)%60))});
                    }
                    if (oTimeObjectEnd.limitAfter != null)  {
                        sEnd += Ext.String.format2(_("<br>but not before {hour}:{minute} o'clock"), {hour:pad(Math.floor(Math.abs(oTimeObjectEnd.limitAfter/3600))), minute:pad(Math.floor(Math.abs(oTimeObjectEnd.limitAfter/60)%60))});
                    }
                    if (oTimeObjectEnd.random != null) {
                        sEnd = '&plusmn;' + sText;
                    }
                }

                return rowTableTag + '<tr><td class="icon-small"><img src="' + dss.staticDataModel.baseImagePath + 'dss/time_condition.png"></td><td style="height:16px;">' +
                    Ext.String.format2(_('only between {startTime} and {endTime}'), {startTime:sStart, endTime:sEnd}) + '</td></tr></table>';
                break;
            case 'zonestate':
                var oZone;
                oZone = oAction.value;

                var sString = rowTableTag + '<tr><td class="icon-small" style="height:16px;">' + dss.staticDataModel.getImageForScene(oZone.zone, oZone.group, oZone.scene) + "</td>";

                sString += '<td>';

                sString += dss.staticDataModel.getNameForScene(oZone.zone, oZone.group, oZone.scene);
                if (oZone.group != 0)
                    sString += ' ' + _('for') + ' ' + dss.staticDataModel.getNameForGroup(oZone.zone, oZone.group);

                if (oZone.zone != 0)
                    sString += ' ' + _('in') + ' ' + dss.staticDataModel.getNameForZone(oZone.zone);
                else
                    if ((oZone.scene < 64) && (oZone.group<64))
                        sString += ' ' + _('in all rooms');

                sString += "</td></tr></table>";
                return sString;
                break;
            case 'state':
                var oArray = oAction.name.split('.');
                if (oArray.length == 3) {
                    if (oArray[0] == 'dev')
                    {
                        var oDevice = dss.staticDataModel.getDevice(oArray[1]);
                        if (oDevice != null) {
                            if ((oDevice.hasStates) && (dss.staticDataModel.getDeviceStates(oArray[1]))) {
                                var oParameter = {
                                        device:oDevice.name,
                                        zone_of_device:dss.staticDataModel.getNameForZone(oDevice.ZoneID)
                                    };
                                var states = dss.staticDataModel.getDeviceStates(oArray[1]);
                                for (var sKey in states) {
                                    if (sKey == oArray[2]) {
                                        oParameter.state_name = states[sKey].title;
                                        for (var sSubkey in states[sKey].options) {
                                            if (sSubkey == oAction.value) {
                                                oParameter.state_value = states[sKey].options[sSubkey];
                                                var sString = Ext.String.format2(_("{#grid_condition_column_vdc_state}{state_value} for {state_name} on {device} in {zone_of_device}"), oParameter);
                                                return rowTableTag + '<tr><td class="icon-small"><img src="' + dss.staticDataModel.getImageForDevice(oDevice) + '"></td><td style="' + sError + '">' + sString + '</td></tr></table>';
                                                fSkip = true;
                                                break;
                                            }
                                        }
                                    }
                                }
                                if (fSkip)
                                    break;
                            } else {
                                var sString = oDevice.name + ' in ';
                                sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID);
                                var typeIndex = parseInt(oArray[2], 10);
                                var type = 255;
                                var oParameter = {
                                        device:oDevice.name,
                                        zone_of_device:dss.staticDataModel.getNameForZone(oDevice.ZoneID)
                                    };
                                var fSkip = false;
                                if (oDevice.specialActions != null)
                                    if (oDevice.specialActions.conditions != null)
                                        if (oDevice.specialActions.conditions.akmEvent != null) {

                                            for (var sKey in oDevice.specialActions.conditions.akmEvent)
                                                if (('' + oDevice.specialActions.conditions.akmEvent[sKey].value) == ('' + oAction.value))
                                                    if (('' + oDevice.specialActions.conditions.akmEvent[sKey].id) == ('' + typeIndex)) {
                                                        oParameter.state_value = oDevice.specialActions.conditions.akmEvent[sKey].name;
                                                        fSkip = true;
                                                    }
                                        }
                                if (!fSkip) {
                                    if (oDevice.binaryInputs != null)
                                        for (var kIndex = 0; kIndex < oDevice.binaryInputs.length; kIndex++)
                                            if (oDevice.binaryInputs[kIndex].index == typeIndex)
                                                type = oDevice.binaryInputs[kIndex].type;
                                    if (type === 15) { // window state - three possible states instead of usual two
                                        oParameter.state_value= dss.staticDataModel.getNameForAKMValue(type, oAction.value);
                                    } else {
                                        oParameter.state_value= dss.staticDataModel.getNameForAKMValue(type, oAction.value == 1);
                                    }
                                }
                                var sString = Ext.String.format2(_("{#grid_condition_column_akm_state}{state_value} on {device} in {zone_of_device}"), oParameter);

                                return rowTableTag + '<tr><td class="icon-small"><img src="' + dss.staticDataModel.getImageForDevice(oDevice) + '"></td><td style="' + sError + '">' + sString + '</td></tr></table>';

                            }
                        }
                    }
                    if (oArray[0] == 'zone') {
                        var sNameOfZone = dss.staticDataModel.getNameForZone(parseInt(oArray[1], 10));
                        if (oArray[2] == 'light') {
                            if (oAction.value == 1) {
                                var sString = Ext.String.format2(_("{#editor_condition_column_zone_state}light in {zone} is on"), {'zone':sNameOfZone});
                                return rowTableTag + '<tr><td class="icon-small"><img src="' + dss.staticDataModel.getImagePathForScene(0, 1, 5) + '"></td><td style="' + sError + '">'+ sString + '</td></tr></table>';
                            } else {
                                var sString = Ext.String.format2(_("{#editor_condition_column_zone_state}light in {zone} is off"), {'zone':sNameOfZone});
                                return rowTableTag + '<tr><td class="icon-small"><img src="' + dss.staticDataModel.getImagePathForScene(0, 1, 0) + '"></td><td style="' + sError + '">' + sString + '</td></tr></table>';
                            }
                            break;
                        } else if (oArray[2] == 'motion') {
                            if (oAction.value == 1) {
                                var sString = Ext.String.format2(_("{#editor_condition_column_zone_state}motion in {zone} is reported"), {'zone':sNameOfZone});
                                return rowTableTag + '<tr><td class="icon-small"><img src="' + dss.staticDataModel.getImagePathForAKMValue(5, true) + '"></td><td style="' + sError + '">'+ sString + '</td></tr></table>';
                            } else {
                                var sString = Ext.String.format2(_("{#editor_condition_column_zone_state}no motion in {zone} is reported"), {'zone':sNameOfZone});
                                return rowTableTag + '<tr><td class="icon-small"><img src="' + dss.staticDataModel.getImagePathForAKMValue(5, false) + '"></td><td style="' + sError + '">' + sString + '</td></tr></table>';
                            }
                            break;
                        } else if (oArray[2] == 'presence') {
                            if (oAction.value == 1) {
                                var sString = Ext.String.format2(_("{#editor_condition_column_zone_state}presence in {zone} is reported"), {'zone':sNameOfZone});
                                return rowTableTag + '<tr><td class="icon-small"><img src="' + dss.staticDataModel.getImagePathForAKMValue(1, true) + '"></td><td style="' + sError + '">'+ sString + '</td></tr></table>';
                            } else {
                                var sString = Ext.String.format2(_("{#editor_condition_column_zone_state}absence in {zone} is reported"), {'zone':sNameOfZone});
                                return rowTableTag + '<tr><td class="icon-small"><img src="' + dss.staticDataModel.getImagePathForAKMValue(1, false) + '"></td><td style="' + sError + '">' + sString + '</td></tr></table>';
                            }
                            break;
                        }
                    }
                } else if (oArray.length === 6) {
                    if (oArray[0] === 'zone' && oArray[2] === 'group' && oArray[4] === 'status') {
                        var zoneId = parseInt(oArray[1], 10);
                        var groupId = parseInt(oArray[3], 10);
                        var groupName = dss.staticDataModel.getNameForGroup(zoneId, groupId);
                        var stateName = oArray[5];
                        var icon = dss.staticDataModel.getImagePathForState(oAction.name, oAction.value);
                        var active = oAction.value === 'active' || oAction.value == '1';
                        if (zoneId === 0) {
                            if (groupId === 64) {
                                // this is global application state - apartment ventilation
                                if (stateName === 'malfunction') {
                                    if (active) {
                                        var sString = Ext.String.format2(_('{#editor_condition_column_zone_state}Malfunction in {group}'), { 'group': groupName });
                                    } else {
                                        var sString = Ext.String.format2(_('{#editor_condition_column_zone_state}no Malfunction in {group}'), { 'group': groupName });
                                    }
                                } else if (stateName === 'service') {
                                    if (active) {
                                        var sString = Ext.String.format2(_('{#editor_condition_column_zone_state}Service needed in {group}'), { 'group': groupName });
                                    } else {
                                        var sString = Ext.String.format2(_('{#editor_condition_column_zone_state}no Service needed in {group}'), { 'group': groupName });
                                    }
                                }
                            }
                        } else {
                            // this is zone state
                            var sNameOfZone = dss.staticDataModel.getNameForZone(zoneId);
                            if (stateName === 'malfunction') {
                                if (active) {
                                    var sString = Ext.String.format2(_('{#editor_condition_column_zone_state}Malfunction detected in {zone} for {group}'), { 'zone': sNameOfZone, 'group': groupName });
                                } else {
                                    var sString = Ext.String.format2(_('{#editor_condition_column_zone_state}no Malfunction in {zone} for {group}'), { 'zone': sNameOfZone, 'group': groupName });
                                }
                            } else if (stateName === 'service') {
                                if (active) {
                                    var sString = Ext.String.format2(_('{#editor_condition_column_zone_state}Service needed in {zone} for {group}'), { 'zone': sNameOfZone, 'group': groupName });
                                } else {
                                    var sString = Ext.String.format2(_('{#editor_condition_column_zone_state}no Service needed in {zone} for {group}'), { 'zone': sNameOfZone, 'group': groupName });
                                }
                            }
                        }
                        return rowTableTag + '<tr><td class="icon-small"><img src="' + icon + '"></td><td style="' + sError + '">' + sString + '</td></tr></table>';
                    }
                }
                if (me.stateGridText) {
                    for (var kIndex = 0; kIndex < me.stateGridText.length; kIndex++) {
                        if (me.stateGridText[kIndex].name == oAction.name)
                            if (me.stateGridText[kIndex].value == oAction.value) {
                                var sIcon = dss.staticDataModel.getImagePathForState(oAction.name, oAction.value);
                                if (sIcon != null)
                                    return rowTableTag + '<tr><td class="icon-small"><img src="' + sIcon + '"></td><td>' + me.stateGridText[kIndex].gridText + '</td><tr></table>';
                                else
                                    return rowTableTag + '<tr><td colspan="2">' + me.stateGridText[kIndex].gridText + '</td><tr></table>';
                            }
                    }
                }

                return Ext.String.format2(_('grid_' + oAction.name + '_' + oAction.value), {
                    sunriseTime:dss.staticDataModel.nowSunrise,
                    sunsetTime:dss.staticDataModel.nowSunset,
                    duskTime:dss.staticDataModel.nowDusk,
                    dawnTime:dss.staticDataModel.nowDawn
                });
                break;
            case 'enabler':
                if (oDataObject != null) {
                    if (oDataObject)
                        return _('entry is enabled');
                    else
                        return _('entry is disabled');
                } else {
                    if (me.values.enabled)
                        return _('entry is enabled');
                    else
                        return _('entry is disabled');
                }
                break;
           case 'weekdays':
               var sString = rowTableTag + '<td class="icon-small"><img src="' + dss.staticDataModel.baseImagePath + 'dss/weekday_condition.png"></td><td><span>';

               var fMo = false;
               var fDi = false;
               var fMi = false;
               var fDo = false;
               var fFr = false;
               var fSa = false;
               var fSo = false;
               var recObject = oAction.value;
               if (recObject) {
                   if (recObject.length == 7)
                       return sString + _("daily") + "</span></td></tr></table>";
                   for ( var iIndex = 0; iIndex < recObject.length; iIndex++) {
                       if (recObject[iIndex] === 'MO')
                           fMo = true;
                       if (recObject[iIndex] === 'TU')
                           fDi = true;
                       if (recObject[iIndex] === 'WE')
                           fMi = true;
                       if (recObject[iIndex] === 'TH')
                           fDo = true;
                       if (recObject[iIndex] === 'FR')
                           fFr = true;
                       if (recObject[iIndex] === 'SA')
                           fSa = true;
                       if (recObject[iIndex] === 'SU')
                           fSo = true;
                   }
                   sString += _('only ') ;
                   if (recObject.length == 2)
                       if (fSa)
                           if (fSo)
                               return sString + _('on weekends') + '</span></td></tr></table>';

                   if (recObject.length == 5)
                       if (fMo)
                           if (fDi)
                               if (fMi)
                                   if (fDo)
                                       if (fFr)
                                           return sString + _('MO-FR') + '</span></td></tr></table>';
                   if (recObject.length == 1) {
                       if (fMo)
                           return sString + _("on monday") + "</span></td></tr></table>";
                       if (fDi)
                           return sString + _("on tuesday") + "</span></td></tr></table>";
                       if (fMi)
                           return sString + _("on wednesday") + "</span></td></tr></table>";
                       if (fDo)
                           return sString + _("on thursday") + "</span></td></tr></table>";
                       if (fFr)
                           return sString + _("on friday") + "</span></td></tr></table>";
                       if (fSa)
                           return sString + _("on saturday") + "</span></td></tr></table>";
                       if (fSo)
                           return sString + _("on sunday") + "</span></td></tr></table>";
                   }
                   if (recObject.length == 0) {
                       return sString + _("no time defined") + "</span></td></tr></table>";
                   }
               }

               if (fMo)
                   sString += '<b>' + _('MO') + '</b> ,';
               else
                   sString += '<span style="color:#aaaaaa;text-decoration:line-through;">' + _('MO') + '</span> ,';
               if (fDi)
                   sString += '<b>' + _('TU') + '</b> ,';
               else
                   sString += '<span style="color:#aaaaaa;text-decoration:line-through;">' + _('TU') + '</span> ,';
               if (fMi)
                   sString += '<b>' + _('WE') + '</b> ,';
               else
                   sString += '<span style="color:#aaaaaa;text-decoration:line-through;">' + _('WE') + '</span> ,';
               if (fDo)
                   sString += '<b>' + _('TH') + '</b> ,';
               else
                   sString += '<span style="color:#aaaaaa;text-decoration:line-through;">' + _('TH') + '</span> ,';
               if (fFr)
                   sString += '<b>' + _('FR') + '</b> ,';
               else
                   sString += '<span style="color:#aaaaaa;text-decoration:line-through;">' + _('FR') + '</span> ,';
               if (fSa)
                   sString += '<b>' + _('SA') + '</b> ,';
               else
                   sString += '<span style="color:#aaaaaa;text-decoration:line-through;">' + _('SA') + '</span> ,';
               if (fSo)
                   sString += '<b>' + _('SU') + '</b>';
               else
                   sString += '<span style="color:#aaaaaa;text-decoration:line-through;">' + _('SU') + '</span>';
               return  sString + "</td></tr></table>";

               break;
        }
        return '';
    },
    setHeight: function (oHeight) {
        this.macPanel.setHeight(oHeight - 30);
        this.iDefaultHeight = oHeight - 57;
        for ( var oKey in this.grids)
            this.grids[oKey].setHeight(this.iDefaultHeight);
        this.callParent(arguments);
    },
    defMacGridRenderer: function (val, metadata, record, colIdx, nochNenParameter, store, view) {    // renders a single item in first column of possible conditions, when adding / editing a condition
        const name = record.get('name');
        if (name === '-') {
            return '<table style="height:16px;width:100%;"><tr style="height:16px;"><td width="100%"><hr></td></tr></table>';
        }

        const icon = record.get('icon');
        const dSID = record.get('dSID');
        const dSUID = record.get('dSUID');
        const groupId = record.get('group');
        const state = record.get('state');
        const stateType = record.get('stateType');
        const type = record.get('type');
        let fHasIcon = icon != null;
        // definition erfordert weiterbutton:
        // ist eine gruppe -> weiterbutton:
        // ist ein device -> weiterbutton:
        let fHasNext = Boolean(record.get('next')) || groupId != null || dSID != null || dSUID != null;
        const fIsCheckable = Boolean(record.get('checkable'));
        if (fIsCheckable) {
            fHasNext = false; // checkboxen niemals weiterbuttons
            fHasIcon = false; // keine Icons nach Checkboxen
        }

        const fIsPassive = Boolean(record.get('passive'));
        if (fIsPassive) {
            fHasNext = false; // spalte disabled -> niemals weiterbutton
            fIsChecked = false; // spalte disabled -> kann nicht gechecked sein
        }

        var sReturn = '';
        var iColspaceCounter = 3;
        if (fHasIcon) {
            iColspaceCounter--;
            sReturn += '<td class="icon-small"><img src="' + icon + '" style="margin-right: 5px;"></td>';
        }
        if (fIsCheckable) {
            iColspaceCounter--;
            if (record.get('checked') == true) {
                sReturn += '<td style="padding-right: 5px;"><div style="width:13px;height:13px;background-image:url('
                        + dss.staticDataModel.baseImagePath + 'dss/checkbox.gif); background-position: 0px -13px;" /></td>';
            } else {
                sReturn += '<td style="padding-right: 5px;"><div style="width:13px;height:13px;background-image:url('
                        + dss.staticDataModel.baseImagePath + 'dss/checkbox.gif);" /></td>';
            }
        }
        if (fHasNext)
            iColspaceCounter--;
        if (fIsPassive) {
            sReturn += '<td style="color:#888888;" colspan="' + iColspaceCounter + '" width="100%">' + record.get('name') + '</td>';
        } else {
            sReturn += '<td colspan="' + iColspaceCounter + '" width="100%">' + record.get('name') + '</td>';
        }
        if (fHasNext) {
            sReturn += '<td>&gt;</td>';
        }
        return '<table style="height:16px;width:100%;"' +
            (dSID ? ' data-dsid="' + dSID + '"' : '') +
            (dSUID ? ' data-dsuid="' + dSUID + '"' : '') +
            (groupId != null && groupId !== '' ? ' data-group-id="' + groupId + '"' : '') +
            (state != null && state !== '' ? ' data-id="' + state + '"' : '') +
            (stateType != null && stateType !== '' ? ' data-value="' + stateType + '"' : '') +
            (type != null && type !== '' ? ' data-type="' + type + '"' : '') +
            (name ? ' data-name="' + joinAndCapitalize(name) + '"' : '') +
            '><tr>' + sReturn + '</tr></table>';
    },
    initGridStore: function () {
        var me = this;
        this.gridStore = Ext.create('Ext.data.Store', {

            fields: [ 'pos', 'condition' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            },
            listeners: {
                datachanged: function (store, eOpts) {
                    var ct = store.getCount();
                    me.lblSelectorAnzeige.setText(Ext.String.format2(_('{count} condition', '{count} conditions', ct), { count: '' + ct }), false);
                },
                clear: function (store, eOpts) {
                    me.lblSelectorAnzeige.setText(_('no condition'), false);
                }
            }
        });
    },

    initStores: function (conditionConfig) {
        if (Ext.data.StoreManager.lookup('conditionConfigurator_userStateStore') == null) {

            Ext.create('Ext.data.Store', {
                storeId: 'conditionConfigurator_userStateStore',
                fields: [ '_id', 'name', 'next', 'icon','passive' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });

            var oUserStates = dss.staticDataModel.getUserStates();
            if (oUserStates) {
                for (var iIndex = 0; iIndex < oUserStates.length; iIndex++) {
                    var Aktionen = [{
                        name: oUserStates[iIndex].displayName,
                        _id: oUserStates[iIndex].name,
                        next: true,
                        icon: null,
                        passive:false
                    }];
                    if (typeof oUserStates[iIndex].value !== 'undefined') {
                        Ext.data.StoreManager.lookup('conditionConfigurator_userStateStore').add(Aktionen);
                    }
                }
            }

        }

        if (Ext.data.StoreManager.lookup('conditionConfigurator_l1Store') == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'conditionConfigurator_l1Store',
                fields: [ 'type', 'name', 'next', 'icon' ,'passive','parameter','distiguishID'],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
        }

        Ext.data.StoreManager.lookup('conditionConfigurator_l1Store').removeAll();

        var Aktionen = [];
        this.stateGridText = [];
        var disID = 0;
        if (conditionConfig && Object.prototype.toString.call(conditionConfig) === '[object Array]') {
            for (var iIndex = 0; iIndex < conditionConfig.length; iIndex++) {
                var oCondObject = conditionConfig[iIndex];
                disID++;
                switch (oCondObject.type) {
                case 'enabler':
                    Aktionen.push({
                        name: (oCondObject.title != null)?oCondObject.title:_('Enabled'),
                        type: 0,
                        next: true,
                        icon: null,
                        passive: false,
                        distiguishID:disID
                    });
                      break;
                case 'weekdays':
                    Aktionen.push({
                        name: (oCondObject.title != null) ? oCondObject.title : _('Weekdays'),
                        type: 1,
                        next: true,
                        icon: null,
                        passive: false,
                        distiguishID:disID
                    });
                      break;
                case 'timeframe':
                     Aktionen.push({
                         name: (oCondObject.title != null) ? oCondObject.title : _('Timeframe'),
                         type: 2,
                         next: true,
                         icon: null,
                         passive: false,
                        distiguishID:disID
                     });
                    break;
                case 'dateframe':
                        Aktionen.push({
                        name: (oCondObject.title != null) ? oCondObject.title : _('Dateframe'),
                        type: 10,
                        next: true,
                        icon: null,
                        passive: false,
                        distiguishID:disID
                    });
                   break;
                case 'zonestate':
                    Aktionen.push({
                        name: (oCondObject.title != null) ? oCondObject.title : _('ZoneState'),
                        type: 3,
                        next: true,
                        icon: null,
                        passive: false,
                        distiguishID:disID
                    });
                      break;
                case 'singleItem':
                    var oParameter = [];
                    if (oCondObject && oCondObject.entry) {
                        for (var jIndex = 0; jIndex < oCondObject.entry.length; jIndex++) {
                            var fShouldAdd = true;
                            if (oCondObject.entry[jIndex].type == 'state') {
                                fShouldAdd = false;
                                if (dss.staticDataModel.StateCache) {
                                    for (var kIndex = 0; kIndex < dss.staticDataModel.StateCache.length; kIndex++) {
                                        if (dss.staticDataModel.StateCache[kIndex].name == oCondObject.entry[jIndex].name)
                                            fShouldAdd = true;
                                        else
                                            if (dss.staticDataModel.userGroups) {
                                                for (var lIndex = 0; lIndex < dss.staticDataModel.userGroups.length; lIndex++) {
                                                    if (dss.staticDataModel.StateCache[kIndex].name == oCondObject.entry[jIndex].name + '.group' + dss.staticDataModel.userGroups[lIndex].id)
                                                        fShouldAdd = true;
                                                }
                                            }
                                    }
                                }
                            }
                            if (fShouldAdd) {
                                oParameter.push({name:oCondObject.entry[jIndex].name, value:oCondObject.entry[jIndex].value, title:oCondObject.entry[jIndex].title});
                                this.stateGridText.push({name:oCondObject.entry[jIndex].name, value:oCondObject.entry[jIndex].value, gridText:oCondObject.entry[jIndex].gridText});
                                if (dss.staticDataModel.StateCache) {
                                    for (var kIndex = 0; kIndex < dss.staticDataModel.StateCache.length; kIndex++) {
                                        for (var lIndex = 0; lIndex < dss.staticDataModel.userGroups.length; lIndex++) {
                                            if (dss.staticDataModel.StateCache[kIndex].name == oCondObject.entry[jIndex].name + '.group' + dss.staticDataModel.userGroups[lIndex].id) {
                                                oParameter.push(
                                                        {name:dss.staticDataModel.StateCache[kIndex].name,
                                                         value:oCondObject.entry[jIndex].value,
                                                         title:oCondObject.entry[jIndex].title + ' ' + _('in') + ' ' + dss.staticDataModel.userGroups[lIndex].name});
                                                this.stateGridText.push(
                                                        {name:dss.staticDataModel.StateCache[kIndex].name,
                                                         value:oCondObject.entry[jIndex].value,
                                                         gridText:oCondObject.entry[jIndex].gridText + ' ' + _('in') + ' ' + dss.staticDataModel.userGroups[lIndex].name});
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    if (oParameter.length>0)
                        Aktionen.push({
                            name: oCondObject.title,
                            type: 4,
                            next: true,
                            icon: null,
                            passive: false,
                            parameter:oParameter,
                            distiguishID:disID
                        });
                      break;
                case 'multiItem':
                    var oParameter = [];
                    if (oCondObject && oCondObject.entry) {
                        var positions = {};
                        for (var jIndex = 0; jIndex < oCondObject.entry.length; jIndex++) {

                            var fShouldAdd = true;
                            if (oCondObject.entry[jIndex].type == 'state') {
                                fShouldAdd = false;
                                if (dss.staticDataModel.StateCache) {
                                    for (var kIndex = 0; kIndex < dss.staticDataModel.StateCache.length; kIndex++) {
                                        if (dss.staticDataModel.StateCache[kIndex].name == oCondObject.entry[jIndex].name)
                                            fShouldAdd = true;
                                        else
                                            if (dss.staticDataModel.userGroups) {
                                                for (var lIndex = 0; lIndex < dss.staticDataModel.userGroups.length; lIndex++) {
                                                    if (dss.staticDataModel.StateCache[kIndex].name == oCondObject.entry[jIndex].name + '.group' + dss.staticDataModel.userGroups[lIndex].id)
                                                        fShouldAdd = true;
                                                }
                                            }
                                    }
                                }
                            }

                            if (fShouldAdd) {
                                var position;
                                if (positions[oCondObject.entry[jIndex].name] === undefined) {
                                    var newLength = oParameter.push({name:oCondObject.entry[jIndex].name, value:oCondObject.entry[jIndex].value, title:oCondObject.entry[jIndex].title, group:oCondObject.entry[jIndex].group});
                                    position = newLength - 1;
                                    positions[oCondObject.entry[jIndex].name] = position;
                                } else {
                                    position = positions[oCondObject.entry[jIndex].name] + 1;
                                    oParameter[position] = {name:oCondObject.entry[jIndex].name, value:oCondObject.entry[jIndex].value, title:oCondObject.entry[jIndex].title, group:oCondObject.entry[jIndex].group};
                                }
                                this.stateGridText.push({name:oCondObject.entry[jIndex].name, value:oCondObject.entry[jIndex].value, gridText:oCondObject.entry[jIndex].gridText});
                                if (dss.staticDataModel.StateCache && dss.staticDataModel.userGroups) {
                                    var offset = 1;
                                    for (var kIndex = 0; kIndex < dss.staticDataModel.StateCache.length; kIndex++) {
                                        for (var lIndex = 0; lIndex < dss.staticDataModel.userGroups.length; lIndex++) {
                                            if (dss.staticDataModel.StateCache[kIndex].name == oCondObject.entry[jIndex].name + '.group' + dss.staticDataModel.userGroups[lIndex].id) {
                                                oParameter[position + 2 * offset] =
                                                        {name:dss.staticDataModel.StateCache[kIndex].name,
                                                         value:oCondObject.entry[jIndex].value,
                                                         title:oCondObject.entry[jIndex].title + ' ' + _('in') + ' ' + dss.staticDataModel.userGroups[lIndex].name};
                                                this.stateGridText.push(
                                                        {name:dss.staticDataModel.StateCache[kIndex].name,
                                                         value:oCondObject.entry[jIndex].value,
                                                         gridText:oCondObject.entry[jIndex].gridText + ' ' + _('in') + ' ' + dss.staticDataModel.userGroups[lIndex].name});
                                                offset++;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    if (oParameter.length>0)
                        Aktionen.push({
                            name: oCondObject.title,
                            type: 5,
                            next: true,
                            icon: null,
                            passive: false,
                            parameter:oParameter,
                            distiguishID:-1*disID
                        });
                      break;
                case 'plainStates':
                    var oParameter = [];
                    if (oCondObject && oCondObject.entry) {
                        for (var jIndex = 0; jIndex < oCondObject.entry.length; jIndex++) {

                            var fShouldAdd = true;
                            if (oCondObject.entry[jIndex].type == 'state') {
                                fShouldAdd = false;
                                if (dss.staticDataModel.StateCache) {
                                    for (var kIndex = 0; kIndex < dss.staticDataModel.StateCache.length; kIndex++) {
                                        if (dss.staticDataModel.StateCache[kIndex].name == oCondObject.entry[jIndex].name)
                                            fShouldAdd = true;
                                        else
                                            if (dss.staticDataModel.userGroups) {
                                                for (var lIndex = 0; lIndex < dss.staticDataModel.userGroups.length; lIndex++) {
                                                    if (dss.staticDataModel.StateCache[kIndex].name == oCondObject.entry[jIndex].name + '.group' + dss.staticDataModel.userGroups[lIndex].id)
                                                        fShouldAdd = true;
                                                }
                                            }
                                    }
                                }
                            }

                            if (fShouldAdd) {
                                oParameter.push({name:oCondObject.entry[jIndex].name, value:oCondObject.entry[jIndex].value, title:oCondObject.entry[jIndex].title, group:oCondObject.entry[jIndex].group});
                                this.stateGridText.push({name:oCondObject.entry[jIndex].name, value:oCondObject.entry[jIndex].value, gridText:oCondObject.entry[jIndex].gridText});
                                if (dss.staticDataModel.StateCache && dss.staticDataModel.userGroups) {
                                    for (var kIndex = 0; kIndex < dss.staticDataModel.StateCache.length; kIndex++) {
                                        for (var lIndex = 0; lIndex < dss.staticDataModel.userGroups.length; lIndex++) {
                                            if (dss.staticDataModel.StateCache[kIndex].name == oCondObject.entry[jIndex].name + '.group' + dss.staticDataModel.userGroups[lIndex].id) {
                                                oParameter.push(
                                                        {name:dss.staticDataModel.StateCache[kIndex].name,
                                                         value:oCondObject.entry[jIndex].value,
                                                         title:oCondObject.entry[jIndex].title + ' ' + _('in') + ' ' + dss.staticDataModel.userGroups[lIndex].name});
                                                this.stateGridText.push(
                                                        {name:dss.staticDataModel.StateCache[kIndex].name,
                                                         value:oCondObject.entry[jIndex].value,
                                                         gridText:oCondObject.entry[jIndex].gridText + ' ' + _('in') + ' ' + dss.staticDataModel.userGroups[lIndex].name});
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    Aktionen.push({
                        name: oCondObject.title,
                        type: 8,
                        next: true,
                        icon: null,
                        passive: false,
                        parameter:oParameter,
                        distiguishID:-1*disID
                    });
                      break;
                case 'userDefinedState':
                    Aktionen.push({
                        name: (oCondObject.title != null) ? oCondObject.title : _('user state'),
                        type: 7,
                        next: true,
                        icon: null,
                        passive:(dss.staticDataModel.getUserStates().length == 0),
                        distiguishID:disID
                    });
                      break;
                case 'vdcDeviceState':
                    var fShowDeviceState = false;
                    if (dss.staticDataModel.deviceCache) {
                        for (var jIndex = 0; jIndex < dss.staticDataModel.deviceCache.length; jIndex++) {
                            if (dss.staticDataModel.deviceCache[jIndex].hasStates)
                                if (dss.staticDataModel.deviceCache[jIndex].present)
                                    fShowDeviceState = true;
                        }
                    }
                    if (fShowDeviceState) {
                        Aktionen.push({
                            name: (oCondObject.title != null) ? oCondObject.title : _('device states'),
                            type: 11,
                            next: true,
                            icon: null,
                            passive: false,
                            distiguishID:disID
                        });
                    }
                    break;
                case 'globalApplication':
                    if (dss.staticDataModel.globalApplicationsAvaible)
                        Aktionen.push({
                            name: (oCondObject.title != null) ? oCondObject.title : _('Enabled'),
                            type: 12,
                            next: true,
                            icon: null,
                            passive: false,
                            distiguishID:disID
                        });
                    break;

                case 'deviceState':
                    var oStateDevices = [];
                    var fPassive = false;
                    var akmDeviceArray = dss.staticDataModel.getAKMDeviceArray();
                    if (akmDeviceArray) {
                        for (var jIndex = 0; jIndex < akmDeviceArray.length; jIndex++) {
                            var fShouldAdd = true;
                            if (oCondObject.filterFunc != null)
                                if (!oCondObject.filterFunc(akmDeviceArray[jIndex]))
                                        fShouldAdd = false;

                            if ((akmDeviceArray[jIndex].present != true)&&(!this.fIgnorePresence))
                                fShouldAdd = false;


                            if (fShouldAdd)
                                oStateDevices.push(akmDeviceArray[jIndex]);

                        }
                    }

                    if (dss.staticDataModel.deviceCache) {
                        for (var jIndex = 0; jIndex < dss.staticDataModel.deviceCache.length; jIndex++) {
                            if (dss.staticDataModel.deviceCache[jIndex].specialActions != null)
                                if (dss.staticDataModel.deviceCache[jIndex].specialActions.conditions != null)
                                    if (dss.staticDataModel.deviceCache[jIndex].specialActions.conditions.akmEvent != null) {
                                        var fDataExisting = false;
                                        for (var kIndex = 0; kIndex < oStateDevices.length; kIndex++) {
                                            if (oStateDevices[kIndex].dSUID == dss.staticDataModel.deviceCache[jIndex].dSUID) {
                                                fDataExisting = true;
                                            }
                                        }
                                        if (!fDataExisting)
                                            if ((dss.staticDataModel.deviceCache[jIndex].present != false)||(this.fIgnorePresence))
                                                oStateDevices.push(dss.staticDataModel.deviceCache[jIndex]);
                                    }
                        }
                    }

                    if (Ext.data.StoreManager.lookup('conditionConfigurator_zoneAKMStore') == null) {
                        Ext.create('Ext.data.Store', {
                            storeId: 'conditionConfigurator_zoneAKMStore',
                            fields: [ 'ZoneID', 'name' ],
                            proxy: {
                                type: 'memory',
                                reader: {
                                    type: 'json',
                                    root: 'items'
                                }
                            }
                        });
                        var oZone = [];
                        for (var jIndex = 0; jIndex < oStateDevices.length; jIndex++)
                            if (oZone.indexOf(oStateDevices[jIndex].ZoneID)<0) {
                                oZone.push(oStateDevices[jIndex].ZoneID);
                            }
                        for (var jIndex = 0; jIndex < oZone.length; jIndex++)
                            Ext.data.StoreManager.lookup('conditionConfigurator_zoneAKMStore').add({
                                ZoneID:oZone[jIndex],
                                name:dss.staticDataModel.getNameForZone(oZone[jIndex])
                            });
                    }

                    if (Ext.data.StoreManager.lookup('conditionConfigurator_deviceAKMStore') == null) {
                        Ext.create('Ext.data.Store', {
                            storeId: 'conditionConfigurator_deviceAKMStore',
                            fields: [ 'dSID', 'dSUID', 'name', 'icon', 'present' ],
                            proxy: {
                                type: 'memory',
                                reader: {
                                    type: 'json',
                                    root: 'items'
                                }
                            }
                        });
                        for (var jIndex = 0; jIndex < oStateDevices.length; jIndex++)
                        {
                            Ext.data.StoreManager.lookup('conditionConfigurator_deviceAKMStore').add(oStateDevices[jIndex] );
                        }
                    }


                    if (oStateDevices.length == 0)
                        fPassive = true;
                    Aktionen.push({
                        name: oCondObject.title,
                        type: 6,
                        next: true,
                        icon: null,
                        passive:fPassive,
                        parameter:oStateDevices,
                        distiguishID:-1*disID
                    });
                      break;
                case 'climateState':
                    // check if apartment_controller state exists - if not, the item should be non-selectable
                    var apartmentControllerStateExists = dss.ajaxSyncRequest('/json/property/query2', {
                        'query': '/usr/addon-states/system-addon-heating-controller/apartment_controller(state)'
                    });
                    try {
                        apartmentControllerStateExists = JSON.parse(apartmentControllerStateExists);
                        apartmentControllerStateExists = apartmentControllerStateExists && apartmentControllerStateExists.ok &&
                            apartmentControllerStateExists.result && Object.keys(apartmentControllerStateExists.result).length;
                    } catch (e) {
                        devLog && devLog(e);
                        apartmentControllerStateExists = false;
                    }
                    Aktionen.push({
                        name: (oCondObject.title != null) ? oCondObject.title : _('Heating Modes'),
                        type: 13,
                        next: true,
                        icon: null,
                        passive: !apartmentControllerStateExists,
                        distiguishID: disID
                    });
                    break;
                }
            }
        }

        Ext.data.StoreManager.lookup('conditionConfigurator_l1Store').add(Aktionen);

        if (Ext.data.StoreManager.lookup('conditionConfigurator_zoneStore') == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'conditionConfigurator_zoneStore',
                fields: [ 'ZoneID', 'name' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            Ext.data.StoreManager.lookup('conditionConfigurator_zoneStore').add(dss.staticDataModel.getZoneArray());
        }

        if (Ext.data.StoreManager.lookup('conditionConfigurator_climateStore') == null) {
            var climateStore = Ext.create('Ext.data.Store', {
                storeId: 'conditionConfigurator_climateStore',
                fields: [ 'name', 'icon', 'next', 'active' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });

            var oClimateSystemCapabilities = [];
            var climateSettings = dss.staticDataModel.getClimateSettings();
            if (climateSettings != null && climateSettings.capabilities != null) {
                oClimateSystemCapabilities.push({
                    name: 'off',    // this goes into property tree (should be untranslated)
                    // value: 0,       // this goes into property tree; doesn't work for some reason, needs to be re-written
                    icon: dss.staticDataModel.baseImagePath + 'state/heating_mode_off.png',
                    next: false,
                    active: true
                });
                oClimateSystemCapabilities.push({
                    name: 'heating',
                    // value: 1,
                    icon: dss.staticDataModel.baseImagePath + 'state/heating_mode_heating.png',
                    next: false,
                    active: true    // check if the system have this capability
                });

                if (climateSettings.capabilities.coolingSupported == true) {
                    oClimateSystemCapabilities.push({
                        name: 'cooling',
                        // value: 2,
                        icon: dss.staticDataModel.baseImagePath + 'state/heating_mode_cooling.png',
                        next: false,
                        active: true    // check if the system have this capability
                    });
                }
            }
            for (var i = 0, len = oClimateSystemCapabilities.length; i < len; i++) {
                climateStore.add(oClimateSystemCapabilities[i]);
            }
        }
    },

    getDeviceAKMStore: function (ZoneID) {
        if (Ext.data.StoreManager.lookup('conditionConfigurator_deviceAKMStore_' + ZoneID) == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'conditionConfigurator_deviceAKMStore_' + ZoneID,
                fields: [ 'dSID', 'dSUID', 'name', 'icon', 'present' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            Ext.data.StoreManager.lookup('conditionConfigurator_deviceAKMStore').each(function (record) {
                var oNewRecord = record.copy();
                var oDevice = dss.staticDataModel.getDevice(record.get('dSUID'));
                if (oNewRecord.get('present')||this.fIgnorePresence)
                    if ((ZoneID == 0) || (ZoneID == oDevice.ZoneID))
                        Ext.data.StoreManager.lookup('conditionConfigurator_deviceAKMStore_' + ZoneID).add(oNewRecord);
            });
        }
        return Ext.data.StoreManager.lookup('conditionConfigurator_deviceAKMStore_' + ZoneID);
    },
    getGroupStore: function (ZoneID) {
        if (Ext.data.StoreManager.lookup('conditionConfigurator_groupStore_' + ZoneID) == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'conditionConfigurator_groupStore_' + ZoneID,
                fields: [ 'group', 'name', 'icon', 'reachable' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            var oGroup = dss.staticDataModel.getGroupForZone(ZoneID);
            Ext.data.StoreManager.lookup('conditionConfigurator_groupStore_' + ZoneID).add(oGroup);
        }
        return Ext.data.StoreManager.lookup('conditionConfigurator_groupStore_' + ZoneID);
    },
    getSceneStore: function (ZoneID, groupID) {
        if (Ext.data.StoreManager.lookup('conditionConfigurator_szeneStore_' + ZoneID + '_' + groupID) == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'conditionConfigurator_szeneStore_' + ZoneID + '_' + groupID,
                fields: [ '_id', 'name', 'reachable', 'icon', 'outputMode' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            var oScenes = dss.staticDataModel.getScenes(ZoneID, groupID);
            if (oScenes) {
                for ( var iIndex = 0; iIndex < oScenes.length; iIndex++) {
                    var oObject = {};
                    oObject.id = oScenes[iIndex].id;
                    oObject._id = oScenes[iIndex].id;
                    oObject.reachable = oScenes[iIndex].reachable;
                    oObject.icon = oScenes[iIndex].icon;
                    if (oScenes[iIndex].name == oScenes[iIndex].defaultName)
                        oObject.name = oScenes[iIndex].name;
                    else
                        oObject.name = oScenes[iIndex].defaultName + ' - ' + oScenes[iIndex].name;
                    Ext.data.StoreManager.lookup('conditionConfigurator_szeneStore_' + ZoneID + '_' + groupID).add(oObject);
                }
            }
        }
        return Ext.data.StoreManager.lookup('conditionConfigurator_szeneStore_' + ZoneID + '_' + groupID);

    },
    constructGridLvlDeviceAKMValue: function (dSUID, distiguishID) {
        var oStore = Ext.create('Ext.data.Store', {
            fields: [ 'name','_id','tooltip','icon','index' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        var fSkip = false;
        var oDevice = dss.staticDataModel.getDevice(dSUID);
        if (oDevice.specialActions != null)
            if (oDevice.specialActions.conditions != null)
                if (oDevice.specialActions.conditions.akmEvent != null) {
                    for (var sKey in oDevice.specialActions.conditions.akmEvent) {
                        var oItem = {
                                'name': oDevice.specialActions.conditions.akmEvent[sKey].name,
                                '_id': oDevice.specialActions.conditions.akmEvent[sKey].value,
                                'tooltip':'',
                                'icon': null,
                                'index': oDevice.specialActions.conditions.akmEvent[sKey].id};
                        oStore.add(oItem);
                        fSkip = true;
                    }
                }
        if (!fSkip) {
            var bIsWindowHandle = dss.staticDataModel.IsWindowHandle(oDevice);
            if (oDevice.binaryInputs) {
                for (var iIndex = 0; iIndex < oDevice.binaryInputs.length; iIndex++) {
                    if (!bIsWindowHandle) {
                        var oItem = {
                                'name': dss.staticDataModel.getNameForAKMValue(oDevice.binaryInputs[iIndex].type, false),
                                '_id':2,
                                'tooltip': dss.staticDataModel.getTooltipForState(oDevice.binaryInputs[iIndex].type, false),
                                'icon': dss.staticDataModel.getImagePathForAKMValue(oDevice.binaryInputs[iIndex].type, false),
                                'index': oDevice.binaryInputs[iIndex].index};
                        oStore.add(oItem);
                        oItem = {
                                'name': dss.staticDataModel.getNameForAKMValue(oDevice.binaryInputs[iIndex].type, true),
                                '_id':1,
                                'tooltip': dss.staticDataModel.getTooltipForState(oDevice.binaryInputs[iIndex].type, true),
                                'icon': dss.staticDataModel.getImagePathForAKMValue(oDevice.binaryInputs[iIndex].type, true),
                                'index': oDevice.binaryInputs[iIndex].index};
                        oStore.add(oItem);
                    } else {
                        if (oDevice.binaryInputs[iIndex].type == 15)  { // BinaryInputIDWindowTilt
                            var oItem = {
                                    'name': dss.staticDataModel.getNameForAKMValue(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Closed),
                                    '_id': dss.whState.WH_Closed,
                                    'tooltip': dss.staticDataModel.getTooltipForState(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Closed),
                                    'icon': dss.staticDataModel.getImagePathForAKMValue(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Closed),
                                    'index': oDevice.binaryInputs[iIndex].index};
                            oStore.add(oItem);
                            oItem = {
                                    'name': dss.staticDataModel.getNameForAKMValue(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Open),
                                    '_id': dss.whState.WH_Open,
                                    'tooltip': dss.staticDataModel.getTooltipForState(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Open),
                                    'icon': dss.staticDataModel.getImagePathForAKMValue(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Open),
                                    'index': oDevice.binaryInputs[iIndex].index};
                            oStore.add(oItem);
                            oItem = {
                                    'name': dss.staticDataModel.getNameForAKMValue(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Tilted),
                                    '_id': dss.whState.WH_Tilted,
                                    'tooltip': dss.staticDataModel.getTooltipForState(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Tilted),
                                    'icon': dss.staticDataModel.getImagePathForAKMValue(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Tilted),
                                    'index': oDevice.binaryInputs[iIndex].index};
                            oStore.add(oItem);
                        }
                    }
                }
            }
        }
        this.grids.lvlDeviceAKMValue = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: 174 + 150,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('type'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlDeviceAKMValue, this.conditionSelectorName + 'GridLvlBinarySensorEventDeviceValue');
        this.grids.lvlDeviceAKMValue.parent = this;
        this.grids.lvlDeviceAKMValue.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            var oCondiObject = { type: 'state' };
            oCondiObject.value = record.get('_id');
            oCondiObject.name = 'dev.' + dSUID + '.' + record.get('index');
            this.parent.setSelectedSubItem(oCondiObject, distiguishID);
        });

        this.grids.lvlDeviceAKMValue.on('itemdblclick', function (grid, record) {
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });
        this.macPanel.add(this.grids.lvlDeviceAKMValue);
    },
    constructGridLvlDeviceAKMs: function (zoneID, distiguishID) {
        var me = this;
        var oDeviceControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'dSID', 'dSUID', {name:'name', sortType:'asUCText'}, 'icon', 'present' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oDeviceStore = this.getDeviceAKMStore(zoneID);
        oDeviceStore.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('present')||me.fIgnorePresence)
                oDeviceControlStore.add(oNewRecord);
        });
        oDeviceControlStore.sort('name');
        this.grids.lvlAKMDevice = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oDeviceControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('device'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlAKMDevice, this.conditionSelectorName + 'GridLvlBinarySensorEventDevice');
        this.grids.lvlAKMDevice.parent = this;
        this.grids.lvlAKMDevice.on('itemclick', function (grid, record) {
            this.parent.macPanel.remove(this.parent.grids.lvlDeviceAKMType);
            this.parent.macPanel.remove(this.parent.grids.lvlDeviceAKMValue);
            var dSID = record.get('dSID');
            var dSUID = record.get('dSUID');
            this.parent.constructGridLvlDeviceAKMValue(dSUID, distiguishID);
        });
        this.macPanel.add(this.grids.lvlAKMDevice);
    },
    constructGridLvlAKM: function (distiguishID) {
        var oZoneControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'ZoneID',  { name: 'name', sortType: 'asUCText' } ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oZoneStore = Ext.data.StoreManager.lookup('conditionConfigurator_zoneAKMStore');
        oZoneStore.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('ZoneID') != 0)
                oZoneControlStore.add(oNewRecord);
        });
        oZoneControlStore.sort('name');
        this.grids.lvlAKMRoom = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oZoneControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {
                    return '<table style="height:16px;width:100%" data-zone-id="' + record.get('ZoneID') + '"> \
                        <tr><td width="100%">' + record.get('name') + '</td><td>&gt;</td></tr> \
                    </table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlAKMRoom, this.conditionSelectorName + 'GridLvlBinarySensorEvent');

        this.grids.lvlAKMRoom.parent = this;
        this.grids.lvlAKMRoom.on('itemclick', function (grid, record) {
            this.parent.macPanel.remove(this.parent.grids.lvlDeviceScene);
            this.parent.macPanel.remove(this.parent.grids.lvlGroup);
            this.parent.macPanel.remove(this.parent.grids.lvlScene);
            this.parent.macPanel.remove(this.parent.grids.lvlDevice);
            this.parent.macPanel.remove(this.parent.grids.lvlDeviceAction);
            this.parent.macPanel.remove(this.parent.grids.lvlSensorDevice);
            this.parent.macPanel.remove(this.parent.grids.lvlSensorMessage);
            this.parent.macPanel.remove(this.parent.grids.lvlDeviceAKMValue);
            this.parent.macPanel.remove(this.parent.grids.lvlDeviceAKMType);
            this.parent.macPanel.remove(this.parent.grids.lvlAKMDevice);
            var zoneID = record.get('ZoneID');
            if (zoneID == -1)
                zoneID = 0;
            this.parent.constructGridLvlDeviceAKMs(zoneID, distiguishID);
        });
        this.macPanel.add(this.grids.lvlAKMRoom);
    },
    constructEnabler: function (distiguishID) {
        var me = this;
        this.grids.lvlEnableCheckbox = Ext.create('Ext.form.RadioGroup', {
            fieldLabel: _('Enabled (choose one to set the condition)?'),
            items: [ {
                inputValue: false,
                boxLabel: _('disabled - that condition is allways disabled'),
                name: 'autoMode'
            }, {
                inputValue: true,
                boxLabel: _('enabled - that condition is enabled'),
                name: 'autoMode'
            } ],
            height: 60,
            width: 610,
            margin: "10 0 0 10",
            columns: 1,
            labelAlign: 'top',
            style: {
                color: "#005138"
            },
            listeners: {
                change: function (oThis, value) {
                    if (typeof(value.autoMode) == 'boolean') {
                        me.oActualValue = null;
                        var oCondiObject = {type:'enabler', value:value.autoMode};
                        me.setSelectedSubItem(oCondiObject, distiguishID);
                    }
                }
            }
        });

        this.grids.lvlEnable = Ext.create('Ext.panel.Panel', {
            height: this.iDefaultHeight,
            width: 624,
            autoScroll: true,
            items: [ this.grids.lvlEnableCheckbox ]
        });
        addDataIdToElement(this.grids.lvlEnable, this.conditionSelectorName + 'GridLvlEnable');

        this.grids.lvlEnable.parent = this;
        this.grids.lvlEnableCheckbox.parent = this;
        this.macPanel.add(this.grids.lvlEnable);
        if (this.values.enabled != null)
            this.grids.lvlEnableCheckbox.setValue({ autoMode: this.values.enabled });

    },
    constructWeekdays: function (distiguishID) {
        var me = this;
        this.grids.lvlWeekdaySelector = Ext.create('DSS.component.WeekdaySelection', {
            id: this.conditionSelectorName + 'WeekdaySelector',
            name: 'lvlWeekdaySelector',
            width:600,
            label: _('Only active on'),
            fOnlyWeekdays: true,
            listeners: {
                change:function (value) {
                    if (value.timeArray.length) {
                        me.oActualValue = null;
                        var oCondiObject = { type: 'weekdays' };
                        oCondiObject.value = value.timeArray;
                        me.setSelectedSubItem(oCondiObject, distiguishID);
                    }
                }
            }
        });
        this.grids.lvlWeekday = Ext.create('Ext.panel.Panel', {
            height: this.iDefaultHeight,
            width: 624,
            autoScroll: true,
            items: [this.grids.lvlWeekdaySelector]
        });
        addDataIdToElement(this.grids.lvlWeekday, this.conditionSelectorName + 'GridLvlWeekday');

        this.grids.lvlWeekday.parent = this;
        this.macPanel.add(this.grids.lvlWeekday);
    },
    constructDateframe: function (distiguishID, fromDay, fromMonth, toDay, toMonth) {
        var me = this;

        var fnChange = function (value) {
            if (me.grids.lvlDateframeFromMonth == null ||
                me.grids.lvlDateframeFromDay == null ||
                me.grids.lvlDateframeToMonth == null ||
                me.grids.lvlDateframeToDay == null ||
                !me.grids.lvlDateframeFromDay.isValid() ||
                !me.grids.lvlDateframeToDay.isValid()) {
                return;
            }
            me.oActualValue = null;
            var oCondiObject = { type: 'date' };
            var pad = function (i) { return (i < 10 ? '0' + i : i); };
            var fromDay = me.grids.lvlDateframeFromDay.getValue();
            var fromMonth = me.grids.lvlDateframeFromMonth.getActiveItem().getItemId().slice(0, 2);
            var toDay = me.grids.lvlDateframeToDay.getValue();
            var toMonth =  me.grids.lvlDateframeToMonth.getActiveItem().getItemId().slice(0, 2);
            var iStartVal = parseInt('' + fromMonth, 10) * 100 + fromDay;
            var iEndVal = parseInt('' + toMonth, 10) * 100 + toDay;

            var sStartDate = '2015' + fromMonth + pad(fromDay);
            var sEndDate = '2015' + toMonth + pad(toDay);
            if (iStartVal > iEndVal) {
                sStartDate = '2014' + fromMonth + pad(fromDay);
            }
            sStartDate += 'T000000';
            sEndDate += 'T235959';
            oCondiObject.value = {
                start: sStartDate,
                end: sEndDate,
                rrule: 'FREQ=YEARLY'};
            me.setSelectedSubItem(oCondiObject, distiguishID);
        };

        function createMonthSelectItems (month) {   // do NOT delete 'm', Ext.button.Cycle freaks out when itemIds are numbers-as-string
            var months = [ {
                itemId: '01m',
                text: _('january')
            }, {
                itemId: '02m',
                text: _('february')
            }, {
                itemId: '03m',
                text:  _('march')
            }, {
                itemId: '04m',
                text: _('april')
            }, {
                itemId: '05m',
                text: _('may'),
            }, {
                itemId: '06m',
                text:  _('june')
            }, {
                itemId: '07m',
                text: _('july')
            }, {
                itemId: '08m',
                text: _('august')
            }, {
                itemId: '09m',
                text:  _('september')
            }, {
                itemId: '10m',
                text: _('october')
            }, {
                itemId: '11m',
                text: _('november')
            }, {
                itemId: '12m',
                text:  _('december')
            } ];
            if (month) {
                months[parseInt(month, 10) - 1].checked = true;
            }
            return months;
        }

        function getMonthMaxValue (month) {
            switch (month) {
                case '01':
                case '03':
                case '05':
                case '07':
                case '08':
                case '10':
                case '12':
                     return 31;
                case '02':
                    return 28;  // hmm, what about leap years?
                default:
                    return 30;
            }
        }

        this.grids.lvlDateframeFromDay = Ext.create('Ext.form.field.Number', {
            allowBlank: false,
            blankText: _('This field is required'),
            invalidText: _('The value in this field is invalid'),
            width: 150,
            margin: '10 5 0 10',
            step: 1,
            value: fromDay ? fromDay : 1,
            minValue: 1,
            maxValue: 30,
            allowDecimals: false,
            autoStripChars: true,
            fieldLabel: _('only from')
        });
        this.grids.lvlDateframeFromMonth = Ext.create('Ext.button.Cycle', {
            margin: '10 5 0 0',
            menu: {
                items: createMonthSelectItems(fromMonth)
            },
            listeners: {
                change: function (btn, item) {
                    var maxValue = getMonthMaxValue(item.getItemId().slice(0, 2));
                    me.grids.lvlDateframeFromDay.setMaxValue(maxValue);
                    if (me.grids.lvlDateframeFromDay.getValue()>maxValue)
                        me.grids.lvlDateframeFromDay.setValue(maxValue);
                    setTimeout(fnChange, 0);
                }
            },
            showText: true,
            width: 100
        });
        this.grids.lvlDateframeToDay = Ext.create('Ext.form.field.Number', {
            allowBlank: false,
            blankText: _('This field is required'),
            invalidText: _('The value in this field is invalid'),
            width: 150,
            margin: '10 5 0 10',
            step: 1,
            value: toDay ? toDay : 1,
            minValue: 1,
            maxValue: 30,
            allowDecimals: false,
            autoStripChars: true,
            fieldLabel: _('until')
        });
        this.grids.lvlDateframeToMonth = Ext.create('Ext.button.Cycle', {
            margin: '10 5 0 0',
            menu: {
                items: createMonthSelectItems(toMonth)
            },
            listeners: {
                change: function (btn, item) {
                    var maxValue = getMonthMaxValue(item.getItemId().slice(0, 2));
                    me.grids.lvlDateframeToDay.setMaxValue(maxValue);
                    if (me.grids.lvlDateframeToDay.getValue()>maxValue)
                        me.grids.lvlDateframeToDay.setValue(maxValue);
                    setTimeout(fnChange, 0);
                }
            },
            showText: true,
            width: 100
        });


        this.grids.lvlDateframe = Ext.create('Ext.panel.Panel', {
            height: this.iDefaultHeight,
            width: 624,
            autoScroll: true,
            layout: {
                type: 'table',
                // The total column count must be specified here
                columns: 2
            },
            items: [this.grids.lvlDateframeFromDay, this.grids.lvlDateframeFromMonth, this.grids.lvlDateframeToDay, this.grids.lvlDateframeToMonth ]
        });
        addDataIdToElement(this.grids.lvlDateframe, this.conditionSelectorName + 'GridLvlDateframe');

        this.grids.lvlDateframeFromDay.on('change', fnChange);
        this.grids.lvlDateframeToDay.on('change', fnChange);

        this.grids.lvlDateframe.parent = this;
        this.macPanel.add(this.grids.lvlDateframe);
    },

    constructTimeframe: function (distiguishID) {
        var me = this;
        this.grids.lvlTimeFrameMin = Ext.create('DSS.component.TimeReference', {
            name: 'lvlTimeFrameMin',
            label: _('Timeframe starts at'),
            fAlternativeDisplay: true,
            fFeatureEnabled:{
                fStaticEnabled: true,
                fDuskEnabled: false,
                fDawnEnabled: false,
                fSunsetEnabled: false,
                fSunriseEnabled: false
            }
        });
        addDataIdToElement(this.grids.lvlTimeFrameMin, this.conditionSelectorName + 'TimeframeMin');
        this.grids.lvlTimeFrameMax = Ext.create('DSS.component.TimeReference', {
            name: 'lvlTimeFrameMax',
            label: _('Timeframe ends at'),
            fAlternativeDisplay: true,
            fFeatureEnabled:{
                fStaticEnabled: true,
                fDuskEnabled: false,
                fDawnEnabled: false,
                fSunsetEnabled: false,
                fSunriseEnabled: false
            }
        });
        addDataIdToElement(this.grids.lvlTimeFrameMax, this.conditionSelectorName + 'TimeframeMax');

        this.grids.lvlTimeframe = Ext.create('Ext.panel.Panel', {
            height: this.iDefaultHeight,
            width: 624,
            autoScroll: true,
            items: [ this.grids.lvlTimeFrameMin, this.grids.lvlTimeFrameMax ]
        });
        addDataIdToElement(this.grids.lvlTimeframe, this.conditionSelectorName + 'GridLvlTimeframe');

        this.grids.lvlTimeframe.parent = this;
        this.macPanel.add(this.grids.lvlTimeframe);

        this.grids.lvlTimeFrameMin.reset();
        this.grids.lvlTimeFrameMax.reset();

        var fnChange = function (value) {
            if (me.grids.lvlTimeFrameMin.isValid())
                if (me.grids.lvlTimeFrameMax.isValid()) {
                    me.oActualValue = null;
                    var oCondiObject = { type: 'timeframe' };
                    oCondiObject.value = {start:me.grids.lvlTimeFrameMin.getValue(), end:me.grids.lvlTimeFrameMax.getValue()};
                    me.setSelectedSubItem(oCondiObject, distiguishID);
                }
        };

        this.grids.lvlTimeFrameMin.on('change', fnChange);
        this.grids.lvlTimeFrameMax.on('change', fnChange);

    },
    constructSystemState: function (oParams, distiguishID) {

        var oSystemStateControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'value', 'name','title' ,'passive','icon' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        if (oParams && Object.prototype.toString.call(oParams) === '[object Array]') {
            for (var iIndex = 0; iIndex < oParams.length; iIndex++) {
                var oItem = {name:oParams[iIndex].name,
                            value:oParams[iIndex].value,
                            title:oParams[iIndex].title,
                            icon:dss.staticDataModel.getImagePathForState(oParams[iIndex].name, oParams[iIndex].value),
                            passive: false};
                oSystemStateControlStore.add(oItem);
            }
        }

        const systemStateTypes = {
            1: 'SolarAltitude',
            9: 'Presence',
            10: 'PresenceSimulation'
        };
        const me = this;
        this.grids.lvlSystemState = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: 624,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oSystemStateControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('State'),
                dataIndex: 'title',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {
                    var iCol = 2;
                    const name = capitalizeFirstLetter(record.get('name'));
                    const value = record.get('value');
                    var sString = '<table data-type="' + systemStateTypes[distiguishID] + '" data-name="' + name + '" data-value="' + value + '"> \
                        <tr style="height:16px;">';
                    if (record.get('icon') != null) {
                        iCol = 1;
                        sString += '<td class="icon-small"><img src="' + record.get('icon') + '" style="margin-right: 5px;"></td>';
                    }
                    if (record.get('passive')) {
                        sString += '<td colspan="' + iCol + '" width="100%" style="color:#888888;">' + record.get('title') + '</td>';
                    } else {
                        sString += '<td colspan="' + iCol + '" width="100%">' + record.get('title') + '</td>';
                    }

                    return sString + "</tr></table>";
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlSystemState, this.conditionSelectorName + 'GridLvl' + systemStateTypes[distiguishID]);

        this.grids.lvlSystemState.parent = this;
        this.grids.lvlSystemState.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            var oCondiObject = { type: 'state' };
            oCondiObject.value = record.get('value');
            oCondiObject.name = record.get('name');
            this.parent.setSelectedSubItem(oCondiObject, distiguishID);
        });

        this.grids.lvlSystemState.on('itemdblclick', function (grid, record) {
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });

        this.macPanel.add(this.grids.lvlSystemState);
    },

    constructAddonStateValue: function (stateID, distiguishID) {

         var oAddonStateStore = Ext.create('Ext.data.Store', {
             fields: [ 'value', 'name', 'title' ,'passive', 'icon' ],
             proxy: {
                 type: 'memory',
                 reader: {
                     type: 'json',
                     root: 'items'
                 }
             }
         });

         var oState = null;
         if (dss.staticDataModel.userStates) {
             for (var iIndex = 0; iIndex < dss.staticDataModel.userStates.length; iIndex++) {
                 if (dss.staticDataModel.userStates[iIndex].name == stateID) {
                     oState = dss.staticDataModel.userStates[iIndex];
                     break;
                 }
             }
        }
        if (oState != null) {
             var oItem = {
                        name: stateID,
                        value: 'active',
                        title: oState.setName,
                        icon: dss.staticDataModel.baseImagePath +  "state/uds_state_active.png",
                        passive: false};
             oAddonStateStore .add(oItem);
             var oItem = {name: stateID,
                     value: 'inactive',
                     title: oState.resetName,
                     icon: dss.staticDataModel.baseImagePath +  "state/uds_state_inactive.png",
                     passive: false};
            oAddonStateStore .add(oItem);
        }
        this.grids.lvlAddonStateValue = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: 324,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oAddonStateStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('State'),
                dataIndex: 'title',
                renderer: function (val, metadata, record, colIdx, nochNenParameter, store, view) {
                    var fHasIcon = false;
                    if (record.get('icon') != null) {
                        fHasIcon = true;
                    }
                    var fHasNext = false;
                    const name = record.get('name');
                    var sReturn = '<table style="height:16px;width:100%;" data-name="' + joinAndCapitalize(name) +
                        '" data-value="' + record.get('value') + '"><tr>';
                    var iColspaceCounter = 3;
                    if (fHasIcon) {
                        iColspaceCounter--;
                        sReturn += '<td class="icon-small"><img src="' + record.get('icon') + '" style="margin-right: 5px;"></td>';
                    }
                    sReturn += '<td colspan="' + iColspaceCounter + '" width="100%">' + record.get('title') + '</td>';
                    return sReturn + '</tr></table>';

                },
                flex: 1
            } ]
        });
        addDataIdToElement(this.grids.lvlAddonStateValue, this.conditionSelectorName + 'GridLvlUDSValue');

        this.grids.lvlAddonStateValue.parent = this;
        this.grids.lvlAddonStateValue.on('itemclick', function (grid, record) {

            this.parent.oActualValue = null;
            var oCondiObject = {type:'addon-state'};
            oCondiObject.stateName = record.get('name');
            oCondiObject.stateValue = (record.get('value') == 'active')?1:2;

            if (dss.staticDataModel.userStates) {
                for (var iIndex = 0; iIndex < dss.staticDataModel.userStates.length; iIndex++) {
                    if (dss.staticDataModel.userStates[iIndex].name == oCondiObject.stateName) {
                        if (record.get('value') == 'active')
                            oCondiObject.name= dss.staticDataModel.userStates[iIndex].setName;
                        else
                            oCondiObject.name= dss.staticDataModel.userStates[iIndex].resetName;
                    }
                }
            }

            oCondiObject.addonid = 'system-addon-user-defined-states';
            this.parent.setSelectedSubItem(oCondiObject, distiguishID);
        });

        this.grids.lvlAddonStateValue.on('itemdblclick', function (grid, record) {
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });

        this.macPanel.add(this.grids.lvlAddonStateValue);
    },
    constructAddonState: function (distiguishID) {


        var lvlTempUserStateStore = Ext.create('Ext.data.Store', {
            fields: ['_id', 'name', 'next', 'icon','passive'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        var me = this;
        var lvlUserStateStore = Ext.data.StoreManager.lookup('conditionConfigurator_userStateStore');  // a list of condition types (first columns of editor)
        lvlUserStateStore.each(function (record) {
            var oNewRecord = record.copy();
            oNewRecord.set('passive', false);

            me.gridStore.each(function (gridRecord) {
            	var oCondition=gridRecord.get('condition');
            	if (oCondition.type=='addon-state')
            		if (oCondition.addonid=='system-addon-user-defined-states')
            			if (oCondition.stateName==oNewRecord.get('_id'))
            	            oNewRecord.set('passive', true);
            });

            oNewRecord.commit();
            lvlTempUserStateStore.add(oNewRecord);
        });


        this.grids.lvlAddonState = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: 300,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: lvlTempUserStateStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('State'),
                dataIndex: 'name',
                renderer: this.defMacGridRenderer,
                flex: 1
            } ]
        });
        addDataIdToElement(this.grids.lvlAddonState, this.conditionSelectorName + 'GridLvlUDS');

        this.grids.lvlAddonState.parent = this;
        this.grids.lvlAddonState.on('itemclick', function (grid, record) {
            this.parent.macPanel.remove(this.parent.grids.lvlAddonStateValue);

        	var fIsPassive = false;
            if (record.get('passive') != null) {
                if (record.get('passive') == true) {
                    fIsPassive = true;
                }
            }

            if (!fIsPassive) {
            	this.parent.constructAddonStateValue(record.get('_id'), distiguishID);
            }
            this.parent.updatePanelDescription();
        });

        this.macPanel.add(this.grids.lvlAddonState);
    },

    constructClimateAddonState: function (distiguishID) {

        this.grids.lvlClimateAddonState = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: 624, // 300
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: Ext.data.StoreManager.lookup('conditionConfigurator_climateStore'),
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('Heating Modes'),
                dataIndex: 'name',
                // custom renderer instead of this.defMacGridRenderer, to translate names
                renderer: function (val, metadata, record, colIdx, nochNenParameter, store, view) {
                    var icon = record.get('icon');
                    var modeName = record.get('name');
                    // further modification for translation strings of heating and cooling (off is ok)
                    switch (modeName) {
                        case 'heating':
                            modeName = 'heating on';
                            break;
                        case 'cooling':
                            modeName = 'cooling on';
                            break;
                    }
                    var sReturn = '<table style="height:16px;width:100%;" data-name="' + joinAndCapitalize(modeName) + '">' +
                        '             <tr>' +
                        (icon
                            ? '         <td class="icon-small"><img src="' + icon + '" style="margin-right: 5px;"></td>'
                            : '') +
                        '               <td width="100%">' + _(modeName) + '</td>' +
                        '             </tr>' +
                        '          </table>';
                    return sReturn;
                },
                flex: 1
            } ]
        });
        addDataIdToElement(this.grids.lvlClimateAddonState, this.conditionSelectorName + 'GridLvlClimate');

        this.grids.lvlClimateAddonState.parent = this;
        this.grids.lvlClimateAddonState.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            var oCondiObject = { type: 'addon-state' };
            oCondiObject.name = record.get('name');
            oCondiObject.stateName = 'apartment_controller';
            if (oCondiObject.name === 'off') {
                oCondiObject.value = 0;
            }
            if (oCondiObject.name === 'heating') {
                oCondiObject.value = 1;
            }
            if (oCondiObject.name === 'cooling') {
                oCondiObject.value = 2;
            }
            oCondiObject.stateValue = oCondiObject.value;
            oCondiObject.icon = record.get('icon');
            oCondiObject.addonid = 'system-addon-heating-controller';
            this.parent.setSelectedSubItem(oCondiObject, distiguishID);
        });

        this.grids.lvlClimateAddonState.on('itemdblclick', function (grid, record) {
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });

        this.macPanel.add(this.grids.lvlClimateAddonState);
    },

    constructSystemStateMulti: function (oParams, distiguishID, ignoreGroup) {

        var oSystemStateControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'value', 'name','title','group' ,'passive','icon'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        if (oParams && Object.prototype.toString.call(oParams) === '[object Array]') {
            for (var iIndex = 0; iIndex < oParams.length; iIndex++) {
                var oItem = {name:oParams[iIndex].name,
                            value:oParams[iIndex].value,
                            title:oParams[iIndex].title,
                            group:oParams[iIndex].group,
                            icon:dss.staticDataModel.getImagePathForState(oParams[iIndex].name, oParams[iIndex].value),
                            passive: false};
                this.gridStore.each(function (oRecord) {
                    if (oRecord.get('pos') == distiguishID)
                        if (oRecord.get('condition').group == oParams[iIndex].group)
                            if (ignoreGroup != oParams[iIndex].group)
                                oItem.passive = true;
                });
                oSystemStateControlStore.add(oItem);
            }
        }

        const systemStateTypes = {
            '-11': 'Weather',
            '-14': 'Security'
        };
        const me = this;
        this.grids.lvlSystemState = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: 624,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oSystemStateControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('State'),
                dataIndex: 'title',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {
                    var iCol = 2;
                    var sString = '<table data-type="' + systemStateTypes[distiguishID] + '" \
                        data-name="' + joinAndCapitalize(record.get('name')) + '" \
                        data-value="' + record.get('value') + '"> \
                        <tr style="height:16px;">';
                    if (record.get('icon') != null) {
                        iCol = 1;
                        sString += '<td class="icon-small"><img src="' + record.get('icon') + '" style="margin-right: 5px;"></td>';
                    }
                    if (record.get('passive'))
                        sString += '<td colspan="' + iCol + '" width="100%" style="color:#888888;">' + record.get('title') + '</td>';
                    else
                        sString += '<td colspan="' + iCol + '" width="100%">' + record.get('title') + '</td>';

                    return sString + '</tr></table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlSystemState, this.conditionSelectorName + 'GridLvl' + systemStateTypes[distiguishID]);

        this.grids.lvlSystemState.parent = this;
        this.grids.lvlSystemState.on('itemclick', function (grid, record) {
            this.parent.oActualValue = null;
            if (record.get('passive')) {
                grid.getSelectionModel().deselect(record);
                return;
            }
            var oCondiObject = { type: 'state' };
            oCondiObject.value = record.get('value');
            oCondiObject.name = record.get('name');
            oCondiObject.group = record.get('group');
            this.parent.setSelectedSubItem(oCondiObject, distiguishID);
        });

        this.grids.lvlSystemState.on('itemdblclick', function (grid, record) {
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });

        this.macPanel.add(this.grids.lvlSystemState);
    },
    constructZoneStateLvlGroup: function (zoneID, distiguishID) {
        var me = this;

        var oGroupControlStore = Ext.create('Ext.data.Store', {
            fields: [ '_id', 'group', 'name', 'icon', 'reachable', 'state', 'stateType' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var needsSplitter = false;
        var oGroupStore = this.getGroupStore(zoneID);
        oGroupStore.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('reachable')) {
                if (oNewRecord.get('group') != 0) {
                    if (oNewRecord.get('group') != 8) {
                        oGroupControlStore.add(oNewRecord);
                        needsSplitter = true;
                    }
                }
            }
        });
        if (needsSplitter) {
            oGroupControlStore.add({
                group: -1,
                name: '-',
                passive: '-',
                icon: null,
                reachable: true
            });
            needsSplitter = false;
        }
        var oSceneGroup0Store;
        if (zoneID == 0) {
            oSceneGroup0Store = this.getSceneStore(dss.staticDataModel.getZoneArray()[1].ZoneID, 0);
        } else {
            oSceneGroup0Store = this.getSceneStore(zoneID, 0);
        }
        oSceneGroup0Store.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('reachable')) {
                if (oNewRecord.get('group') != 6) {
                    if (oNewRecord.get('group') != 7) {
                        oGroupControlStore.add(oNewRecord);
                        needsSplitter = true;
                    }
                }
            }
        });

        var states = dss.staticDataModel.getStates();
        var idCounter = -2;
        if (states) {
            for (var iIndex = 0; iIndex < states.length; iIndex++) {
                var oState = states[iIndex];
                var oStateArray = oState.name.split('.');
                if (oStateArray[0] === 'zone') {
                    if (oStateArray[1] == '' + zoneID) {
                        if ([ 'light', 'heating' ].indexOf(oStateArray[2]) === -1 && (oStateArray[2] !== 'group' || [ '10', '12' ].indexOf(oStateArray[3]) === -1)) {  // these states are included already in the groups above
                            if (needsSplitter) {
                                oGroupControlStore.add({
                                    group: -1,
                                    name: '-',
                                    passive: '-',
                                    icon: null,
                                    reachable: true
                                });
                                needsSplitter = false;
                            }
                            var sIcon, sText;
                            if (oStateArray[2] === 'motion') {
                                sIcon = dss.staticDataModel.getImagePathForAKMValue(5, true);
                                sText = Ext.String.format2(_('{#editor_trigger_zonestates}motion is active'));
                            } else if (oStateArray[2] === 'presence') {
                                sIcon = dss.staticDataModel.getImagePathForAKMValue(1, true);
                                sText = Ext.String.format2(_('{#editor_trigger_zonestates}presence is active'));
                            } else if (oStateArray[2] === 'group') {
                                var stateName = oStateArray[oStateArray.length - 1];
                                sIcon = dss.staticDataModel.getImagePathForState(oState.name, 'active'),
                                sText = dss.staticDataModel.getNameForState(stateName, 'active');
                            } else {
                                sIcon = dss.staticDataModel.getImagePathForAKMValue(0, true);
                                sText = Ext.String.format2(_('{#editor_condition_zonestates_generic}{state} is active'), { state: oStateArray[2] });
                            }

                            oGroupControlStore.add({
                                id: idCounter,
                                _id: idCounter,
                                state: oState.name,
                                name: sText,
                                icon: sIcon,
                                group: null,
                                stateType: 1    // active
                            });
                            idCounter--;

                            if (oStateArray[2] === 'motion') {
                                sIcon = dss.staticDataModel.getImagePathForAKMValue(5, false);
                                sText = Ext.String.format2(_('{#editor_trigger_zonestates}motion is inactive'));
                            } else if (oStateArray[2] === 'presence') {
                                sIcon = dss.staticDataModel.getImagePathForAKMValue(1, false);
                                sText = Ext.String.format2(_('{#editor_trigger_zonestates}presence is inactive'));
                            } else if (oStateArray[2] === 'group') {
                                sIcon = dss.staticDataModel.getImagePathForState(oState.name, 'inactive'),
                                sText = dss.staticDataModel.getNameForState(stateName, 'inactive');
                            } else {
                                sIcon = dss.staticDataModel.getImagePathForAKMValue(0, false);
                                sText = Ext.String.format2(_('{#editor_condition_zonestates_generic}{state} is inactive'), { state: oStateArray[2] });
                            }

                            oGroupControlStore.add({
                                id: idCounter,
                                _id: idCounter,
                                state: oState.name,
                                name: sText,
                                icon: sIcon,
                                group: null,
                                stateType: 2    // inactive
                            });
                            idCounter--;
                        }
                    }
                }
            }
        }

        this.grids.lvlGroup = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oGroupControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('activity'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlGroup, this.conditionSelectorName + 'GridLvlZoneStateGroup');

        this.grids.lvlGroup.on('itemdblclick', function (grid, record) {
            if (record.get('_id') > idCounter) {
                me.macPanel.collapse(Ext.Component.DIRECTION_BOTTOM, true);
            }
        });
        this.grids.lvlGroup.on('itemclick', function (grid, record) {
            this.parent.macPanel.remove(this.parent.grids.lvlScene);
            var id = record.get('_id');
            var groupId = record.get('group');
            var state = record.get('state');
            if (groupId == -1) {
                grid.getSelectionModel().deselect(record);
                this.parent.updatePanelDescription();
                return;
            }
            if (id < 0) {
                me.oActualValue = null;
                var oCondiObject = { type: 'state' };
                oCondiObject.value = record.get('stateType') + '';
                oCondiObject.name = state;
                me.setSelectedSubItem(oCondiObject, distiguishID);
                this.parent.updatePanelDescription();
                return;
            }
            var zoneId = this.parent.grids.lvlRoom.getSelectionModel().getSelection()[0].get('ZoneID');
            if (id >= 0) {
                me.oActualValue = null;
                var oCondiObject = { type: 'zonestate' };
                oCondiObject.value = { zone: zoneId, group: 0, scene: id };
                me.setSelectedSubItem(oCondiObject, distiguishID);
            } else {    // id == undefined
                if (zoneId != null && groupId != null) {
                    this.parent.constructZoneStateLvlGroupScene(zoneId, groupId, distiguishID);
                }
            }
            this.parent.updatePanelDescription();
        });

        this.grids.lvlGroup.parent = this;
        this.macPanel.add(this.grids.lvlGroup);
    },

    addShowAllPresets: function (store, fAll) {
        store.add({
            id: -3,
            _id: -3,
            name: _('show all scenes'),
            icon: null,
            reachable: true,
            checkable: true,
            checked: fAll
        });
        store.add({
            id: -1,
            _id: -1,
            name: '-',
            passive: true,
            icon: null,
            reachable: true
        });
    },

    addLightScenes: function (store, zoneId) {
        // light-states
        store.add({
            id: -10,
            _id: -10,
            type: 'state',
            name: 'zone.' + zoneId + '.light',
            value: 'active',
            icon: dss.staticDataModel.getImagePathForScene(0, 1, 5),
            reachable: true
        });
        store.add({
            id: -11,
            _id: -11,
            type: 'state',
            name: 'zone.' + zoneId + '.light',
            value: 'inactive',
            icon: dss.staticDataModel.getImagePathForScene(0, 1, 0),
            reachable: true
        });
        store.add({
            id: -12,
            _id: -12,
            name: '-',
            passive: true,
            icon: null,
            reachable: true
        });
    },

    addZoneVentilationScenes: function (store, zoneId, groupId) {
        store.add({
            id: -10,
            _id: -10,
            type: 'state',
            name: 'zone.' + zoneId + '.group.' + groupId + '.status.malfunction',
            value: 'active',
            icon: dss.staticDataModel.getImagePathForState('zone.' + zoneId + '.group.' + groupId + '.status.malfunction', 'active'),
            passive: false,
            reachable: true
        });
        store.add({
            id: -11,
            _id: -11,
            type: 'state',
            name: 'zone.' + zoneId + '.group.' + groupId + '.status.malfunction',
            value: 'inactive',
            icon: dss.staticDataModel.getImagePathForState('zone.' + zoneId + '.group.' + groupId + '.status.malfunction', 'inactive'),
            passive: false,
            reachable: true
        });
        store.add({
            id: -12,
            _id: -12,
            type: 'state',
            name: 'zone.' + zoneId + '.group.' + groupId + '.status.service',
            value: 'active',
            icon: dss.staticDataModel.getImagePathForState('zone.' + zoneId + '.group.' + groupId + '.status.service', 'active'),
            passive: false,
            reachable: true
        });
        store.add({
            id: -13,
            _id: -13,
            type: 'state',
            name: 'zone.' + zoneId + '.group.' + groupId + '.status.service',
            value: 'inactive',
            icon: dss.staticDataModel.getImagePathForState('zone.' + zoneId + '.group.' + groupId + '.status.service', 'inactive'),
            passive: false,
            reachable: true
        });
        store.add({
            id: -14,
            _id: -14,
            name: '-',
            passive: true,
            icon: null,
            reachable: true
        });
    },

    constructZoneStateLvlGroupScene: function (zoneID, groupID, distiguishID, fAll, fForce) {
        var me = this;
        var oSceneControlStore = Ext.create('Ext.data.Store', {
            fields: [ '_id', 'type', 'name', 'value', 'icon', 'reachable', 'checkable', 'checked' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        if (fAll == null)
            fAll = false;
        if (fForce == null)
            fForce = false;
        if (zoneID == -1)
            zoneID = 0;
        var oSceneStore = this.getSceneStore(zoneID, groupID);
        me.addShowAllPresets(oSceneControlStore, fAll);
        if (groupID == 1) {
            me.addLightScenes(oSceneControlStore, zoneID);
        }
        if (groupID == 10 || groupID == 12) {
            me.addZoneVentilationScenes(oSceneControlStore, zoneID, groupID);
        }
        oSceneStore.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('reachable') || fAll) {
                if (oNewRecord.get('_id') < 64 && !oNewRecord.get('stateid')) {
                    oNewRecord.type = 'scene';
                    oSceneControlStore.add(oNewRecord);
                }
            }
            // else if (oNewRecord.get('_id') == 0) {    // 'off' scene
            //     oNewRecord.set('reachable', true);
            //     oNewRecord.type = 'scene';
            //     oSceneControlStore.add(oNewRecord);
            // }
        });

        this.grids.lvlScene = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: 324,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oSceneControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('scene'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {    // this.defMacGridRenderer
                    var name = record.get('name');
                    if (name === '-') {
                        return '<table style="height:16px;width:100%;"><tr style="height:16px;"><td width="100%"><hr></td></tr></table>';
                    }
                    var icon = record.get('icon');
                    if (record.get('type') === 'state') {
                        var stateSplit = name.split('.');
                        var stateName = stateSplit[stateSplit.length - 1];
                        name = dss.staticDataModel.getNameForState(stateName, record.get('value'));
                    }
                    var checkbox = '';
                    if (record.get('checkable')) {
                        checkbox = '<td style="padding-right: 5px;">' +
                            '<div style="width: 13px; height: 13px; background-image:url(' + dss.staticDataModel.baseImagePath + 'dss/checkbox.gif);' +
                                (record.get('checked') ? ' background-position: 0px -13px;' : '') + '">' +
                            '</div>'
                            '</td>';
                    }
                    icon = icon ? '<td class="icon-small"><img src="' + icon + '"></td>' : checkbox;
                    const value = record.get('value');
                    const techName = record.get('name');
                    const id = record.get('id');
                    var sReturn = '<table style="height:16px;width:100%;"' +
                        (id != null && id !== '' ? 'data-id="' + record.get('id') + '"' : '') +
                        'data-name="' + joinAndCapitalize(techName) + '"' +
                        (value != null ? 'data-value="' + value + '"' : '') + '>' +
                        '             <tr>' + icon +
                        '               <td width="100%">' + _(name) + '</td>' +
                        '             </tr>' +
                        '          </table>';
                    return sReturn;
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlScene, this.conditionSelectorName + 'GridLvlZoneStateGroupScene');
        this.grids.lvlScene.parent = this;

        this.grids.lvlScene.on('itemdblclick', function (grid, record) {
            me.macPanel.collapse(Ext.Component.DIRECTION_BOTTOM, true);
        });

        this.grids.lvlScene.on('itemclick', function (grid, record) {
            if (record.get('passive')) {
                this.getSelectionModel().deselect(record);
                me.updatePanelDescription();
                return;
            }

            var zoneID = this.parent.grids.lvlRoom.getSelectionModel().getSelection()[0].get('ZoneID');
            var groupID = this.parent.grids.lvlGroup.getSelectionModel().getSelection()[0].get('group');

            if (record.get('checkable') != null) {
                if (record.get('checkable')) {
                    this.getSelectionModel().deselect(record);
                    var fChecked = !record.get('checked');
                    record.set('checked', fChecked);
                    grid.getSelectionModel().deselectAll();
                    if (record.get('_id') == -3) {
                        // alle Stimmungen
                        grid.store.removeAll();

                        me.addShowAllPresets(grid.store, fChecked);
                        if (groupID == 1) {
                            me.addLightScenes(oSceneControlStore, zoneID);
                        }
                        if (groupID == 10 || groupID == 12) {
                            me.addZoneVentilationScenes(oSceneControlStore, zoneID, groupID);
                        }

                        oSceneStore.each(function (record) {
                            var oNewRecord = record.copy();
                            if (oNewRecord.get('reachable') || fChecked)
                                if (oNewRecord.get('_id') < 64) {
                                    oNewRecord.type = 'scene';
                                    grid.store.add(oNewRecord);
                                }
                        });
                    }
                    return;
                }
                // me.updatePanelDescription();
            }

            if (zoneID != null && groupID != null) {
                var oCondiObject = {};
                me.oActualValue = null;
                var name = record.get('name');
                var type = record.get('type')

                if (type === 'state') {
                    oCondiObject.type = 'state';
                    oCondiObject.value = (record.get('value') === 'active' ? '1' : '2');
                    oCondiObject.name = name;
                } else {
                    oCondiObject.type = 'zonestate';
                    oCondiObject.value = {
                        zone: zoneID,
                        group: groupID,
                        scene: record.get('_id')
                    };
                }
                oCondiObject.pos = 15;
                me.setSelectedSubItem(oCondiObject, distiguishID);
            }
            me.updatePanelDescription();
        });
        this.macPanel.add(this.grids.lvlScene);
    },
    constructZoneState: function (distiguishID) {
         var oZoneControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'ZoneID', {name:'name', sortType:'asUCText'} ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oZoneStore = Ext.data.StoreManager.lookup('conditionConfigurator_zoneStore');
        oZoneStore.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('ZoneID') != 0)
                oZoneControlStore.add(oNewRecord);
        });
        oZoneControlStore.sort('name');

        this.grids.lvlRoom = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oZoneControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {
                    return '<table style="height:16px;width:100%" data-zone-id="' + record.get('ZoneID') + '"> \
                        <tr><td width="100%">' + record.get('name') + '</td><td>&gt;</td></tr> \
                    </table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlRoom, this.conditionSelectorName + 'GridLvlZoneState');

        this.grids.lvlRoom.parent = this;
        this.grids.lvlRoom.on('itemclick', function (grid, record) {
            this.parent.macPanel.remove(this.parent.grids.lvlGroup);
            this.parent.macPanel.remove(this.parent.grids.lvlScene);
            var zoneID = record.get('ZoneID');
            if (zoneID == -1)
                zoneID = 0;
            this.parent.constructZoneStateLvlGroup(zoneID, distiguishID);
            this.parent.updatePanelDescription();
        });
        this.macPanel.add(this.grids.lvlRoom);
    },
    constructGridLvlDeviceStateValue:function (dSUID, type, distiguishID) {
        var oStates = dss.staticDataModel.getDeviceStates(dSUID);

        var oStateDeviceValueControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'type', {name:'name', sortType:'asUCText'},'selector' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        var oValueArray = [];

        var maxLabelWidth = 0;
        var calcWidth = this.calculateWidthHelper('x-grid-cell');

        for (var sKey in oStates) {
            if (sKey == type) {
                for (var sSubKey in oStates[sKey].options) {
                    oValueArray.push({type:sSubKey, name:oStates[sKey].options[sSubKey], selector:'dev.' + dSUID + '.' + type});
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oStates[sKey].options[sSubKey]));
                }
            }
        }
        oStateDeviceValueControlStore.add(oValueArray);
        oStateDeviceValueControlStore.sort('name');

         this.grids.lvlDeviceStateValue = Ext.create('Ext.grid.Panel', {
             hideHeaders: true,
             height: this.iDefaultHeight,
             width:  Math.max((this.getWidth()-(this.iDefaultWidth*4))-2, maxLabelWidth),
             autoScroll: true,
             viewConfig: {
                 stripeRows: true
             },
             store: oStateDeviceValueControlStore,
             columns: [ {
                 sortable: false,
                 hideable: false,
                 text: _('room'),
                 dataIndex: 'name',
                 flex: 1,
                 renderer: function (val, metadata, record, colIdx, store, view) {
                     return '<table style="height:16px;width:100%" data-type="' + record.get('type') + '" data-name="' + joinAndCapitalize(record.get('name')) + '"> \
                        <tr><td width="100%">' + record.get('name') + '</td></tr> \
                    </table>';
                 }
             } ]
         });
         addDataIdToElement(this.grids.lvlDeviceStateValue, this.conditionSelectorName + 'GridLvlDeviceStateZoneDeviceStateValue');

         var me = this;

         this.grids.lvlDeviceStateValue.parent = this;
         this.grids.lvlDeviceStateValue.on('itemclick', function (grid, record) {
            var value = record.get('type');
            var selector = record.get('selector');
             me.oActualValue = null;
            var oCondiObject = { type: 'state' };
            oCondiObject.value = value;
            oCondiObject.name = selector;
            me.setSelectedSubItem(oCondiObject, distiguishID);

         });
         this.grids.lvlDeviceStateValue.on('itemdblclick', function (grid, record) {
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
         });

         this.macPanel.add(this.grids.lvlDeviceStateValue);
    },
    constructGridLvlDeviceStateState:function (dSUID, distiguishID) {
        var oStates = dss.staticDataModel.getDeviceStates(dSUID);

        var oStateDeviceStateControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'type', {name:'name', sortType:'asUCText'} ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        var oStateArray = [];

        var maxLabelWidth = 0;
        var calcWidth = this.calculateWidthHelper('x-grid-cell');

        for (var sKey in oStates) {
            oStateArray.push({type:sKey, name:oStates[sKey].title});
            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oStates[sKey].title));
        }
        oStateDeviceStateControlStore.add(oStateArray);

        oStateDeviceStateControlStore.sort('name');

        this.grids.lvlDeviceStateState = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: Math.max(this.iDefaultWidth, maxLabelWidth),
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oStateDeviceStateControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {
                    const name = record.get('name');
                    return '<table style="height:16px;width:100%" data-type="' + record.get('type') + '" data-name="' + joinAndCapitalize(name) + '"> \
                        <tr><td width="100%">' + name + '</td><td>&gt;</td></tr> \
                    </table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlDeviceStateState, this.conditionSelectorName + 'GridLvlDeviceStateZoneDeviceState');

        this.grids.lvlDeviceStateState.parent = this;
        this.grids.lvlDeviceStateState.on('itemclick', function (grid, record) {
            this.parent.macPanel.remove(this.parent.grids.lvlDeviceStateValue);
            var type = record.get('type');
            this.parent.constructGridLvlDeviceStateValue(dSUID, type, distiguishID);
            this.parent.updatePanelDescription();
        });
        this.macPanel.add(this.grids.lvlDeviceStateState);
    },
    calculateWidthHelper: function (styleClass) {
        var el = document.getElementById('WidthHelper');
        if (!el) {
            el = document.createElement('DIV');
            el.id = 'WidthHelper';
            el.className = styleClass;
            el.style.position = 'fixed';
            el.style.top = '0px';
            el.style.left = '0px';
            el.style.marginLeft = '-99999px';
            document.body.appendChild(el);
        }
        return function (str) {
            el.innerHTML = str;
            return el.offsetWidth;
        }
    },
    constructGridLvlDeviceStateDevice: function (zoneID, distiguishID) {

        var oDeviceControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'dSID', 'dSUID', { name: 'name', sortType: 'asUCText' }, 'icon', 'present'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var maxLabelWidth = 0;
        var calcWidth = this.calculateWidthHelper('x-grid-cell');

        var oDeviceArray = [];
        if (dss.staticDataModel.deviceCache) {
            for (var iIndex = 0; iIndex < dss.staticDataModel.deviceCache.length; iIndex++) {
                if (dss.staticDataModel.deviceCache[iIndex].present) {
                    if (dss.staticDataModel.deviceCache[iIndex].hasStates) {
                        if (dss.staticDataModel.deviceCache[iIndex].ZoneID == zoneID) {
                            oDeviceControlStore.add(dss.staticDataModel.deviceCache[iIndex]);
                            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(dss.staticDataModel.deviceCache[iIndex].name));
                        }
                    }
                }
            }
        }

        oDeviceControlStore.sort('name');
        this.grids.lvlDeviceStateDevice = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: Math.max(this.iDefaultWidth, maxLabelWidth + 21),   // + 21 for icon
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oDeviceControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('device'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlDeviceStateDevice, this.conditionSelectorName + 'GridLvlDeviceStateZoneDevice');
        this.grids.lvlDeviceStateDevice.parent = this;
        this.grids.lvlDeviceStateDevice.on('itemclick', function (grid, record) {
            this.parent.macPanel.remove(this.parent.grids.lvlDeviceStateState);
            this.parent.macPanel.remove(this.parent.grids.lvlDeviceStateValue);
            var dSUID = record.get('dSUID');
            this.parent.constructGridLvlDeviceStateState(dSUID, distiguishID);
            this.parent.updatePanelDescription();
        });
        this.macPanel.add(this.grids.lvlDeviceStateDevice);

    },
    constructGridDeviceStateRoom: function (distiguishID) {
        var oRoomArray = [];
        if (dss.staticDataModel.deviceCache) {
            for (var iIndex = 0; iIndex < dss.staticDataModel.deviceCache.length; iIndex++) {
                if (dss.staticDataModel.deviceCache[iIndex].present)
                    if (dss.staticDataModel.deviceCache[iIndex].hasStates)
                        if (oRoomArray.indexOf(dss.staticDataModel.deviceCache[iIndex].ZoneID)<0) {
                            oRoomArray.push(dss.staticDataModel.deviceCache[iIndex].ZoneID);
                        }
            }
        }
        var oRoomDeviceStateControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'ZoneID', {name:'name', sortType:'asUCText'} ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        var oArray = [];
        for (var iIndex = 0; iIndex < oRoomArray.length; iIndex++) {
            oArray.push({ZoneID:oRoomArray[iIndex], name:dss.staticDataModel.getNameForZone(oRoomArray[iIndex])});
        }
        oRoomDeviceStateControlStore.add(oArray);
        oRoomDeviceStateControlStore.sort('name');

        this.grids.lvlDeviceStateRoom = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oRoomDeviceStateControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {
                    return '<table style="height:16px;width:100%" data-zone-id="' + record.get('ZoneID') + '"> \
                        <tr><td width="100%">' + record.get('name') + '</td><td>&gt;</td></tr> \
                    </table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlDeviceStateRoom, this.conditionSelectorName + 'GridLvlDeviceStateZone');

        this.grids.lvlDeviceStateRoom.parent = this;
        this.grids.lvlDeviceStateRoom.on('itemclick', function (grid, record) {
            this.parent.macPanel.remove(this.parent.grids.lvlDeviceStateDevice);
            this.parent.macPanel.remove(this.parent.grids.lvlDeviceStateState);
            this.parent.macPanel.remove(this.parent.grids.lvlDeviceStateValue);
            var zoneID = record.get('ZoneID');
            this.parent.constructGridLvlDeviceStateDevice(zoneID, distiguishID);
            this.parent.updatePanelDescription();
        });
        this.macPanel.add(this.grids.lvlDeviceStateRoom);
    },
    constructGridLvlGlobalApplicationScenes: function (GroupID, distiguishID) {
        var oGAValueStore = Ext.create('Ext.data.Store', {
            fields: [ 'pos', 'value', 'name', 'title' ,'passive', 'icon','groupID','sceneID' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oPossibleStates = [];
        switch (GroupID) {
            case 64:
                oPossibleStates.push({
                    name: 'zone.0.group.64.status.malfunction',
                    type: 'malfunction'
                });
                oPossibleStates.push({
                    name: 'zone.0.group.64.status.service',
                    type: 'service'
                });
            break;
        }

        for (var iIndex = 0, len = oPossibleStates.length; iIndex < len; iIndex++) {
            var oItem = {
                name: oPossibleStates[iIndex].name,
                value: 'active',
                title: dss.staticDataModel.getNameForState(oPossibleStates[iIndex].type, 'active'),
                icon: dss.staticDataModel.getImagePathForState(oPossibleStates[iIndex].name, 'active'),
                passive: false,
                pos: 8
            };
            oGAValueStore.add(oItem);
            var oItem = {
                 name: oPossibleStates[iIndex].name,
                 value: 'inactive',
                 title: dss.staticDataModel.getNameForState(oPossibleStates[iIndex].type, 'inactive'),
                 icon: dss.staticDataModel.getImagePathForState(oPossibleStates[iIndex].name, 'inactive'),
                 passive: false,
                 pos: 8
            };
            oGAValueStore.add(oItem);
        }
        var scenes = dss.staticDataModel.getScenes(0, GroupID);
        if (scenes && scenes.length>0) {
            oGAValueStore.add({
                name: '-',
                value: null,
                title: null,
                icon: null,
                passive: true,
                pos: 8
            });
            for (var iIndex = 0; iIndex < scenes.length; iIndex++) {
                // if (iIndex === 0 && scenes[iIndex].id !== 0) {  // add 'Off' scene if it isn't in basic active scenes
                //     oGAValueStore.add({
                //         groupID: GroupID,
                //         value: null,
                //         sceneID: 0,
                //         name: _('Off'),
                //         title: _('Off'),
                //         icon: dss.staticDataModel.getImagePathForScene(0, GroupID, 0),
                //         passive: true,
                //         pos: 8
                //     });
                // }
                oGAValueStore.add({
                    groupID: GroupID,
                    value: null,
                    sceneID: scenes[iIndex].id,
                    name:scenes[iIndex].name,
                    title: scenes[iIndex].name,
                    icon: scenes[iIndex].icon,
                    passive: true,
                    pos: 8
                });
            }
            // @TODO - IMHO all possible scenes for ventilation should be added here
        }
        this.grids.lvlGlobalApplicationScenes = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: 324,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oGAValueStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('State'),
                dataIndex: 'title',
                renderer: function (val, metadata, record, colIdx, nochNenParameter, store, view) {
                    if (record.get('name') === '-') {
                        return '<table style="height:16px;width:100%;"><tr style="height:16px;"><td width="100%"><hr></td></tr></table>';
                    }

                    var fHasIcon = false;
                    if (record.get('icon') != null) {
                        fHasIcon = true;
                    }

                    var fHasNext = false;
                    const sceneId = record.get('sceneID');
                    const name = record.get('name');
                    const value = record.get('value');
                    var sReturn = '<table style="height:16px;width:100%;" \
                        data-name="' + joinAndCapitalize(name) + '"' +
                        (sceneId != null && sceneId !== '' ? ' data-scene-id="' + sceneId + '"' : '') +
                        (value != null ? ' data-value="' + value + '"' : '') +
                        '><tr>';
                    var iColspaceCounter = 3;
                    if (fHasIcon) {
                        iColspaceCounter--;
                        sReturn += '<td class="icon-small"><img src="' + record.get('icon') + '" style="margin-right: 5px;"></td>';
                    }
                    sReturn += '<td colspan="' + iColspaceCounter + '" width="100%">' + record.get('title') + '</td>';
                    return sReturn + '</tr></table>';

                },
                flex: 1
            } ]
        });
        addDataIdToElement(this.grids.lvlGlobalApplicationScenes, this.conditionSelectorName + 'GridLvlApplicationScenes');

        this.grids.lvlGlobalApplicationScenes.parent = this;
        this.grids.lvlGlobalApplicationScenes.on('itemclick', function (grid, record) {

            this.parent.oActualValue = null;
            if (record.get('value') != null) {
                this.parent.oActualValue = null;
                var oCondiObject = { type: 'state' };
                oCondiObject.value = record.get('value');
                oCondiObject.name = record.get('name');
                this.parent.setSelectedSubItem(oCondiObject, record.get('pos'));
            } else {
                this.parent.oActualValue = null;
                var oCondiObject = { type: 'zonestate' };
                oCondiObject.value = {zone:0, group:record.get('groupID'), scene:record.get('sceneID')};
                this.parent.setSelectedSubItem(oCondiObject, record.get('pos'));
            }
        });

        this.grids.lvlGlobalApplicationScenes.on('itemdblclick', function (grid, record) {
            if (this.parent.oActualValue != null) {
                this.parent.macPanel.collapse();
            }
        });

        this.macPanel.add(this.grids.lvlGlobalApplicationScenes);

    },
    constructGridLvlGlobalApplication: function (distiguishID) {
        var oGAControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'GroupID', {name:'name', sortType:'asUCText'} ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        var oArray = [];
        if (dss.staticDataModel.nameCache) {
            for (var iIndex = 0; iIndex < dss.staticDataModel.nameCache.length; iIndex++) {
                if (dss.staticDataModel.nameCache[iIndex].ZoneID == 0 && dss.staticDataModel.nameCache[iIndex].groups) {
                    for (var jIndex = 0; jIndex < dss.staticDataModel.nameCache[iIndex].groups.length; jIndex++) {
                        if (dss.staticDataModel.nameCache[iIndex].groups[jIndex].group > 63) {
                            var oGroup = {
                                GroupID: dss.staticDataModel.nameCache[iIndex].groups[jIndex].group,
                                name: dss.staticDataModel.nameCache[iIndex].groups[jIndex].name
                            };
                            if (dss.staticDataModel.getDevicesForZone(0, dss.staticDataModel.nameCache[iIndex].groups[jIndex].group).length == 0)
                                oGroup.passive = true;
                            oArray.push(oGroup);
                        }
                    }
                }
            }
        }
        oGAControlStore.add(oArray);
        oGAControlStore.sort('name');

        this.grids.lvlGlobalApplication = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: 300,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oGAControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {
                    const name = record.get('name');
                    return '<table style="height:16px;width:100%" data-group-id="' + record.get('GroupID') + '" data-name="' + joinAndCapitalize(name) + '"> \
                        <tr><td width="100%">' + name + '</td><td>&gt;</td></tr> \
                    </table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlGlobalApplication, this.conditionSelectorName + 'GridLvlApplication');

        this.grids.lvlGlobalApplication.parent = this;
        this.grids.lvlGlobalApplication.on('itemclick', function (grid, record) {
            this.parent.macPanel.remove(this.parent.grids.lvlGlobalApplicationScenes);
            var GroupID = record.get('GroupID');
            this.parent.constructGridLvlGlobalApplicationScenes(GroupID, distiguishID);
            this.parent.updatePanelDescription();
        });
        this.macPanel.add(this.grids.lvlGlobalApplication);
    },
    constructMacGrids: function (iIgnoreDisID) {
        if (iIgnoreDisID == null)
            iIgnoreDisID = -1;
        var lvl1TempStore = Ext.create('Ext.data.Store', {
            fields: [ 'type', 'name', 'next', 'icon' ,'passive','parameter','distiguishID'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        var me = this;
        var lvl1Store = Ext.data.StoreManager.lookup('conditionConfigurator_l1Store');  // a list of condition types (first columns of editor)
        lvl1Store.each(function (record) {
            var oNewRecord = record.copy();
            var oDistinguidID = oNewRecord.get('distiguishID');
            if (oDistinguidID >= 0)
                if (oDistinguidID != iIgnoreDisID) {
                    if (me.gridStore.find('pos', oDistinguidID, 0, false, false, true) >= 0) {
                        oNewRecord.set('passive', true);
                        if (oNewRecord.get('type')==7)
                        	oNewRecord.set('passive', false);
                        if (this.oActualValue != null)
                            if (this.oActualValue.distiguishID != null)
                                if (this.oActualValue.distiguishID == oDistinguidID)
                                    oNewRecord.set('passive', false);
                        oNewRecord.commit();
                    }
                }
            lvl1TempStore.add(oNewRecord);
        });

        this.grids.lvl1 = Ext.create('Ext.grid.Panel', {
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            hideHeaders: true,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: lvl1TempStore,
            columns: [ {
                sortable: false,
                hideable: false,
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        this.grids.lvl1.getSelectionModel().store = this.grids.lvl1.store;
        this.grids.lvl1.parent = this;
        addDataIdToElement(this.grids.lvl1, this.conditionSelectorName + 'GridLvl1');

        this.grids.lvl1.on('itemclick', function (grid, record) {
            for (var sKey in this.parent.grids) {
                if (sKey != 'lvl1')
                    this.parent.macPanel.remove(this.parent.grids[sKey]);
            }
            this.parent.oActualValue = null;
            var fIsPassive = false;
            if (record.get('passive') != null) {
                if (record.get('passive') == true) {
                    fIsPassive = true;
                }
            }

            var type = record.get('type');
            if (!fIsPassive) {
                if (record.get('type') == 0) {
                    this.parent.constructEnabler(record.get('distiguishID'));
                }
                if (record.get('type') == 1) {
                    this.parent.constructWeekdays(record.get('distiguishID'));
                }
                if (record.get('type') == 2) {
                    this.parent.constructTimeframe(record.get('distiguishID'));
                }
                if (record.get('type') == 3) {
                    this.parent.constructZoneState(record.get('distiguishID'));
                }
                if (record.get('type') == 4) {
                    this.parent.constructSystemState(record.get('parameter'), record.get('distiguishID'));
                }
                if (record.get('type') == 5) {
                    this.parent.constructSystemStateMulti(record.get('parameter'), record.get('distiguishID'));
                }
                if (record.get('type') == 6) {
                    this.parent.constructGridLvlAKM(record.get('distiguishID'));
                }
                if (record.get('type') == 7) {
                    this.parent.constructAddonState(record.get('distiguishID'));
                }
                if (record.get('type') == 10) {
                    this.parent.constructDateframe(record.get('distiguishID'));
                }
                if (record.get('type') == 11) {
                    this.parent.constructGridDeviceStateRoom(record.get('distiguishID'));
                }
                if (record.get('type') == 12) {
                    this.parent.constructGridLvlGlobalApplication(record.get('distiguishID'));
                }
                if (record.get('type') == 13) {
                    this.parent.constructClimateAddonState(record.get('distiguishID'));
                }
            } else {
                 this.getSelectionModel().deselect(record);
            }
            this.parent.macPanel.doLayout();
        });
       this.macPanel.add(this.grids.lvl1);
    },
    setSelectedSubItem: function (oObject, distiguishID) {
        this.oActualValue = oObject;
        this.oActualValue.distiguishID = distiguishID;
        if (this.getSelectedMainRecord() == null) {
            var oNewRecord = this.gridStore.add({
                pos: distiguishID,
                condition: oObject
            });
            this.setSelectedMainRecord(this.gridStore.getCount() - 1, true);
        } else {
            this.getSelectedMainRecord().set('condition', oObject);
            this.getSelectedMainRecord().set('pos', distiguishID);
            this.getSelectedMainRecord().commit();
        }
        this.oActualValue = this.getSelectedMainRecord();
        this.updatePanelDescription();
        this.removeCls('myInvalid');
    },
    setSelectedMainRecord: function (index, fNoMacgridRebuild) {
        this.mainGrid.getSelectionModel().select(index);
        if (fNoMacgridRebuild != null)
            if (fNoMacgridRebuild)
                return;
        if (!this.macPanel.collapsed)
            this.setMacGrid(this.getSelectedMainRecord());
    },
    getSelectedMainRecord: function () {
        if (this.mainGrid.getSelectionModel().getSelection() == null)
            return null;
        return this.mainGrid.getSelectionModel().getSelection()[0];
    },
    deselectMainGrid: function () {
        var oldSelect = this.getSelectedMainRecord();
        if (oldSelect != null) {
            this.mainGrid.getSelectionModel().deselect(oldSelect);
        } else
            this.mainGrid.getSelectionModel().deselectAll();
    },
    setMacGrid: function (oObject) {
        for (var sKey in this.grids) {
            if (this.grids.hasOwnProperty(sKey)) {
                this.macPanel.remove(this.grids[sKey]);
            }
        }
        if (oObject) {
            this.oActualValue = oObject;
        }
        if (oObject == null) {
            this.constructMacGrids();
        } else {
            var distiguishID = (oObject.distiguishID ? oObject.distiguishID : oObject.get('pos'));
            this.constructMacGrids(distiguishID);

            var oLVL1Record = this.grids.lvl1.store.findRecord('distiguishID', distiguishID, 0, false, false, true);
            var selModel = this.grids.lvl1.getSelectionModel();
            if (!selModel.views[0]) {
                selModel.views.push(selModel.view);
            }
            selModel.select(oLVL1Record);
            var condition = oObject.get('condition');
            var value = condition.value;
            var name = (typeof condition.name !== 'undefined' ? condition.name : '');
            var recordDistinguishID = oLVL1Record.get('distiguishID');
            var recordType = oLVL1Record.get('type');
            var recordParameter = oLVL1Record.get('parameter');
            var recordApp;
            var recordZone;
            var recordScene;
            var recordGroup;
            var recordValue;
            var recordDevice;
            var oDevice;
            var oSelectAbleRecord;
            var nameArray = (name.indexOf('.') > -1 ? name.split('.') : []);

            if (recordType == 0) {
                this.constructEnabler(recordDistinguishID);
            }

            if (recordType == 1) {
                this.constructWeekdays(recordDistinguishID);
                this.grids.lvlWeekdaySelector.setValue({
                    recurrenceBase: 'weekly',
                    timeArray: value
                });
            }

            if (recordType == 2) {
                this.constructTimeframe(recordDistinguishID);
                this.grids.lvlTimeFrameMin.setValue(value.start);
                this.grids.lvlTimeFrameMax.setValue(value.end);
            }

            if (recordType == 3) {
                var zoneId = value.zone;
                if (typeof zoneId === 'undefined' && nameArray[0] === 'zone' && nameArray[1].match(/^[0-9]+$/gi)) {
                    zoneId = parseInt(nameArray[1], 10);
                }
                var groupId = value.group;
                if (typeof groupId === 'undefined' && nameArray[2] === 'group' && nameArray[3].match(/^[0-9]+$/gi)) {
                    groupId = parseInt(nameArray[3], 10);
                }

                this.constructZoneState(recordDistinguishID);
                recordZone = this.grids.lvlRoom.store.find('ZoneID', zoneId, 0, false, false, true);
                if (recordZone === -1) {
                    return;
                } else {    // this is zone state
                    this.grids.lvlRoom.getSelectionModel().select(recordZone);

                    this.constructZoneStateLvlGroup(zoneId, recordDistinguishID);
                    if (typeof groupId === 'undefined' && nameArray.length === 3) {
                        switch(nameArray[2]) {
                            case 'heating':
                                groupId = 3;
                            break;
                            case 'light':
                                groupId = 1;
                            break;
                        }
                    }
                    if (!groupId) {
                        recordGroup = this.grids.lvlGroup.store.find('_id', value.scene, 0, false, false, true);
                        if (recordGroup != -1) {
                            this.grids.lvlGroup.getSelectionModel().select(recordGroup);
                        } else {
                            recordGroup = this.grids.lvlGroup.store.queryBy(function (rec) {
                                return rec.get('state') === name && rec.get('stateType') + '' === value;
                            }).items;
                            if (recordGroup.length && recordGroup[0] != -1) {
                                this.grids.lvlGroup.getSelectionModel().select(recordGroup[0]);
                            } else {
                                return;
                            }
                        }
                    } else {
                        recordGroup = this.grids.lvlGroup.store.find('group', groupId, 0, false, false, true);
                        if (recordGroup != -1) {
                            this.grids.lvlGroup.getSelectionModel().select(recordGroup);
                        } else {
                            return;
                        }
                        this.constructZoneStateLvlGroupScene(zoneId, groupId, recordDistinguishID, false, false);

                        if (typeof value.scene === 'undefined') {
                            // this is state, not scene, but it should be in the lvlScene store
                            recordScene = this.grids.lvlScene.store.findBy(function (record) {
                                return record.get('name') === name && record.get('value') === (value === '1' ? 'active' : 'inactive');
                            });
                        } else {
                            recordScene = this.grids.lvlScene.store.find('_id', value.scene, 0, false, false, true);
                        }
                        if (recordScene != -1) {
                            this.grids.lvlScene.getSelectionModel().select(recordScene);
                        } else {
                            this.macPanel.remove(this.grids.lvlScene);
                            this.constructZoneStateLvlGroupScene(zoneId, groupId, recordDistinguishID, true, false);
                            recordScene = this.grids.lvlScene.store.find('_id', value.scene, 0, false, false, true);
                            if (recordScene != -1) {
                                this.grids.lvlScene.getSelectionModel().select(recordScene);
                            } else {
                                return;
                            }
                        }
                    }
                }
            }

            if (recordType == 4) {
                this.constructSystemState(recordParameter, recordDistinguishID);
                oSelectAbleRecord = null;
                this.grids.lvlSystemState.store.each(function (oRecord) {
                    if ((oRecord.get('name') == name) && (oRecord.get('value') == value)) {
                        oSelectAbleRecord = oRecord;
                    }
                });
                if (oSelectAbleRecord != null) {
                    this.grids.lvlSystemState.getSelectionModel().select(oSelectAbleRecord);
                }
            }

            if (recordType == 5) {
                this.constructSystemStateMulti (recordParameter, recordDistinguishID, condition.group);
                oSelectAbleRecord = null;
                this.grids.lvlSystemState.store.each(function (oRecord) {
                    if ((oRecord.get('name') == name) && (oRecord.get('value') == value)) {
                        oSelectAbleRecord = oRecord;
                    }
                });
                if (oSelectAbleRecord != null) {
                    this.grids.lvlSystemState.getSelectionModel().select(oSelectAbleRecord);
                }
            }

            if (recordType == 6) {
                if (nameArray.length == 3) {
                    if (nameArray[0] === 'dev') {
                        oDevice = dss.staticDataModel.getDevice(nameArray[1]);
                        if (oDevice != null) {
                            var typeIndex = parseInt(nameArray[2], 10);
                            var type = 255;
                            if (oDevice.binaryInputs) {
                                for (var k = 0; k < oDevice.binaryInputs.length; k++) {
                                    if (oDevice.binaryInputs[k].index == typeIndex) {
                                        type = oDevice.binaryInputs[k].type;
                                    }
                                }
                            }
                            this.constructGridLvlAKM(recordDistinguishID);
                            recordZone = this.grids.lvlAKMRoom.store.find('ZoneID', oDevice.ZoneID, 0, false, false, true);
                            if (recordZone != -1) {
                                this.grids.lvlAKMRoom.getSelectionModel().select(recordZone);
                                this.constructGridLvlDeviceAKMs(oDevice.ZoneID, recordDistinguishID);
                                recordDevice = this.grids.lvlAKMDevice.store.find('dSUID', nameArray[1], 0, false, false, true);
                                if (recordDevice != -1) {
                                    this.grids.lvlAKMDevice.getSelectionModel().select(recordDevice);
                                    this.constructGridLvlDeviceAKMValue(nameArray[1], recordDistinguishID);

                                    recordValue = this.grids.lvlDeviceAKMValue.store.findBy(function (record) {
                                        if (record.get('index') == parseInt(nameArray[2], 10)) {
                                            if (record.get('_id') == condition.value) {
                                                return true;
                                            }
                                        }
                                        return false;
                                    });
                                    if (recordValue != -1) {
                                        this.grids.lvlDeviceAKMValue.getSelectionModel().select(recordValue);
                                    }
                                }
                            }
                        }
                    }
                }
            }

            if (recordType == 7) {
                this.constructAddonState(recordDistinguishID);
                oSelectAbleRecord = null;
                this.grids.lvlAddonState.store.each(function (oRecord) {
                    if (oRecord.get('_id') == condition.stateName) {
                        oSelectAbleRecord = oRecord;
                    }
                });
                if (oSelectAbleRecord != null) {
                    this.grids.lvlAddonState.getSelectionModel().select(oSelectAbleRecord);
                    this.constructAddonStateValue(oSelectAbleRecord.get('_id'), recordDistinguishID);

                    var oSelectAbleRecord2 = null;
                    var targetVal = 'active';
                    if (condition.stateValue == 2) {
                        targetVal = 'inactive';
                    }
                    this.grids.lvlAddonStateValue.store.each(function (oRecord) {
                        if (oRecord.get('value') == targetVal) {
                            oSelectAbleRecord2 = oRecord;
                        }
                    });
                    if (oSelectAbleRecord2 != null) {
                        this.grids.lvlAddonStateValue.getSelectionModel().select(oSelectAbleRecord2);
                    }
                }
            }

            if (recordType == 10) {
                var startDate = value.start;
                var endDate = value.end;

                this.constructDateframe(recordDistinguishID, parseInt(startDate.substr(6, 2), 10), startDate.substr(4, 2), parseInt(endDate.substr(6, 2), 10), endDate.substr(4, 2));
            }

            if (recordType == 11) {
                this.constructGridDeviceStateRoom(recordDistinguishID);

                if ((nameArray.length == 3) && (nameArray[0] === 'dev')) {
                    oDevice = dss.staticDataModel.getDevice(nameArray[1]);
                    if (oDevice != null) {
                        recordZone = this.grids.lvlDeviceStateRoom.store.find('ZoneID', oDevice.ZoneID, 0, false, false, true);
                        if (recordZone != -1) {
                            this.grids.lvlDeviceStateRoom.getSelectionModel().select(recordZone); // selects room
                            this.constructGridLvlDeviceStateDevice(oDevice.ZoneID, recordDistinguishID);
                            recordDevice = this.grids.lvlDeviceStateDevice.store.find('dSUID', nameArray[1], 0, false, false, true);
                            if (recordDevice != -1) {
                                this.grids.lvlDeviceStateDevice.getSelectionModel().select(recordDevice);  // selects tielsa (or other vdc)
                                this.constructGridLvlDeviceStateState(oDevice.dSUID, recordDistinguishID);
                                var recordState = this.grids.lvlDeviceStateState.store.find('type', nameArray[2], 0, false, false, true);
                                if (recordState != -1) {
                                    this.grids.lvlDeviceStateState.getSelectionModel().select(recordState);    // select criterion
                                    this.constructGridLvlDeviceStateValue(oDevice.dSUID, nameArray[2], recordDistinguishID);
                                    recordValue = this.grids.lvlDeviceStateValue.store.find('type', value, 0, false, false, true);
                                    if (recordValue != -1) {
                                        this.grids.lvlDeviceStateValue.getSelectionModel().select(recordValue);    // select criterion value
                                    }
                                }
                            }
                        }
                    }
                }
            }

            if (recordType == 12) {
                this.constructGridLvlGlobalApplication(recordDistinguishID);
                if (condition.type === 'zonestate') {
                    if (value.group == 64) {
                        recordApp = this.grids.lvlGlobalApplication.store.find('GroupID', 64, 0, false, false, true);
                        if (recordApp != -1) {
                            this.grids.lvlGlobalApplication.getSelectionModel().select(recordApp);
                            this.constructGridLvlGlobalApplicationScenes(64);
                            recordValue = this.grids.lvlGlobalApplicationScenes.store.find('sceneID', value.scene, 0, false, false, true);
                            if (recordValue != -1) {
                                this.grids.lvlGlobalApplicationScenes.getSelectionModel().select(recordValue);
                            }
                        }
                    }
                } else {
                    if (name.indexOf('zone.0.group.64') > -1) {
                        recordApp = this.grids.lvlGlobalApplication.store.find('GroupID', 64, 0, false, false, true);
                        if (recordApp != -1) {
                            this.grids.lvlGlobalApplication.getSelectionModel().select(recordApp);
                            this.constructGridLvlGlobalApplicationScenes(64);
                            recordValue = this.grids.lvlGlobalApplicationScenes.store.findBy(function (record) {
                                return record.get('name') === name && record.get('value') === value;
                            });
                            if (recordValue != -1) {
                                this.grids.lvlGlobalApplicationScenes.getSelectionModel().select(recordValue);
                            }
                        }
                    }
                }
            }

            if (recordType == 13) {
                this.constructClimateAddonState(recordDistinguishID);
                oSelectAbleRecord = null;
                this.grids.lvlClimateAddonState.store.each(function (oRecord) {
                    if (oRecord.get('name') === 'off' && condition.stateValue === 0 ||
                        oRecord.get('name') === 'heating' && condition.stateValue === 1 ||
                        oRecord.get('name') === 'cooling' && condition.stateValue === 2) {
                        oSelectAbleRecord = oRecord;
                    }
                });
                this.grids.lvlClimateAddonState.getSelectionModel().select(oSelectAbleRecord);
            }

        }
        this.updatePanelDescription();
    },
    constructor: function (config) {
        /* conditions: [
         * 	 {type: <[enabler,weekdays,timeframe,zonestate]>, <[defaultValue:true]>, <[readonly:true]>}
         *   {type: <[singleItem,multiItem]>, entry: [{type:state,name:'<statename>',value:<['active','inactive']>, defaultValue: [{name,state}]}]
         *   |type: deviceState, <filterFunc:callback>, defaultValue: [{name,state}]|[type:zonestate, name: [<'motion','presence'>]] }
         * ]
         * */

        Ext.tip.QuickTipManager.init();

        var mThis = this;
        this.fEditMode = false;

        this.initGridStore();
        this.initStores(config.conditions);
        Ext.data.StoreManager.on('clear', function () {mThis.initStores(config.conditions);});

        this.fIgnorePresence = false;
        if (config.ignorePresence != null)
            this.fIgnorePresence = config.ignorePresence;

        this.fHeatingOperationMode = false;
        if (config.heatingOperationMode != null)
            this.fHeatingOperationMode = config.heatingOperationMode;

        this.lblSelectorAnzeige = Ext.create('Ext.form.Label', {
            width: 200,
            height: 22,
            padding: 4,
            style: {
                textAlign: 'right'
            }
        });

        this.conditionSelectorName = config.myID
            ? config.myID
            : config.autoEl && config.autoEl['data-id']
                ? config.autoEl['data-id']
                : '';

        this.oButtonNew = Ext.create('Ext.button.Button', {
            iconCls: 'icon-add-action',
            text: _('new condition'),
            handler: function () {
                mThis.oActualValue = null;
                if (mThis.macPanel.collapsed) {
                    mThis.macPanel.expand(true);
                } else {
                    mThis.setMacGrid();
                }
                mThis.deselectMainGrid();
                mThis.oButtonEdit.setDisabled(mThis.gridStore.getCount()===0);
            }
        });
        addDataIdToElement(this.oButtonNew, this.conditionSelectorName + 'NewConditionButton');
        this.oButtonNew.parent = this;

        this.oButtonEdit = Ext.create('Ext.button.Button', {
            iconCls: 'icon-edit-action',
            text: _('show all conditions'), // default of macPanel is expanded
            handler: function () {
                if (mThis.macPanel.collapsed) {
                    mThis.oActualValue = mThis.getSelectedMainRecord();
                    mThis.macPanel.expand(true);
                } else {
                    mThis.macPanel.collapse();
                }
            }
        });
        addDataIdToElement(this.oButtonEdit, this.conditionSelectorName + 'EditOrShowAllConditionsButton');
        this.oButtonEdit.parent = this;

        this.oButtonUp = Ext.create('Ext.button.Button', {
            iconCls: 'icon-left-action',
            disabled: true,
            handler: function () {
                var iIndex = mThis.mainGrid.store.indexOf(mThis.getSelectedMainRecord());
                mThis.setSelectedMainRecord(iIndex - 1);
            }
        });
        addDataIdToElement(this.oButtonUp, this.conditionSelectorName + 'ConditionUpButton');
        this.oButtonUp.parent = this;

        this.oButtonDown = Ext.create('Ext.button.Button', {
            iconCls: 'icon-right-action',
            disabled: true,
            handler: function () {
                var iIndex = mThis.mainGrid.store.indexOf(mThis.getSelectedMainRecord());
                mThis.setSelectedMainRecord(iIndex + 1);
            }
        });
        addDataIdToElement(this.oButtonDown, this.conditionSelectorName + 'ConditionDownButton');
        this.oButtonDown.parent = this;

        this.oButtonDelete = Ext.create('Ext.button.Button', {
            iconCls: 'icon-delete-action',
            tooltip: _('delete condition'),
            disabled: true,
            handler: function () {
                mThis.macPanel.collapse();
                mThis.gridStore.remove(mThis.getSelectedMainRecord());
                mThis.macPanel.setTitle('');
            }
        });
        addDataIdToElement(this.oButtonDelete, this.conditionSelectorName + 'DeleteConditionButton');
        this.oButtonDelete.parent = this;

        this.mainGrid = Ext.create('Ext.grid.Panel', {
            enableColumnHide: false,
            enableColumnMove: false,
            selType: 'rowmodel',
            border: '0',
            region: 'center',
            store: this.gridStore,
            hideHeaders: true,
            overflowX:'hidden',
            columns: [ {
                header: _('condition'),
                dataIndex: 'condition',
                sortable: false,
                hideable: false,
                width:754,
                renderer: this.defMainGridRenderer
            } ],
            tbar: [ this.oButtonNew, this.oButtonEdit,
            '->', this.lblSelectorAnzeige, this.oButtonUp, this.oButtonDown, this.oButtonDelete ]
        });
        this.mainGrid.parent = this;

        this.mainGrid.on({
            selectionchange: function (view, selections, options) {
                if (selections.length > 0) {
                    mThis.oButtonEdit.setDisabled(false);
                    mThis.oButtonDelete.setDisabled(false);
                    var selIndex = -1;
                    for ( var iIndex = 0; iIndex < view.store.getCount(); iIndex++) {
                        if (view.store.getAt(iIndex) == selections[0])
                            selIndex = iIndex;
                    }
                    mThis.oButtonUp.setDisabled(selIndex == 0);
                    mThis.oButtonDown.setDisabled(selIndex == (view.store.getCount() - 1));
                    var iIndex = mThis.mainGrid.store.indexOf(selections[0]);
                    mThis.lblSelectorAnzeige.setText(Ext.String.format2(_('condition {index}/{count}'), {
                        index: '' + (iIndex + 1),
                        count: '' + view.store.getCount()
                    }), false);

                    mThis.setSelectedMainRecord(iIndex, true);
                    mThis.oActualValue = mThis.getSelectedMainRecord();
                    mThis.updatePanelDescription();
                } else {
                    mThis.oButtonEdit.setDisabled(true);
                    mThis.oButtonDelete.setDisabled(true);
                    mThis.oButtonUp.setDisabled(true);
                    mThis.oButtonDown.setDisabled(true);
                    var ct = view.store.getCount();
                    mThis.lblSelectorAnzeige.setText(Ext.String.format2(_('{count} condition', '{count} conditions', ct), { count: '' + ct }), false);

                    mThis.oActualValue = null;
                }
            },
            itemdblclick: function (grid, record, item, index, e) {
                mThis.oActualValue = mThis.getSelectedMainRecord();
                mThis.macPanel.expand(true);
            }
        });

        this.macPanel = Ext.create('Ext.panel.Panel', {
            border: '0',
            region: 'south',
            title: _('show condition editor'),
            collapsible: true,
            collapsed: false,
            titleCollapse: true,
            autoScroll: true,
            animCollapse: true,
            height: 320,
            layout: {
                type: 'hbox'
            },
            listeners: {
                beforecollapse: function (pPanel) {
                    pPanel.setTitle('');
                },
                collapse: function (pPanel) {
                    mThis.macPanel.setTitle('');
                    mThis.oButtonEdit.setDisabled(mThis.getSelectedMainRecord() == null);
                    mThis.oButtonEdit.setText(_('edit condition'));
                    for (var sKey in mThis.grids) {
                        mThis.macPanel.remove(mThis.grids[sKey]);
                    }
                },
                expand: function (pPanel) {
                    mThis.oButtonEdit.setDisabled(mThis.gridStore.getCount()===0);
                    mThis.oButtonEdit.setText(_('show all conditions'));
                    if (mThis.oActualValue != null)
                        mThis.setMacGrid(mThis.oActualValue);
                    else
                        mThis.setMacGrid();
                    mThis.updatePanelDescription();
                }
            },
            scroll: 'horizontal'

        });
        Ext.override(this.macPanel, {
            add: function () {
                this.callParent(arguments);
                if (this.body) {
                    this.body.dom.scrollLeft = 9999;    // scroll to the maximum right
                }
            }
        });

        Ext.apply(this, {
            items: [ this.mainGrid, this.macPanel ]
        });

        this.callParent(arguments);
    },
    reset: function () {
        this.setValue(null);
    },
    setValue: function (values) {
        this.oActualValue = null;
        this.hiddenValues = [];
        this.gridStore.removeAll();

        // if any condition is invalid, remove it; currently - only Apartment Ventilation states when AV not available in the system
        if (values && values.systemState && values.systemState.length && !dss.staticDataModel.globalApplicationsAvaible) {
            var temp = [];
            for (var i = 0; i < values.systemState.length; i++) {
                if (values.systemState[i].name.indexOf('zone.0.group.64.status') === -1) {
                    temp.push(values.systemState[i]);
                }
            }
            values.systemState = temp.length ? temp : null;
            var anyConditionsLeft = Object.keys(values).reduce(function (anyLeft, conditionType) {
                return anyLeft || values[conditionType] !== null && values[conditionType].length;
            }, false);
            if (!anyConditionsLeft) {
                values = null;
            }
        }

        if (values == null) {
            if (this.macPanel.collapsed) {
                this.macPanel.expand(false);
            } else {
                this.setMacGrid();
            }
            this.oButtonEdit.setText(_('show all conditions'));
            this.oButtonEdit.setDisabled(true);
            this.lblSelectorAnzeige.setText(_('no condition'), false);
        } else {
            var lvl1Store = Ext.data.StoreManager.lookup('conditionConfigurator_l1Store');
            if (values.date != null) {
                for (var kIndex = 0; kIndex < values.date.length; kIndex++) {
                    var oObject = {condition:{type:'date', value:values.date[kIndex]}};
                    lvl1Store.each(function (record) {
                        if (record.get('type') == 10)
                            oObject.pos = record.get('distiguishID');
                    });
                    if (oObject.pos != null)
                        this.gridStore.add(oObject);
                    else
                        this.hiddenValues.push(oObject);

                }
            }
            if (values.enabled != null) {
                var oObject = {condition:{type:'enabler', value:values.enabled}};

                lvl1Store.each(function (record) {
                    if (record.get('type') == 0)
                        oObject.pos = record.get('distiguishID');
                });
                if (oObject.pos != null)
                    this.gridStore.add(oObject);
                else
                    this.hiddenValues.push(oObject);
            }
            if (values.timeframe != null) {
                for (var kIndex = 0; kIndex < values.timeframe.length; kIndex++) {
                    var oObject = {condition:{type:'timeframe', value:values.timeframe[kIndex]}};
                    lvl1Store.each(function (record) {
                        if (record.get('type') == 2)
                            oObject.pos = record.get('distiguishID');
                    });
                    if (oObject.pos != null)
                        this.gridStore.add(oObject);
                    else
                        this.hiddenValues.push(oObject);
                }
            }
            if (values.weekdays != null) {
                var oObject = {condition:{type:'weekdays', value:values.weekdays}};
                lvl1Store.each(function (record) {
                    if (record.get('type') == 1)
                        oObject.pos = record.get('distiguishID');
                });
                if (oObject.pos != null)
                    this.gridStore.add(oObject);
                else
                    this.hiddenValues.push(oObject);
            }
            if (values.systemState != null) {
                for (var kIndex = 0; kIndex < values.systemState.length; kIndex++) {
                    var oObject = {condition:{type:'state', value:values.systemState[kIndex].value, name:values.systemState[kIndex].name}};

                    var fAdded = false;
                    var oArray = values.systemState[kIndex].name.split('.');
                    if (oArray.length == 3) {
                        if (oArray[0] == 'dev')
                        {
                            var oDevice = dss.staticDataModel.getDevice(oArray[1]);
                            if (oDevice != null) {
                                var typeIndex = parseInt(oArray[2], 10);
                                var type = 255;
                                if (oDevice.binaryInputs) {
                                    for (var lIndex = 0; lIndex < oDevice.binaryInputs.length; lIndex++)
                                        if (oDevice.binaryInputs[lIndex].index == typeIndex)
                                            type = oDevice.binaryInputs[lIndex].type;
                                }

                                lvl1Store.each(function (record) {
                                    if (oDevice.binaryInputs.length > 0 && record.get('type') == 6) {
                                        oObject.pos = record.get('distiguishID');
                                    }
                                    if (oDevice.binaryInputs.length == 0 && record.get('type') == 11) {
                                        oObject.pos = record.get('distiguishID');
                                    }
                                });
                                if (oObject.pos != null)
                                    this.gridStore.add(oObject);
                                else
                                    this.hiddenValues.push(oObject);
                                fAdded = true;
                            }
                        }
                        if (oArray[0] == 'zone') {
                            lvl1Store.each(function (record) {
                                if (record.get('type') == 3) {
                                    oObject.pos = record.get('distiguishID');
                                }
                            });
                            if (oObject.pos != null)
                                this.gridStore.add(oObject);
                            else
                                this.hiddenValues.push(oObject);
                            fAdded = true;
                        }
                    } else if (oArray.length === 6) {
                        if (oArray[0] === 'zone') {
                            if (oArray[1] == 0 && oArray[2] === 'group' && oArray[3] == 64) {
                                lvl1Store.each(function (record) {
                                    if (record.get('type') == 12) {
                                        oObject.pos = record.get('distiguishID');
                                    }
                                });
                            } else {
                                lvl1Store.each(function (record) {
                                    if (record.get('type') == 3) {
                                        oObject.pos = record.get('distiguishID');
                                    }
                                });
                            }
                            if (oObject.pos != null)
                                this.gridStore.add(oObject);
                            else
                                this.hiddenValues.push(oObject);
                            fAdded = true;
                        }
                    }
                    if (!fAdded) {
                        lvl1Store.each(function (record) {
                            if (record.get('type') == 4) {
                                var param = record.get('parameter');
                                if (param) {
                                    for (var lIndex = 0; lIndex < param.length; lIndex++)
                                        if (param[lIndex].name == values.systemState[kIndex].name)
                                            if (param[lIndex].value == values.systemState[kIndex].value)
                                                oObject.pos = record.get('distiguishID');
                                }
                            }
                            if (record.get('type') == 5) {
                                var param = record.get('parameter');
                                if (param) {
                                    for (var lIndex = 0; lIndex < param.length; lIndex++)
                                        if (param[lIndex].name == values.systemState[kIndex].name)
                                            if (param[lIndex].value == values.systemState[kIndex].value) {
                                                oObject.pos = record.get('distiguishID');
                                                oObject.condition.group = param[lIndex].group;
                                            }
                                }
                            }
                            if (record.get('type') == 12) {
                                if (values.systemState[kIndex].name.indexOf('zone.0.group')>-1) {
                                    oObject.pos = record.get('distiguishID');
                                }
                            }
                        });
                        if (oObject.pos != null)
                            this.gridStore.add(oObject);
                        else
                            this.hiddenValues.push(oObject);
                    }
                }
            }
            if (values.zoneState != null) {
                for (var kIndex = 0; kIndex < values.zoneState.length; kIndex++) {
                    var oObject = {condition:{type:'zonestate', value:values.zoneState[kIndex]}};
                    lvl1Store.each(function (record) {
                        if ((values.zoneState[kIndex].zone == 0) && (values.zoneState[kIndex].group>63)) {
                            if (record.get('type') == 12)
                                oObject.pos = record.get('distiguishID');
                        } else
                            if (record.get('type') == 3)
                                oObject.pos = record.get('distiguishID');
                    });
                    if (oObject.pos != null)
                        this.gridStore.add(oObject);
                    else
                        this.hiddenValues.push(oObject);
                }
            }

            if (values.addonStates != null) {
                for (var kIndex = 0; kIndex < values.addonStates.length; kIndex++) {
                    var oObject = {
                            condition:{
                                type:'addon-state',
                                stateValue:values.addonStates[kIndex].value,
                                stateName:values.addonStates[kIndex].name,
                                addonid:values.addonStates[kIndex].scriptID
                            }
                    };

                    if (values.addonStates[kIndex].scriptID === 'system-addon-user-defined-states') {
                        for (var iIndex = 0; iIndex < Ext.data.StoreManager.lookup('conditionConfigurator_userStateStore').count(); iIndex++) {
                            var oRecord = Ext.data.StoreManager.lookup('conditionConfigurator_userStateStore').getAt(iIndex);
                            if (('uds_' + oRecord.get('_id').split('_')[0]) == values.addonStates[kIndex].name)
                                if (((oRecord.get('_id').split('_')[1] == 'active')?1:2) == values.addonStates[kIndex].value)
                                    oObject.condition.name = oRecord.get('name');
                        }

                        lvl1Store.each(function (record) {
                            if (record.get('type') == 7) {
                                oObject.pos = record.get('distiguishID');
                            }
                        });
                    }

                    if (values.addonStates[kIndex].scriptID === 'system-addon-heating-controller') {
                        var climateStore = Ext.data.StoreManager.lookup('conditionConfigurator_climateStore');
                        for (var i = 0, len = climateStore.count(); i < len; i++) {
                            var oRecord = climateStore.getAt(i);
                            if (oObject.condition.stateName === 'apartment_controller') {
                                switch (oRecord.get('name')) {
                                    case 'off':
                                        oObject.condition.name = 'grid_climate_off';   // 'only if heating is inactive';
                                        break;
                                    case 'heating':
                                        oObject.condition.name = 'grid_climate_heating';   // 'only if heating is active';
                                        break;
                                    case 'cooling':
                                        oObject.condition.name = 'grid_climate_cooling';   // 'only if cooling is active';
                                        break;
                                }
                            }
                        }

                        lvl1Store.each(function (record) {
                            if (record.get('type') == 13) {
                                oObject.pos = record.get('distiguishID');
                            }
                        });
                    }

                    if (oObject.pos != null)
                        this.gridStore.add(oObject);
                    else
                        this.hiddenValues.push(oObject);

                }
            }
            if (this.gridStore.getCount() == 0) {
                if (this.macPanel.collapsed)
                    this.macPanel.expand(false);
                else
                    this.setMacGrid();
            } else {
                this.deselectMainGrid();
                this.macPanel.collapse(false);
            }
            this.oButtonEdit.setDisabled(this.gridStore.getCount()===0);
        }
    },
    hiddenValues: null,
    isValid: function () {
        return true;
    },
    getValue: function () {
        var localValues= {
                enabled: null, // singular value
                weekdays: null, // singularvalue
                timeframe: null, // array ....
                zoneState: null,
                systemState: null,
                addonState: null
            };
        this.gridStore.each(function (oRecord) {
            var oAction = oRecord.get('condition');
            switch (oAction.type) {
                case 'timeframe':
                    if (localValues.timeframe == null)
                        localValues.timeframe = [];
                    localValues.timeframe.push(oAction.value);
                    break;
                case 'zonestate':
                    if (localValues.zoneState == null)
                        localValues.zoneState = [];
                    localValues.zoneState.push(oAction.value);
                    break;
                case 'state':
                    if (localValues.systemState == null)
                        localValues.systemState = [];
                    localValues.systemState.push({name:oAction.name, value:oAction.value});
                    break;
                case 'addonState':
                    if (localValues.addonStates == null)
                        localValues.addonStates = {};
                    if (localValues.addonStates[oAction.addonid] == null)
                        localValues.addonStates[oAction.addonid] = {};
                    localValues.addonStates[oAction.addonid].push({name:oAction.name, value:oAction.value, addonState:oAction.addonid});
                    break;
                case 'enabler':
                    localValues.enabled = oAction.value;
                    break;
                case 'weekdays':
                    localValues.weekdays = oAction.value;
                    break;
                case 'addon-state':
                    if (localValues.addonStates == null)
                        localValues.addonStates = {};
                    if (localValues.addonStates[oAction.addonid] == null)
                        localValues.addonStates[oAction.addonid] = [];
                    localValues.addonStates[oAction.addonid].push({name:oAction.stateName, value:oAction.stateValue, addonState:oAction.addonid});
                break;
                case 'date':
                    if (localValues.date == null)
                        localValues.date = [];
                    localValues.date.push(oAction.value);
                break;
            }
        });
        if (this.hiddenValues) {
            for (var iIndex = 0; iIndex < this.hiddenValues.length; iIndex++) {
                switch (this.hiddenValues[iIndex].condition.type) {
                    case 'timeframe':
                        if (localValues.timeframe == null)
                            localValues.timeframe = [];
                        localValues.timeframe.push(this.hiddenValues[iIndex].condition.value);
                        break;
                    case 'zonestate':
                        if (localValues.zoneState == null)
                            localValues.zoneState = [];
                        localValues.zoneState.push(this.hiddenValues[iIndex].condition.value);
                        break;
                    case 'state':
                        if (localValues.systemState == null)
                            localValues.systemState = [];
                        localValues.systemState.push({name:this.hiddenValues[iIndex].condition.name, value:this.hiddenValues[iIndex].condition.value});
                        break;
                    case 'enabler':
                        localValues.enabled = this.hiddenValues[iIndex].condition.value;
                        break;
                    case 'weekdays':
                        localValues.weekdays = this.hiddenValues[iIndex].condition.value;
                        break;
                    case 'addon-state':
                        if (localValues.addonStates == null)
                            localValues.addonStates = {};
                        if (localValues.addonStates[this.hiddenValues[iIndex].condition.addonid] == null)
                            localValues.addonStates[this.hiddenValues[iIndex].condition.addonid] = [];
                        localValues.addonStates[this.hiddenValues[iIndex].condition.addonid].push({name:this.hiddenValues[iIndex].condition.stateName, value:this.hiddenValues[iIndex].condition.stateValue, addonState:this.hiddenValues[iIndex].condition.addonid});
                    break;
                    case 'dateframe':
                        if (localValues.date == null)
                            localValues.date = [];
                        localValues.date.push(this.hiddenValues[iIndex].condition.value);
                    break;
                }
            }
        }
        return localValues;
    }
});
