Ext.define('DSS.component.MessageEditor', {
    extend : 'DSS.component.Base',
    mixins : {
        observable : 'Ext.util.Observable'
    },
    height : 340,
    txtReceiver : null,
    txtSubject : null,
    txtBody : null,
    padding : "0 0 0 10",
    constructor : function(config) {
    	if (config.labelWidth==null)
    		config.labelWidth=84;
        this.txtReceiver = Ext.create('Ext.form.field.Text', {
            fieldLabel : _('Receiver'),
            allowBlank : false,
            width : (750-config.labelWidth),
            labelWidth : config.labelWidth,
            regex : /\b[A-Z0-9._%-]+@[A-Z0-9.-]+\.[A-Z]+\b/i,
            regexText : _('wrong format'),
            blankText : _('Receiver must be provided'),
            invalidText : _('Receiver must be provided')
        });

        this.txtSubject = Ext.create('DSS.component.Textedit', {
            fieldLabel : _('Subject'),
            width : (750-config.labelWidth),
            labelWidth : config.labelWidth,
            blankText :  _('Subject must be provided'),
            invalidText : _('Subject must be provided')
        });

        this.txtBody = Ext.create('DSS.component.TextAreaedit', {
            fieldLabel : _('Body'),
            width : (750-config.labelWidth),
            height : 260,
            labelWidth : config.labelWidth,
            blankText : _('Body must be provided'),
            invalidText : _('Body must be provided')
        });
        var oControlGroup = Ext.create('Ext.form.FieldSet', {
            title : config.title,
            width : "100%",
            items : [ this.txtReceiver, this.txtSubject, this.txtBody ]
        });

        Ext.apply(this, {
            items : [ this.txtReceiver, this.txtSubject, this.txtBody ]
        });
        this.callParent(arguments);
    },
    reset : function() {
        this.txtBody.reset();
        this.txtSubject.reset();
        this.txtReceiver.reset();
    },
    setValue : function(value) {
        // messageType:"email",
        this.txtBody.setValue(value.message);
        this.txtSubject.setValue(value.subject);
        this.txtReceiver.setValue(value.receiver);
    },
    isValid : function() {
        var fValid=false;
        this.txtSubject.markInvalid();
        this.txtBody.markInvalid();
        
        if (this.txtSubject.getValue()!=null)
            if (this.txtSubject.getValue().length>0)
                fValid=true;
        
        if (this.txtBody.getValue()!=null)
            if (this.txtBody.getValue().length>0)
                fValid=true;
        
        if (!fValid) {
            this.txtSubject.markInvalid(_('Either Subject or Body must be provided'));
            this.txtBody.markInvalid(_('Either Subject or Body must be provided'));
        }
        
        if (!this.txtReceiver.isValid()) {
            fValid=false;
        }
        return fValid;
    },
    getValue : function() {
        var oObject = {};
        oObject.messageType = 'email';
        oObject.receiver = this.txtReceiver.getValue();
        oObject.subject = this.txtSubject.getValue();
        if (oObject.subject==null)
            oObject.subject="";
        oObject.message = this.txtBody.getValue();
        if (oObject.message==null)
            oObject.message="";
        return oObject;
    }
});
