/* globals Ext, _ */

Ext.define('DSS.component.Wizard', {
    extend: 'Ext.window.Window',

    /** Textfield holding the event name */
    editControls: {},
    buttonStart: null,
    buttonNext: null,
    buttonFinish: null,
    buttonCancel: null,
    buttonBack: null,
    innerHeight: 275,
    finishFunc: null,
    makeControl: function (subConfig) {
        var oControl;
        switch (subConfig.type) {
            case 'hidden': {
                var oPseudoControl = {
                    value: null
                };
                oPseudoControl.setValue = function (oValue) {
                    this.value = oValue;
                };
                oPseudoControl.reset = function () {
                    this.value = null;
                };
                oPseudoControl.getValue = function () {
                    return this.value;
                };
                oPseudoControl.isValid = function () {
                    return true;
                };
                this.editControls[subConfig.id] = oPseudoControl;
                return null;
            }
            case 'custom': {
                oControl = subConfig.control;
                this.editControls[subConfig.id] = oControl;
                oControl.dssEditor = this;
                return oControl;
            }
            case 'deviceSelection': {
                oControl = Ext.create('DSS.component.DeviceSelector', {
                    filterFunc: subConfig.filterFunc,
                    specialDeviceRenderer: subConfig.specialDeviceRenderer,
                    myID: subConfig.id,
                    bMultiselect: subConfig.multiselect,
                    width: 614,
                    height: innerHeight,
                    margin: subConfig.margin
                });
                this.editControls[subConfig.id] = oControl;
                oControl.dssEditor = this;
                return oControl;
            }
        }
        return Ext.create('Ext.panel.Panel', {
            border: '0'
        });
    },
    oStartPage: null,
    oPages: [],
    value: {},
    actIndex: 0,
    pageDefinition: null,
    startLongTitle: null,
    validatePage: function () {
        var fNext = true;
        var pageDef = this.pageDefinition[this.actIndex];
        if (pageDef && pageDef.controls) {
            for (var iIndex = 0; iIndex < pageDef.controls.length; iIndex++) {
                var id = pageDef.controls[iIndex].id;
                if (!this.editControls[id].isValid()) {
                    fNext = false;
                }
            }
        }
        this.buttonNext.setDisabled(!fNext);
        this.buttonFinish.setDisabled(!fNext);
    },
    takeOverValue: function () {
        var pageDef = this.pageDefinition[this.actIndex];
        if (pageDef && pageDef.controls) {
            for (var iIndex = 0; iIndex < pageDef.controls.length; iIndex++) {
                var id = pageDef.controls[iIndex].id;
                this.value[id] = this.editControls[id].getValue();
            }
        }
    },
    constructor: function (config) {
        config.closeAction = 'hide';
        config.modal = true;
        if (config.height == null) {
            config.height = 500;
        }
        this.innerHeight = config.height - 125;
        if (config.width == null) {
            config.width = 800;
        }
        config.layout = 'fit';
        config.appName = '';
        config.resizable = false;

        this.editControls = {};
        this.oPages = [];

        this.addEvents({
            change: true
        });

        var me = this;
        var sCancelText = _('cancel');
        if (config.cancelTitle != null) {
            sCancelText = config.cancelTitle;
        }
        var sFinishText = _('finish');
        if (config.finishTitle != null) {
            sFinishText = config.finishTitle;
        }
        var sNextText = _('next');
        if (config.nextTitle != null) {
            sNextText = config.nextTitle;
        }
        var sStartText = _('start');
        if (config.startTitle != null) {
            sStartText = config.startTitle;
        }
        var sBackText = _('back');
        if (config.backTitle != null) {
            sBackText = config.backTitle;
        }
        if (config.startLongTitle != null) {
            this.startLongTitle = config.startLongTitle;
        }
        if (config.finishFunc != null) {
            this.finishFunc = config.finishFunc;
        }

        if (!config.hideBack) {
            me.buttonBack = Ext.create('Ext.Button', {
                text: sBackText,
                handler: function () {
                    me.previousStep();
                }
            });
        }
        if (!config.hideCancel) {
            me.buttonCancel = Ext.create('Ext.Button', {
                text: sCancelText,
                handler: function () {
                    me.fireEvent('cancel');
                    me.hide();
                }
            });
        }
        if (!config.hideFinish) {
            me.buttonFinish = Ext.create('Ext.Button', {
                text: sFinishText,
                handler: function () {
                    me.takeOverValue();
                    me.finishFunc(me.value);
                    me.hide();
                }
            });
        }
        if (!config.hideNext) {
            me.buttonNext = Ext.create('Ext.Button', {
                text: sNextText,
                handler: function () {
                    me.nextStep();
                }
            });
        }
        if (!config.hideStart) {
            me.buttonStart = Ext.create('Ext.Button', {
                text: sStartText,
                handler: function () {
                    me.setStep(0);
                    var nextRecord = me.leftStore.findRecord('_id', me.actIndex + 1, 0, false, true, true);
                    if (nextRecord != null) {
                        me.leftPanel.getSelectionModel().select(nextRecord);
                    } else {
                        me.leftPanel.getSelectionModel().deselectAll();
                    }
                }
            });
        }
        this.addEvents({
            cancel: true,
            finish: true
        });
        this.leftStore = Ext.create('Ext.data.Store', {
            fields: ['name', '_id', 'value', 'definition'],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        if (config.startDescription != null) {
            this.oStartPage = Ext.create('Ext.panel.Panel', {
                border: 0,
                width: '100%',
                height: config.height - 125,
                html: config.startDescription
            });
        }
        if (config.pages != null) {
            this.pageDefinition = config.pages;
            this.leftStore.add({
                _id: -3,
                name: '',
                value: null
            });
            for (var iIndex = 0; iIndex < config.pages.length; iIndex++) {
                var pageDef = config.pages[iIndex];
                this.leftStore.add({
                    definition: pageDef,
                    name: pageDef.shortTitle,
                    _id: iIndex + 1,
                    value: null
                });
                var items = [ {
                    xtype: 'panel',
                    html: pageDef.longDescription,
                    border: 0
                } ];
                if (pageDef.controls) {
                    for (var kIndex = 0; kIndex < pageDef.controls.length; kIndex++) {
                        var oControl = this.makeControl(pageDef.controls[kIndex]);
                        oControl.on('change', function () {
                            me.validatePage();
                        });
                        items.push(oControl);
                    }
                }
                var page = Ext.create('Ext.panel.Panel', {
                    border: 0,
                    width: '100%',
                    height: config.height - 125,
                    items: items,
                    hidden: true
                });
                this.oPages.push(page);
            }
            var fSkipLeftFinish = false;
            if (config.hideLeftFinish != null) {
                fSkipLeftFinish = config.hideLeftFinish;
            }
            if (!fSkipLeftFinish) {
                this.leftStore.add({
                    _id: -2,
                    name: '',
                    value: null
                });
                this.leftStore.add({
                    _id: -1,
                    name: 'Finish',
                    value: null
                });
            }
        }

        this.displayBottomText = Ext.create('Ext.form.field.Display', {
            border: 0,
            value: '',
            style: {
                fontSize: '11px'
            },
            padding: '0 0 2 5',
            flex: 1
        });
        var oButtons = [ this.displayBottomText ];

        if (me.buttonStart != null) {
            oButtons.push(me.buttonStart);
        }
        if (me.buttonBack != null) {
            oButtons.push(me.buttonBack);
        }
        if (me.buttonNext != null) {
            oButtons.push(me.buttonNext);
        }
        if (me.buttonFinish != null) {
            oButtons.push(me.buttonFinish);
        }
        if (me.buttonCancel != null) {
            oButtons.push(me.buttonCancel);
        }

        var insideItems = Ext.create('Ext.panel.Panel', {
            bodyPadding: 10,
            border: '1px',
            items: this.oPages
        });
        insideItems.add(this.oStartPage);
        insideItems.updateLayout();

        this.leftPanel = Ext.create('Ext.grid.Panel', {
            store: this.leftStore,
            border: 0,
            hideHeaders: true,
            columnLines: false,
            rowLines: false,
            viewConfig: {
                stripeRows: false
            },
            columns: [ {
                dataIndex: name,
                renderer: function (val, metadata, record) { // , rowIdx, colIdx, store, view
                    if (record.get('_id') == -1) {
                        return record.get('name');
                    }
                    if (record.get('_id') == -2) {
                        return '';
                    }
                    if (record.get('_id') == -3) {
                        return '<div style="height:40px;"></div>';
                    }
                    return record.get('_id') + '. ' + record.get('name');
                }
            } ],
            width: 100
        });

        this.leftPanel.on('beforeselect', function (grid, record) {
            var fResult = true;
            if (record.get('_id') < 0) {
                fResult = false;
            }
            if (record.get('definition')) {
                if (record.get('definition').constrainFunc) {
                    fResult = record.get('definition').constrainFunc(record.get('_id'), me.value);
                }
            }
            if (!fResult) {
                if (me.actIndex >= 0) {
                    var nextRecord = me.leftStore.findRecord('_id', me.actIndex + 1, 0, false, true, true);
                    if (nextRecord != null) {
                        me.leftPanel.getSelectionModel().select(nextRecord);
                    } else {
                        me.leftPanel.getSelectionModel().deselectAll();
                    }
                }
            }
            return fResult;
        });

        this.leftPanel.on('selectionchange', function (view, selections) {
            if (selections.length === 1) {
                var index = selections[0].get('_id');
                me.setStep(index - 1);
            }
        });

        this.topIndicator = Ext.create('Ext.form.Label', {
            text: 'Wizzard!'
        });
        var topPanel = Ext.create('Ext.panel.Panel', {
            border: false,
            items: [ this.topIndicator ],
            padding: '10 0 0 10',
            height: 40
        });

        Ext.apply(me, {
            items: [ {
                xtype: 'panel',
                items: insideItems,
                border: 0,
                dockedItems: [ {
                    xtype: 'toolbar',
                    dock: 'bottom',
                    ui: 'footer',
                    defaults: { minWidth: 75 },
                    items: oButtons
                }, {
                    dock: 'left',
                    items: this.leftPanel
                }, {
                    dock: 'top',
                    items: topPanel
                } ]
            } ]
        });
        this.callParent(arguments);
    },
    nextStep: function () {
        this.takeOverValue();
        this.setStep(this.actIndex + 1);

        var nextRecord = this.leftStore.findRecord('_id', this.actIndex + 1, 0, false, true, true);
        if (nextRecord != null) {
            this.leftPanel.getSelectionModel().select(nextRecord);
        } else {
            this.leftPanel.getSelectionModel().deselectAll();
        }
    },
    previousStep: function () {
        this.setStep(this.actIndex - 1);

        var nextRecord = this.leftStore.findRecord('_id', this.actIndex + 1, 0, false, true, true);
        if (nextRecord != null) {
            this.leftPanel.getSelectionModel().select(nextRecord);
        } else {
            this.leftPanel.getSelectionModel().deselectAll();
        }
    },
    startWizard: function (oStartValues) {
        for (var sKey in this.editControls) {
            this.editControls[sKey].reset();
        }
        this.value = {};
        if (oStartValues != null) {
            this.value = oStartValues;
        }

        this.show();
        if (this.oStartPage != null) {
            this.setStep(-1);
        } else {
            this.setStep(0);
        }
    },
    setStep: function (selIndex) {
        var iIndex;
        this.actIndex = selIndex;
        if (this.oStartPage) {
            this.oStartPage.setVisible(selIndex === -1);
        }
        if (this.oPages) {
            for (iIndex = 0; iIndex < this.oPages.length; iIndex++) {
                this.oPages[iIndex].setVisible(iIndex === selIndex);
            }
        }

        this.buttonStart.setVisible(selIndex === -1);
        this.buttonNext.setVisible(selIndex < this.oPages.length - 1 && selIndex >= 0);
        this.buttonFinish.setVisible(selIndex === this.oPages.length - 1);
        this.buttonBack.setVisible(selIndex > 0);

        if (selIndex === -1) {
            this.topIndicator.setText(this.startLongTitle);
        } else {
            this.topIndicator.setText(this.pageDefinition[selIndex].longTitle);
        }
        this.validatePage();
        if (this.actIndex === -1) {
            return;
        }
        var pageDef = this.pageDefinition[this.actIndex];
        if (pageDef && pageDef.controls) {
            for (iIndex = 0; iIndex < pageDef.controls.length; iIndex++) {
                var id = pageDef.controls[iIndex].id;
                this.editControls[id].setValue(this.value[id], this.value);
            }
            if (pageDef.hideBack != null) {
                if (pageDef.hideBack) {
                    this.buttonBack.setVisible(false);
                }
            }
        }
    }
});
