// 'use strict';
/* jshint esversion: 6 */
/* jshint multistr: true */
/* jshint eqnull: true */
/* jshint eqeqeq: false */
/* globals Ext, dss, _, capitalizeFirstLetter, addDataIdToElement, joinAndCapitalize */

var SHOW_ALL_SCENES_ID = -3;

var upperCaseSort = function (o1, o2) {
    return o1.name.toUpperCase() < o2.name.toUpperCase() ? -1 : 1;
};

Ext.define('DSS.component.TriggerConfigurator', {
    extend: 'DSS.component.Base',
    mixins: {
        observable: 'Ext.util.Observable'
    },
    height: '100%',
    iDefaultWidth: 150,
    makeClickListeners: function () {
        var me = this;
        return {
            itemdblclick: function () {
                if (me.isValid()) {
                    me.fireEvent('valueDoubleClicked', me);
                }
            },
            itemclick: {
                fn: function () {
                    me.fireEvent('valueClicked', me);
                },
                // single: true,
                buffer: 300
            }
        };
    },
    setHeight: function (oHeight) {
        this.macPanel.setHeight(oHeight);
        this.iDefaultHeight = oHeight - 19;   // minus horizontal scroll bar height (always visible)
        for (var oKey in this.grids) {
            if (Object.prototype.hasOwnProperty.call(this.grids, oKey)) {
                this.grids[oKey].setHeight(this.iDefaultHeight);
            }
        }
        this.callParent(arguments);
    },
    defMacGridRenderer: function (val, metadata, record, colIdx, store, view) {            // jshint ignore:line
        const name = record.get('name');
        if (name === '-') {
            return '<table style="height:10px;width:100%;"><tr style="height:10px;"><td width="100%"><hr></td></tr></table>';
        }

        const icon = record.get('icon');
        const groupId = record.get('group');
        const dSID = record.get('dSID');
        const dSUID = record.get('dSUID');
        let id = record.get('_id');
        if (id == null) {
            id = record.get('id');
        }
        const type = record.get('type');
        const stateId = record.get('stateid');
        const value = record.get('value');
        const valueAndState = record.get('valueandstate');
        let fHasIcon = false;
        if (icon != null) {
            fHasIcon = true;
        }
        // definition requires '>' button,
        // or it's a group
        let fHasNext = Boolean(record.get('next')) || groupId != null;
        if (dSID != null || dSUID != null) { // '>' button should also be shown for a device
            fHasNext = true;
            const sTooltip = dss.staticDataModel.getTooltipForDevice(dss.staticDataModel.getDevice(dSUID));
            if (sTooltip) {
                metadata.tdAttr = 'data-qtip="' + sTooltip + '"';
            }
        }
        const tooltip = record.get('tooltip');
        if (tooltip) {
            metadata.tdAttr = 'data-qtip="' + tooltip + '"';
        }

        let fIsCheckable = Boolean(record.get('checkable'));
        if (fIsCheckable) {
            fHasNext = false; // for checkboxes, no '>' button is needed
            fHasIcon = false; // checkbox items need no icon
        }

        const fIsPassive = Boolean(record.get('passive'));
        if (fIsPassive) {
            fHasNext = false;        // column disabled == no '>' button
            fIsCheckable = false;    // column disabled == cannot be checked
        }

        let sReturn = '';
        let iColspaceCounter = 3;
        if (fHasIcon) {
            iColspaceCounter--;
            sReturn += '<td><img src="' + icon + '" style="margin-right: 5px;"></td>';
        }
        if (fIsCheckable) {
            iColspaceCounter--;
            if (record.get('checked')) {
                sReturn += '<td style="padding-right: 5px;">' +
                    '<div style="width:13px;height:13px;background-image:url(' + dss.staticDataModel.baseImagePath + 'dss/checkbox.gif); background-position: 0px -13px;" />' +
                '</td>';
            } else {
                sReturn += '<td style="padding-right: 5px;">' +
                    '<div style="width:13px;height:13px;background-image:url(' + dss.staticDataModel.baseImagePath + 'dss/checkbox.gif);" />' +
                '</td>';
            }
        }
        if (fHasNext) {
            iColspaceCounter--;
        }
        if (fIsPassive) {
            sReturn += '<td style="color:#888888;" colspan="' + iColspaceCounter + '" width="100%">' + name + '</td>';
        } else {
            sReturn += '<td colspan="' + iColspaceCounter + '" width="100%">' + name + '</td>';
        }
        if (fHasNext) {
            sReturn += '<td>&gt;</td>';
        }
        return '<table style="height:16px;width:100%;"' +
            (dSID ? ' data-dsid="' + dSID + '"' : '') +
            (dSUID ? ' data-dsuid="' + dSUID + '"' : '') +
            (groupId != null && groupId !== '' ? ' data-group-id="' + groupId + '"' : '') +
            (id != null && id !== '' ? ' data-id="' + id + '"' : '') +
            (type != null && type !== '' ? ' data-type="' + type + '"' : '') +
            (stateId != null && stateId !== '' ? ' data-state-id="' + stateId + '"' : '') +
            (value != null && value !== '' ? ' data-value="' + value + '"' : '') +
            (valueAndState ? ' data-state-value="' + valueAndState + '"' : '') +
            (name ? ' data-name="' + joinAndCapitalize(name) + '"' : '') +
            '><tr>' + sReturn + '</tr></table>';
    },
    initStores: function () {   // fShowUserstates
        if (Ext.data.StoreManager.lookup('triggerConfigurator_l1Store') == null) {
            const Aktionen = [ {
                name: _('activity in room'),
                _id: 0,
                next: true,
                icon: null,
                passive: dss.staticDataModel.getZoneArray().length === 1
            }, {
                name: _('activity on device'),
                _id: 1,
                next: true,
                icon: null,
                passive: dss.staticDataModel.getZoneArray().length === 1
            }, {
                name: _('automatic devices'),
                _id: 7,
                next: true,
                icon: null,
                passive: !dss.staticDataModel.getAKMDeviceArray().length
            } ];

            Aktionen.push({
                name: _('consumptionmessage'),
                _id: 3,
                next: true,
                icon: null,
                passive: !dss.staticDataModel.getSensorDeviceArray().length
            });
            Aktionen.push({
                name: _('zone trigger'),
                _id: 9,
                next: true,
                icon: null
            });

            if (dss.staticDataModel.globalApplicationsAvaible) {
                Aktionen.push({
                    name: _('Global Application states'),
                    _id: 12,
                    next: true,
                    icon: null
                });
            }

            Aktionen.push({
                name: '-',
                _id: -1,
                passive: true,
                next: false,
                icon: null
            });

            Aktionen.push({
                name: _('user def. actions'),
                _id: 2,
                next: true,
                icon: null,
                passive: !dss.staticDataModel.getHLE().length
            });
            if (dss.staticDataModel.getUserStates().length) {
                Aktionen.push({
                    name: _('user states'),
                    _id: 8,
                    next: true,
                    icon: null,
                    passive: false
                });
            }


            Aktionen.push({
                name: '-',
                _id: -1,
                passive: true,
                next: false,
                icon: null
            });

            Aktionen.push({
                name: _('access trigger'),
                _id: 4,
                next: true,
                icon: null
            });
            Aktionen.push({
                name: _('weather trigger'),
                _id: 5,
                next: true,
                icon: null
            });
            Aktionen.push({
                name: _('security trigger'),
                _id: 6,
                next: true,
                icon: null
            });
            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_l1Store',
                fields: [ '_id', 'name', 'next', 'icon', 'passive' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            Ext.data.StoreManager.lookup('triggerConfigurator_l1Store').add(Aktionen);
        }

        if (Ext.data.StoreManager.lookup('triggerConfigurator_userStateStore') == null) {

            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_userStateStore',
                fields: [ '_id', 'name', 'next', 'icon', 'type' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });

            var oUserStates = dss.staticDataModel.getUserStates();
            if (oUserStates) {
                for (let iIndex = 0; iIndex < oUserStates.length; iIndex++) {
                    const Aktionen = [ {
                        name: oUserStates[iIndex].displayName,
                        _id: oUserStates[iIndex].name,
                        next: true,
                        icon: null,
                        type: oUserStates[iIndex].type
                    } ];
                    Ext.data.StoreManager.lookup('triggerConfigurator_userStateStore').add(Aktionen);
                }
            }

        }
        if (Ext.data.StoreManager.lookup('triggerConfigurator_zoneZoneStore') == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_zoneZoneStore',
                fields: [ 'ZoneID', 'name' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            Ext.data.StoreManager.lookup('triggerConfigurator_zoneZoneStore').add(dss.staticDataModel.getZoneArray());
        }
        if (Ext.data.StoreManager.lookup('triggerConfigurator_deviceZoneStore') == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_deviceZoneStore',
                fields: [ 'ZoneID', 'name' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            var zoneArray = dss.staticDataModel.getZoneArray();
            if (zoneArray) {
                for (let iIndex = 0; iIndex < zoneArray.length; iIndex++) {
                    if (this.getDeviceStore(zoneArray[iIndex].ZoneID).count() > 0) {
                        Ext.data.StoreManager.lookup('triggerConfigurator_deviceZoneStore').add(zoneArray[iIndex]);
                    }
                }
            }
        }

        if (Ext.data.StoreManager.lookup('triggerConfigurator_zoneSensorStore') == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_zoneSensorStore',
                fields: [ 'ZoneID', 'name' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            const oArray = dss.staticDataModel.getZoneArray();
            const deviceArray = dss.staticDataModel.getSensorDeviceArray();
            if (oArray) {
                for (let iIndex = 0; iIndex < oArray.length; iIndex++) {
                    let fAdd = false;
                    if (deviceArray) {
                        for (let jIndex = 0; jIndex < deviceArray.length; jIndex++) {
                            if (deviceArray[jIndex].ZoneID == oArray[iIndex].ZoneID) {
                                fAdd = true;
                                break;
                            }
                        }
                    }
                    if (fAdd) {
                        Ext.data.StoreManager.lookup('triggerConfigurator_zoneSensorStore').add(oArray[iIndex]);
                    }
                }
            }
        }
        if (Ext.data.StoreManager.lookup('triggerConfigurator_zoneAKMStore') == null) {

            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_zoneAKMStore',
                fields: [ 'ZoneID', 'name' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            const oArray = dss.staticDataModel.getZoneArray();
            const deviceArray = dss.staticDataModel.getAKMDeviceArray();
            if (oArray) {
                for (let iIndex = 0; iIndex < oArray.length; iIndex++) {
                    let fAdd = false;
                    if (deviceArray) {
                        for (let jIndex = 0; jIndex < deviceArray.length; jIndex++) {
                            if (deviceArray[jIndex].ZoneID == oArray[iIndex].ZoneID) {
                                fAdd = true;
                                break;
                            }
                        }
                    }
                    const deviceCache = dss.staticDataModel.deviceCache;
                    if (!fAdd && deviceCache) {
                        for (let jIndex = 0; jIndex < deviceCache.length; jIndex++) {
                            if (deviceCache[jIndex].present) {
                                if (deviceCache[jIndex].ZoneID == oArray[iIndex].ZoneID) {
                                    if (deviceCache[jIndex].specialActions) {
                                        if (deviceCache[jIndex].specialActions.triggers) {
                                            if (deviceCache[jIndex].specialActions.triggers.binaryInputs) {
                                                for (var sKey in deviceCache[jIndex].specialActions.triggers.binaryInputs) {
                                                    if (Object.prototype.hasOwnProperty.call(deviceCache[jIndex].specialActions.triggers.binaryInputs, sKey)) {
                                                        fAdd = true;
                                                        break;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    if (fAdd) {
                        Ext.data.StoreManager.lookup('triggerConfigurator_zoneAKMStore').add(oArray[iIndex]);
                    }
                }
            }
        }

        if (Ext.data.StoreManager.lookup('triggerConfigurator_accessStore') == null) {
            const Aktionen = [ {
                name: dss.staticDataModel.getNameForScene(0, 0, 73),
                _id: 73,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(0, 0, 73)
            }, {
                name: dss.staticDataModel.getNameForScene(0, 0, 72),
                _id: 72,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(0, 0, 72)
            }, {
                name: dss.staticDataModel.getNameForScene(0, 0, 71),
                _id: 71,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(0, 0, 71)
            } ];
            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_accessStore',
                fields: [ '_id', 'name', 'next', 'icon' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            Ext.data.StoreManager.lookup('triggerConfigurator_accessStore').add(Aktionen);
        }

        if (Ext.data.StoreManager.lookup('triggerConfigurator_weatherStore') == null) {
            const Aktionen = [ {
                name: _('Sun protection'),
                _id: 'sun',
                next: true,
                icon: dss.staticDataModel.getImagePathForState('sun', 'active')
            }, {
                name: dss.staticDataModel.getNameForState('rain', 'active'),
                _id: 'rain_active',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('rain', 'active')
            }, {
                name: dss.staticDataModel.getNameForState('rain', 'inactive'),
                _id: 'rain_inactive',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('rain', 'inactive')
            }, {
                name: dss.staticDataModel.getNameForState('wind', 'active'),
                _id: 'wind_active',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('wind', 'active')
            }, {
                name: dss.staticDataModel.getNameForState('wind', 'inactive'),
                _id: 'wind_inactive',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('wind', 'inactive')
            } ];

            var states = dss.staticDataModel.getStateArray();
            var clusters = dss.staticDataModel.getUsergroups();
            for (const stateKey in states) {
                if (Object.prototype.hasOwnProperty.call(states, stateKey)) {
                    var state = states[stateKey];
                    if (state.name.substring(0, 10) === 'wind.group') {
                        var clusterId = parseInt(state.name.substring(10), 10);
                        var clusterName = Ext.String.format2(_('Group {group}'), { group: clusterId });
                        for (let iIndex = 0; iIndex < clusters.length; iIndex++) {
                            var cluster = clusters[iIndex];
                            if (cluster.id === clusterId) {
                                clusterName = cluster.name;
                                break;
                            }
                        }
                        Aktionen.push({
                            name: Ext.String.format1(_('{state} in {clustername}'), { state: dss.staticDataModel.getNameForState('wind', 'active'), clustername: clusterName }),
                            _id: state.name + '_active',
                            next: false,
                            icon: dss.staticDataModel.getImagePathForState('wind', 'active')
                        });
                        Aktionen.push({
                            name: Ext.String.format1(_('{state} in {clustername}'), { state: dss.staticDataModel.getNameForState('wind', 'inactive'), clustername: clusterName }),
                            _id: state.name + '_inactive',
                            next: false,
                            icon: dss.staticDataModel.getImagePathForState('wind', 'inactive')
                        });
                    }
                }
            }

            Aktionen.push({
                name: dss.staticDataModel.getNameForState('hail', 'active'),
                _id: 'hail_active',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('hail', 'active')
            });
            Aktionen.push({
                name: dss.staticDataModel.getNameForState('hail', 'inactive'),
                _id: 'hail_inactive',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('hail', 'inactive')
            });

            var fFrost = false;
            var fSun = false;
            var fHeating = false;
            var fService = false;
            if (dss.staticDataModel.StateCache) {
                for (let iIndex = 0; iIndex < dss.staticDataModel.StateCache.length; iIndex++) {
                    if (dss.staticDataModel.StateCache[iIndex].name === 'frost') {
                        fFrost = true;
                    }
                    if (dss.staticDataModel.StateCache[iIndex].name === 'sun') {
                        fSun = true;
                    }
                    if (dss.staticDataModel.StateCache[iIndex].name === 'heating_mode') {
                        fHeating = true;
                    }
                    if (dss.staticDataModel.StateCache[iIndex].name === 'service') {
                        fService = true;
                    }
                }
            }

            if (fFrost) {
                Aktionen.push({
                    name: dss.staticDataModel.getNameForState('frost', 'active'),
                    _id: 'frost_active',
                    next: false,
                    icon: dss.staticDataModel.getImagePathForState('frost', 'active')
                });
                Aktionen.push({
                    name: dss.staticDataModel.getNameForState('frost', 'inactive'),
                    _id: 'frost_inactive',
                    next: false,
                    icon: dss.staticDataModel.getImagePathForState('frost', 'inactive')
                });
            }

            if (fSun) {
                Aktionen.push({
                    name: dss.staticDataModel.getNameForState('sun', 'active'),
                    _id: 'sun_active',
                    next: false,
                    icon: dss.staticDataModel.getImagePathForState('sun', 'active')
                });
                Aktionen.push({
                    name: dss.staticDataModel.getNameForState('sun', 'inactive'),
                    _id: 'sun_inactive',
                    next: false,
                    icon: dss.staticDataModel.getImagePathForState('sun', 'inactive')
                });
            }

            if (fHeating) {
                Aktionen.push({
                    name: dss.staticDataModel.getNameForState('heating_mode', 'off'),
                    _id: 'heating_mode_off',
                    next: false,
                    icon: dss.staticDataModel.getImagePathForState('heating_mode', 'off')
                });
                Aktionen.push({
                    name: dss.staticDataModel.getNameForState('heating_mode', 'heating'),
                    _id: 'heating_mode_heating',
                    next: false,
                    icon: dss.staticDataModel.getImagePathForState('heating_mode', 'heating')
                });
                Aktionen.push({
                    name: dss.staticDataModel.getNameForState('heating_mode', 'cooling'),
                    _id: 'heating_mode_cooling',
                    next: false,
                    icon: dss.staticDataModel.getImagePathForState('heating_mode', 'cooling')
                });
                Aktionen.push({
                    name: dss.staticDataModel.getNameForState('heating_mode', 'auto'),
                    _id: 'heating_mode_auto',
                    next: false,
                    icon: dss.staticDataModel.getImagePathForState('heating_mode', 'auto')
                });
            }

            if (fService) {
                Aktionen.push({
                    name: dss.staticDataModel.getNameForState('service', 'active'),
                    _id: 'service_active',
                    next: false,
                    icon: dss.staticDataModel.getImagePathForState('service', 'active')
                });
                Aktionen.push({
                    name: dss.staticDataModel.getNameForState('service', 'inactive'),
                    _id: 'service_inactive',
                    next: false,
                    icon: dss.staticDataModel.getImagePathForState('service', 'inactive')
                });
            }

            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_weatherStore',
                fields: [ '_id', 'name', 'next', 'icon' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            Ext.data.StoreManager.lookup('triggerConfigurator_weatherStore').add(Aktionen);
        }

        if (Ext.data.StoreManager.lookup('triggerConfigurator_weatherSunStore') == null) {
            const Aktionen = [];
            var protectionDirections = [ 'east', 'south', 'west' ];
            for (const dir in protectionDirections) {
                if (Object.prototype.hasOwnProperty.call(protectionDirections, dir)) {
                    Aktionen.push({
                        name: Ext.String.format2(_('Sun protection {direction} {state}'), { direction: protectionDirections[dir], state: 'active' }),
                        _id: 'sun_' + protectionDirections[dir] + '_active',
                        eventName: 'sunshine',
                        value: 1,
                        direction: protectionDirections[dir],
                        next: false,
                        icon: dss.staticDataModel.getImagePathForState('sun', 'active')
                    });
                    Aktionen.push({
                        name: Ext.String.format2(_('Sun protection {direction} {state}'), { direction: protectionDirections[dir], state: 'inactive' }),
                        _id: 'sun_' + protectionDirections[dir] + '_inactive',
                        eventName: 'sunshine',
                        value: 2,
                        direction: protectionDirections[dir],
                        next: false,
                        icon: dss.staticDataModel.getImagePathForState('sun', 'inactive')
                    });
                }
            }

            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_weatherSunStore',
                fields: [ '_id', 'name', 'next', 'icon', 'value', 'direction', 'eventName' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            Ext.data.StoreManager.lookup('triggerConfigurator_weatherSunStore').add(Aktionen);
        }

        if (Ext.data.StoreManager.lookup('triggerConfigurator_securityStore') == null) {
            const Aktionen = [ {
                name: dss.staticDataModel.getNameForScene(0, 0, 65),
                _id: 65,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(0, 0, 65)
            }, {
                name: _('Undo ') + dss.staticDataModel.getNameForScene(0, 0, 65),
                _id: 1065,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(0, 0, 65, true)
            }, {
                name: dss.staticDataModel.getNameForState('fire', 'active'),
                _id: 'fire_active',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('fire', 'active')
            }, {
                name: dss.staticDataModel.getNameForState('fire', 'inactive'),
                _id: 'fire_inactive',
                next: false,
                icon: dss.staticDataModel.getImagePathForState('fire', 'inactive')
            }, {
                name: dss.staticDataModel.getNameForScene(0, 0, 74),
                _id: 74,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(0, 0, 74)
            }, {
                name: _('Undo ') + dss.staticDataModel.getNameForScene(0, 0, 74),
                _id: 1074,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(0, 0, 74, true)
            }, {
                name: dss.staticDataModel.getNameForScene(0, 0, 83),
                _id: 83,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(0, 0, 83)
            }, {
                name: _('Undo ') + dss.staticDataModel.getNameForScene(0, 0, 83),
                _id: 1083,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(0, 0, 83, true)
            }, {
                name: dss.staticDataModel.getNameForScene(0, 0, 84),
                _id: 84,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(0, 0, 84)
            }, {
                name: _('Undo ') + dss.staticDataModel.getNameForScene(0, 0, 84),
                _id: 1084,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(0, 0, 84, true)
            }, {
                name: dss.staticDataModel.getNameForScene(0, 0, 85),
                _id: 85,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(0, 0, 85)
            }, {
                name: _('Undo ') + dss.staticDataModel.getNameForScene(0, 0, 85),
                _id: 1085,
                next: false,
                icon: dss.staticDataModel.getImagePathForScene(0, 0, 85, true)
            } ];
            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_securityStore',
                fields: [ '_id', 'name', 'next', 'icon' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            Ext.data.StoreManager.lookup('triggerConfigurator_securityStore').add(Aktionen);
        }

    },
    getGroupStore: function (ZoneID) {
        if (Ext.data.StoreManager.lookup('triggerConfigurator_groupStore_' + ZoneID) == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_groupStore_' + ZoneID,
                fields: [ 'group', 'name', 'icon', 'reachable' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            var oGroup = dss.staticDataModel.getGroupForZone(ZoneID);
            Ext.data.StoreManager.lookup('triggerConfigurator_groupStore_' + ZoneID).add(oGroup);
        }
        return Ext.data.StoreManager.lookup('triggerConfigurator_groupStore_' + ZoneID);
    },
    getGlobalAppStateStore: function (groupId) {
        // global app state means: zoneId == 0
        var oStore = Ext.data.StoreManager.lookup('triggerConfigurator_globalAppStateStore_' + groupId);
        if (oStore == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_globalAppStateStore_' + groupId,
                fields: [ '_id', 'name', 'reachable', 'icon', 'outputMode', 'forced', 'value', 'stateid', 'valueandstate' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            oStore = Ext.data.StoreManager.lookup('triggerConfigurator_globalAppStateStore_' + groupId);
            if (dss.staticDataModel.StateCache) {
                for (var i = 0; i < dss.staticDataModel.StateCache.length; i++) {
                    if (dss.staticDataModel.StateCache[i].name.indexOf('zone.0.group.' + groupId) > -1) {
                        var oState = dss.staticDataModel.StateCache[i];
                        var oArray = oState.name.split('.');
                        var stateType = oArray[5] ? oArray[5] : oState.name;
                        oStore.add({
                            stateid: oState.name,
                            reachable: true,
                            name: dss.staticDataModel.getNameForState(stateType, 'inactive'),
                            value: 'inactive',
                            icon: dss.staticDataModel.getImagePathForState(oState.name, 'inactive'),
                            valueandstate: oState.name + '_inactive'
                        });
                        oStore.add({
                            stateid: oState.name,
                            name: dss.staticDataModel.getNameForState(stateType, 'active'),
                            reachable: true,
                            value: 'active',
                            icon: dss.staticDataModel.getImagePathForState(oState.name, 'active'),
                            valueandstate: oState.name + '_active'
                        });
                    }
                }
            }
        }
        return oStore;
    },
    getSceneStore: function (ZoneID, groupID) {
        if (Ext.data.StoreManager.lookup('triggerConfigurator_szeneStore_' + ZoneID + '_' + groupID) == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_szeneStore_' + ZoneID + '_' + groupID,
                fields: [ '_id', 'name', 'reachable', 'icon', 'outputMode', 'forced', 'value', 'stateid', 'valueandstate' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });

            var oScenes = dss.staticDataModel.getScenes(ZoneID, groupID);
            if (oScenes) {
                for (var iIndex = 0; iIndex < oScenes.length; iIndex++) {
                    var oObject = {};
                    oObject._id = oScenes[iIndex].id;
                    oObject.reachable = oScenes[iIndex].reachable;
                    oObject.icon = oScenes[iIndex].icon;
                    oObject.stateid = '';
                    if (oScenes[iIndex].name == oScenes[iIndex].defaultName) {
                        oObject.name = oScenes[iIndex].name;
                    } else {
                        oObject.name = oScenes[iIndex].defaultName + ' - ' + oScenes[iIndex].name;
                    }
                    Ext.data.StoreManager.lookup('triggerConfigurator_szeneStore_' + ZoneID + '_' + groupID).add(oObject);

                    if (oObject.undoable) {
                        oObject = {};
                        oObject._id = -1 * oScenes[iIndex].id;
                        oObject.reachable = oScenes[iIndex].reachable;
                        oObject.icon = oScenes[iIndex].icon;
                        if (oScenes[iIndex].name == oScenes[iIndex].defaultName) {
                            oObject.name = _('Undo ') + oScenes[iIndex].name;
                        } else {
                            oObject.name = _('Undo ') + oScenes[iIndex].defaultName + ' - ' + oScenes[iIndex].name;
                        }
                        Ext.data.StoreManager.lookup('triggerConfigurator_szeneStore_' + ZoneID + '_' + groupID).add(oObject);
                    }

                    if (oScenes[iIndex].id == 0) {
                        if (groupID > 63 || groupID == 10 || groupID == 12) {

                        } else if (groupID == 2) {
                            Ext.data.StoreManager.lookup('triggerConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                                _id: 0,
                                name: _('all closed'),
                                icon: oScenes[iIndex].icon,
                                reachable: oScenes[iIndex].reachable,
                                forced: true
                            });
                        } else {
                            Ext.data.StoreManager.lookup('triggerConfigurator_szeneStore_' + ZoneID + '_' + groupID).add({
                                _id: 0,
                                name: _('all off'),
                                icon: oScenes[iIndex].icon,
                                reachable: oScenes[iIndex].reachable,
                                forced: true
                            });
                        }
                    }
                }
            }
        }
        return Ext.data.StoreManager.lookup('triggerConfigurator_szeneStore_' + ZoneID + '_' + groupID);
    },
    getSensorMessage: function (dSUID) {
        if (Ext.data.StoreManager.lookup('triggerConfigurator_sensorMessageStore_' + dSUID) == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_sensorMessageStore_' + dSUID,
                fields: [ 'name', '_id' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            var oDeviceArray = dss.staticDataModel.getSensorDeviceArray();
            if (oDeviceArray) {
                for (var iIndex = 0; iIndex < oDeviceArray.length; iIndex++) {
                    if ((dSUID == -1) || (oDeviceArray[iIndex].dSUID == dSUID)) {
                        if (oDeviceArray[iIndex].sensorEvents) {
                            for (var jIndex = 0; jIndex < oDeviceArray[iIndex].sensorEvents.length; jIndex++) {
                                Ext.data.StoreManager.lookup('triggerConfigurator_sensorMessageStore_' + dSUID).add({
                                    name: oDeviceArray[iIndex].sensorEvents[jIndex].name,
                                    _id: oDeviceArray[iIndex].sensorEvents[jIndex].id
                                });
                            }
                        }
                    }
                }
            }
        }
        return Ext.data.StoreManager.lookup('triggerConfigurator_sensorMessageStore_' + dSUID);
    },
    getSensorDeviceStore: function (ZoneID) {
        if (Ext.data.StoreManager.lookup('triggerConfigurator_deviceSensorStore_' + ZoneID) == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_deviceSensorStore_' + ZoneID,
                fields: [ 'dSID', 'dSUID', 'name', 'icon', 'present' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            var oDeviceArray = dss.staticDataModel.getSensorDeviceArray();
            if (oDeviceArray) {
                for (var iIndex = 0; iIndex < oDeviceArray.length; iIndex++) {
                    if (ZoneID == 0 || ZoneID == oDeviceArray[iIndex].ZoneID) {
                        Ext.data.StoreManager.lookup('triggerConfigurator_deviceSensorStore_' + ZoneID).add(oDeviceArray[iIndex]);
                    }
                }
            }
        }
        return Ext.data.StoreManager.lookup('triggerConfigurator_deviceSensorStore_' + ZoneID);
    },
    getDeviceAKMStore: function (ZoneID) {
        if (Ext.data.StoreManager.lookup('triggerConfigurator_deviceAKMStore_' + ZoneID) == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_deviceAKMStore_' + ZoneID,
                fields: [ 'dSID', 'dSUID', 'name', 'icon', 'present' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            var oDeviceArray = dss.staticDataModel.getAKMDeviceArray();
            var targetArray = [];
            if (oDeviceArray) {
                for (let iIndex = 0; iIndex < oDeviceArray.length; iIndex++) {
                    if (ZoneID == 0 || ZoneID == oDeviceArray[iIndex].ZoneID) {
                        targetArray.push(oDeviceArray[iIndex]);
                    }
                }
            }
            const deviceCache = dss.staticDataModel.deviceCache;
            if (deviceCache) {
                for (let iIndex = 0; iIndex < deviceCache.length; iIndex++) {
                    var fAdd = false;
                    if (deviceCache[iIndex].present) {
                        if (deviceCache[iIndex].ZoneID == ZoneID) {
                            if (deviceCache[iIndex].specialActions) {
                                if (deviceCache[iIndex].specialActions.triggers) {
                                    if (deviceCache[iIndex].specialActions.triggers.binaryInputs) {
                                        for (var sKey in deviceCache[iIndex].specialActions.triggers.binaryInputs) {
                                            if (Object.prototype.hasOwnProperty.call(deviceCache[iIndex].specialActions.triggers.binaryInputs, sKey)) {
                                                fAdd = true;
                                                break;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    for (let jIndex = 0; jIndex < targetArray.length && fAdd; jIndex++) {
                        if (targetArray[jIndex].dSUID == deviceCache[iIndex].dSUID) {
                            fAdd = false;
                        }
                    }
                    if (fAdd) {
                        targetArray.push(deviceCache[iIndex]);
                    }
                }
            }
            Ext.data.StoreManager.lookup('triggerConfigurator_deviceAKMStore_' + ZoneID).add(targetArray);
        }

        return Ext.data.StoreManager.lookup('triggerConfigurator_deviceAKMStore_' + ZoneID);
    },
    getDeviceStore: function (ZoneID) {
        if (Ext.data.StoreManager.lookup('triggerConfigurator_deviceStore_' + ZoneID) == null) {
            Ext.create('Ext.data.Store', {
                storeId: 'triggerConfigurator_deviceStore_' + ZoneID,
                fields: [ 'dSID', 'dSUID', 'name', 'icon', 'present' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });
            var oDeviceArray = dss.staticDataModel.getDevicesForZone(ZoneID, 0);
            if (oDeviceArray) {
                for (var iIndex = 0; iIndex < oDeviceArray.length; iIndex++) {
                    var fShouldAdd = false;
                    if (oDeviceArray[iIndex].hasStates) {
                        fShouldAdd = true;
                    }
                    if (oDeviceArray[iIndex].deviceEvents && Object.keys(oDeviceArray[iIndex].deviceEvents).length) {
                        fShouldAdd = true;
                    }
                    if (oDeviceArray[iIndex].deviceCustomActions && Object.keys(oDeviceArray[iIndex].deviceCustomActions).length) {
                        fShouldAdd = true;
                    }
                    if (oDeviceArray[iIndex].deviceStandardActions && Object.keys(oDeviceArray[iIndex].deviceStandardActions).length) {
                        fShouldAdd = true;
                    }
                    if (oDeviceArray[iIndex].deviceActions && Object.keys(oDeviceArray[iIndex].deviceActions).length) {
                        fShouldAdd = true;
                    }
                    if (oDeviceArray[iIndex].tasterMode == 15) {
                        fShouldAdd = true;
                    } else if (oDeviceArray[iIndex].tasterMode == 0 && oDeviceArray[iIndex].buttonType != 65) {
                        fShouldAdd = true;
                    } else if (oDeviceArray[iIndex].specialActions) {
                        if (oDeviceArray[iIndex].specialActions.triggers) {
                            if (oDeviceArray[iIndex].specialActions.triggers.deviceMsg != null || oDeviceArray[iIndex].specialActions.triggers.deviceAction != null) {
                                fShouldAdd = true;
                            }
                        }
                    }
                    if (fShouldAdd) {
                        Ext.data.StoreManager.lookup('triggerConfigurator_deviceStore_' + ZoneID).add(oDeviceArray[iIndex]);
                    }
                }
            }
        }
        return Ext.data.StoreManager.lookup('triggerConfigurator_deviceStore_' + ZoneID);
    },
    constructGridLvlRoomStateState: function (zoneID) {
        var oStates = dss.staticDataModel.getStates();
        var oShowStates = [];
        if (oStates) {
            for (let iIndex = 0; iIndex < oStates.length; iIndex++) {
                var sSplitName = oStates[iIndex].name.split('.');
                if (sSplitName[0] === 'zone') {
                    if (sSplitName.length === 3) {
                        if ((sSplitName[1] == '' + zoneID) && (sSplitName[2] !== 'heating')) {
                            oShowStates.push(oStates[iIndex]);
                        }
                    } else if (sSplitName.length === 6) {
                        if ((sSplitName[1] == '' + zoneID) && (sSplitName[2] === 'group')) {
                            // show item only if there are given group actors in the zone
                            if (dss.staticDataModel.getDevicesForZone(zoneID, sSplitName[3]).length) {
                                oShowStates.push(oStates[iIndex]);
                            }
                        }
                    }
                }
            }
        }
        oShowStates.sort(upperCaseSort);

        var oRoomStateStateControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'name', 'stateid', 'value', 'icon', 'tooltip', 'valueandstate' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        var oNewArray = [];
        for (let iIndex = 0; iIndex < oShowStates.length; iIndex++) {
            var oNameArray = oShowStates[iIndex].name.split('.');
            var oItem = {
                stateid: oShowStates[iIndex].name,
                value: 'active',
                tooltip: '',
                icon: '',
                valueandstate: oShowStates[iIndex].name + '_active'
            };
            if (oNameArray.length === 3) {
                if (oNameArray[2] === 'light') {
                    oItem.icon = dss.staticDataModel.getImagePathForScene(0, 1, 5);
                    oItem.name = Ext.String.format2(_('{#editor_trigger_zonestates}light is active'), {});
                } else if (oNameArray[2] === 'motion') {
                    oItem.icon = dss.staticDataModel.getImagePathForAKMValue(5, true);
                    oItem.name = Ext.String.format2(_('{#editor_trigger_zonestates}motion is active'), {});
                } else if (oNameArray[2] === 'presence') {
                    oItem.icon = dss.staticDataModel.getImagePathForAKMValue(1, true);
                    oItem.name = Ext.String.format2(_('{#editor_trigger_zonestates}presence is active'), {});
                } else {
                    oItem.icon = dss.staticDataModel.getImagePathForAKMValue(0, true);
                    oItem.name = Ext.String.format2(_('{#editor_trigger_zonestates}{state} is active'), {
                        state: oNameArray[2]
                    });
                }
            } else if (oNameArray.length === 6) {
                if (oNameArray[0] === 'zone' && oNameArray[2] === 'group' && oNameArray[4] === 'status') {
                    const zoneId = parseInt(oNameArray[1], 10);
                    const groupId = parseInt(oNameArray[3], 10);
                    const groupName = dss.staticDataModel.getNameForGroup(zoneId, groupId);
                    const stateName = oNameArray[5];
                    oItem.icon = dss.staticDataModel.getImagePathForState(oShowStates[iIndex].name, 'active');

                    if (zoneId === 0) {
                        if (groupId === 64) {
                            // this is global application state - apartment ventilation
                            if (stateName === 'malfunction') {
                                oItem.name = Ext.String.format2(_('{#editor_trigger_zone_state}Malfunction in {group}'), {
                                    group: groupName
                                });
                            } else if (stateName === 'service') {
                                oItem.name = Ext.String.format2(_('{#editor_trigger_zone_state}Service needed in {group}'), {
                                    group: groupName
                                });
                            }
                        }
                    } else {
                        // this is zone state
                        if (stateName === 'malfunction') {
                            oItem.name = Ext.String.format2(_('{#editor_trigger_zone_state}Malfunction in {group}'), {
                                group: groupName
                            });
                        } else if (stateName === 'service') {
                            oItem.name = Ext.String.format2(_('{#editor_trigger_zone_state}Service needed in {group}'), {
                                group: groupName
                            });
                        }
                    }
                }
            }
            oNewArray.push(oItem);

            oItem = {
                stateid: oShowStates[iIndex].name,
                value: 'inactive',
                tooltip: '',
                icon: '',
                valueandstate: oShowStates[iIndex].name + '_inactive'
            };
            if (oNameArray.length === 3) {
                if (oNameArray[2] === 'light') {
                    oItem.icon = dss.staticDataModel.getImagePathForScene(0, 1, 0);
                    oItem.name = Ext.String.format2(_('{#editor_trigger_zonestates}light is inactive'), {});
                } else if (oNameArray[2] === 'motion') {
                    oItem.icon = dss.staticDataModel.getImagePathForAKMValue(5, false);
                    oItem.name = Ext.String.format2(_('{#editor_trigger_zonestates}motion is inactive'), {});
                } else if (oNameArray[2] === 'presence') {
                    oItem.icon = dss.staticDataModel.getImagePathForAKMValue(1, false);
                    oItem.name = Ext.String.format2(_('{#editor_trigger_zonestates}presence is inactive'), {});
                } else {
                    oItem.icon = dss.staticDataModel.getImagePathForAKMValue(0, false);
                    oItem.name = Ext.String.format2(_('{#editor_trigger_zonestates}{state} is inactive'), {
                        state: oNameArray[2]
                    });
                }
            } else if (oNameArray.length === 6) {
                if (oNameArray[0] === 'zone' && oNameArray[2] === 'group' && oNameArray[4] === 'status') {
                    const zoneId = parseInt(oNameArray[1], 10);
                    const groupId = parseInt(oNameArray[3], 10);
                    const groupName = dss.staticDataModel.getNameForGroup(zoneId, groupId);
                    const stateName = oNameArray[5];
                    oItem.icon = dss.staticDataModel.getImagePathForState(oShowStates[iIndex].name, 'inactive');

                    if (zoneId === 0) {
                        if (groupId === 64) {
                            // this is global application state - apartment ventilation
                            if (stateName === 'malfunction') {
                                oItem.name = Ext.String.format2(_('{#editor_trigger_zone_state}no Malfunction in {group}'), {
                                    group: groupName
                                });
                            } else if (stateName === 'service') {
                                oItem.name = Ext.String.format2(_('{#editor_trigger_zone_state}no Service needed in {group}'), {
                                    group: groupName
                                });
                            }
                        }
                    } else {
                        // this is zone state
                        if (stateName === 'malfunction') {
                            oItem.name = Ext.String.format2(_('{#editor_trigger_zone_state}no Malfunction in {group}'), {
                                group: groupName
                            });
                        } else if (stateName === 'service') {
                            oItem.name = Ext.String.format2(_('{#editor_trigger_zone_state}no Service needed in {group}'), {
                                group: groupName
                            });
                        }
                    }
                }
            }
            oNewArray.push(oItem);
        }
        oRoomStateStateControlStore.add(oNewArray);

        var me = this;

        this.grids.lvlRoomStateState = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: (this.getWidth() - (this.iDefaultWidth * 2)) - 2, // 474,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oRoomStateStateControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlRoomStateState, this.triggerSelectorName + 'GridLvlZoneStateValue');
        this.grids.lvlRoomStateState.parent = this;

        this.macPanel.add(this.grids.lvlRoomStateState);

    },
    constructGridLvlDeviceStateValue: function (dSUID, type) {
        var oStates = dss.staticDataModel.getDeviceStates(dSUID);

        var oStateDeviceValueControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'type', {
                name: 'name',
                sortType: 'asUCText'
            }, 'selector' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        var oValueArray = [];

        var maxLabelWidth = 0;
        var calcWidth = this.calculateWidthHelper('x-grid-cell');

        for (var sKey in oStates) {
            if (sKey == type) {
                for (var sSubKey in oStates[sKey].options) {
                    if (Object.prototype.hasOwnProperty.call(oStates[sKey].options, sSubKey)) {
                        oValueArray.push({
                            type: sSubKey,
                            name: oStates[sKey].options[sSubKey],
                            selector: 'dev.' + dSUID + '.' + type
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oStates[sKey].options[sSubKey]));
                    }
                }
            }
        }
        oStateDeviceValueControlStore.add(oValueArray);
        oStateDeviceValueControlStore.sort('name');

        var width = (this.getWidth() - (this.iDefaultWidth * 4)) - 2;

        var me = this;
        this.grids.lvlDeviceStateValue = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: Math.max(width, maxLabelWidth),
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            selModel: {
                mode: 'MULTI'
            },
            store: oStateDeviceValueControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {           // jshint ignore:line
                    const name = record.get('name');
                    return '<table style="height:16px;width:100%" ' +
                        'data-type="' + record.get('type') + '" data-name="' + joinAndCapitalize(name) + '" data-state-id="' + record.get('selector') + '">' +
                        '<tr><td width="100%">' + name + '</td></tr>' +
                    '</table>';
                }
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlDeviceStateValue, this.triggerSelectorName + 'GridLvlDeviceActionDeviceClassState');
        this.grids.lvlDeviceStateValue.parent = this;

        this.macPanel.add(this.grids.lvlDeviceStateValue);
    },
    constructGridLvlRoomStateRoom: function () {
        var oRoomStateControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'ZoneID', {
                name: 'name',
                sortType: 'asUCText'
            } ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oZoneStore = Ext.data.StoreManager.lookup('triggerConfigurator_zoneZoneStore');
        oZoneStore.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('ZoneID') != 0) {
                oRoomStateControlStore.add(oNewRecord);
            }
        });
        oRoomStateControlStore.sort('name');

        this.grids.lvlRoomStateRoom = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oRoomStateControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {           // jshint ignore:line
                    return '<table style="height:16px;width:100%" data-zone-id="' + record.get('ZoneID') + '">' +
                        '<tr><td width="100%">' + record.get('name') + '</td><td>&gt;</td></tr>' +
                    '</table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlRoomStateRoom, this.triggerSelectorName + 'GridLvlZoneState');
        this.grids.lvlRoomStateRoom.parent = this;
        this.grids.lvlRoomStateRoom.on('itemclick', function (grid, record) {
            this.parent.grids.lvlRoomStateState && this.parent.macPanel.remove(this.parent.grids.lvlRoomStateState);

            var zoneID = record.get('ZoneID');
            this.parent.constructGridLvlRoomStateState(zoneID);
        });
        this.macPanel.add(this.grids.lvlRoomStateRoom);
    },

    constructGridLvlDeviceEventEventSubcategories: function (dSUID, category, fAdvanced) {

        var oEventArray = [];
        var oStateDeviceEventControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'event', '_dsuid', {
                name: 'name',
                sortType: 'asUCText'
            } ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var maxLabelWidth = 0;
        var calcWidth = this.calculateWidthHelper('x-grid-cell');

        var oDevice = dss.staticDataModel.getDevice(dSUID);
        for (var sKey in oDevice.deviceEvents) {
            if (Object.prototype.hasOwnProperty.call(oDevice.deviceEvents, sKey)) {
                var fSkip = false;
                if (oDevice.deviceEvents[sKey].accessLevel) {
                    if (oDevice.deviceEvents[sKey].accessLevel === 'service') {
                        if (!this.fShowEventService) {
                            fSkip = true;
                        }
                    }
                    if (oDevice.deviceEvents[sKey].accessLevel === 'advanced') {
                        if (!fAdvanced) {
                            fSkip = true;
                        }
                    }
                }
                if (!fSkip) {
                    if (oDevice.deviceEvents[sKey].category == category) {
                        oEventArray.push({
                            event: sKey,
                            _dsuid: dSUID,
                            name: oDevice.deviceEvents[sKey].title
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oDevice.deviceEvents[sKey].title));
                    }
                }
            }
        }

        oStateDeviceEventControlStore.add(oEventArray);
        var me = this;
        this.grids.lvlDeviceEventEventSub = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: Math.max(this.iDefaultWidth, maxLabelWidth),
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oStateDeviceEventControlStore,
            selModel: {
                mode: 'MULTI'
            },
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {           // jshint ignore:line
                    const name = record.get('name');
                    const dsuid = record.get('_dsuid');
                    const eventType = record.get('event');
                    return '<table style="height:16px;width:100%" data-dsuid="' + dsuid + '" data-event="' + eventType + '" data-name="' + joinAndCapitalize(name) + '">' +
                        '<tr><td width="100%">' + record.get('name') + '</td></tr>' +
                    '</table>';
                }
            } ],
            listeners: this.makeClickListeners
        });
        addDataIdToElement(this.grids.lvlDeviceEventEventSub, this.triggerSelectorName + 'GridLvlDeviceActionDeviceActionClassEventSub');
        this.grids.lvlDeviceEventEventSub.parent = this;

        this.grids.lvlDeviceEventEventSub.on('added', function (grid, record) {       // jshint ignore:line
            for (var sKey in me.grids) {
                if (Object.prototype.hasOwnProperty.call(me.grids, sKey)) {
                    me.grids[sKey].setHeight(me.iDefaultHeight - 17);
                }
            }
        });
        this.grids.lvlDeviceEventEventSub.on('removed', function (grid, record) {     // jshint ignore:line
            for (var sKey in me.grids) {
                if (Object.prototype.hasOwnProperty.call(me.grids, sKey)) {
                    me.grids[sKey].setHeight(me.iDefaultHeight);
                }
            }
        });

        this.macPanel.add(this.grids.lvlDeviceEventEventSub);
        this.macPanel.scrollBy(1000, 0, false);

    },
    constructGridLvlDeviceEventEvent: function (dSUID, fAdvanced, wider) {
        const device = dss.staticDataModel.getDevice(dSUID);
        const oEvents = device.deviceEvents;
        const oCategory = device.deviceCategory;
        const oStateDeviceEventControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'event', '_dsuid', {
                name: 'name',
                sortType: 'asUCText'
            }, 'type', 'next', 'orderID', 'checkable', 'checked', 'fAdvanced' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        const oEventArray = [];
        const oEventCategories = [];

        const oEventCatsAndDogs = {};

        let maxLabelWidth = 0;
        const calcWidth = this.calculateWidthHelper('x-grid-cell');

        let fHasAdvanced = false;
        if (fAdvanced == null) {
            fAdvanced = false;
        }
        for (const sKey in oEvents) {
            if (Object.prototype.hasOwnProperty.call(oEvents, sKey)) {
                let fSkip = false;
                if (oEvents[sKey].accessLevel != null) {
                    if (oEvents[sKey].accessLevel === 'service' && !this.fShowEventService) {
                        fSkip = true;
                    }
                    if (oEvents[sKey].accessLevel === 'advanced') {
                        fHasAdvanced = true;
                        if (!fAdvanced) {
                            fSkip = true;
                        }
                    }
                }
                if (!fSkip) {
                    var selectedCategory = null;
                    if (oEvents[sKey].category != null) {
                        selectedCategory = oCategory[oEvents[sKey].category];
                    }
                    if (selectedCategory == null) {
                        oEventArray.push({
                            type: 'event',
                            event: sKey,
                            _dsuid: dSUID,
                            name: oEvents[sKey].title,
                            next: false,
                            checkable: null,
                            fAdvanced: fAdvanced
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oEvents[sKey].title));
                    } else if (!selectedCategory.alwaysVisible) {
                        oEventCatsAndDogs[oEvents[sKey].category] = [ {
                            type: 'category',
                            event: oEvents[sKey].category,
                            _dsuid: dSUID,
                            name: selectedCategory.title,
                            next: true,
                            orderID: selectedCategory.orderId,
                            checkable: null,
                            fAdvanced: fAdvanced
                        } ];
                    } else {
                        if (oEventCatsAndDogs[oEvents[sKey].category] == null) {
                            oEventCatsAndDogs[oEvents[sKey].category] = [];
                        }
                        oEventCatsAndDogs[oEvents[sKey].category].push({
                            type: 'event',
                            event: sKey,
                            _dsuid: dSUID,
                            name: oEvents[sKey].title,
                            next: false,
                            checkable: null,
                            fAdvanced: fAdvanced
                        });
                    }
                }
            }
        }
        if (fHasAdvanced) {
            oStateDeviceEventControlStore.add({
                _dsuid: dSUID,
                name: _('show more events'),
                icon: null,
                checkable: true,
                checked: fAdvanced,
                orderID: -1
            });
            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('show more events')));
            oStateDeviceEventControlStore.add({
                event: '-',
                _dsuid: null,
                name: '-',
                type: 'spacer',
                subentry: null
            });
        }
        var orderArray = [];
        for (var sKey in oEventCatsAndDogs) {
            if (Object.prototype.hasOwnProperty.call(oEventCatsAndDogs, sKey)) {
                orderArray.push(sKey);
            }
        }

        orderArray.sort(function (aKey, bKey) {
            var a = oCategory[aKey];
            var b = oCategory[bKey];
            return parseInt(a.orderId.toString(), 10) < parseInt(b.orderId.toString(), 10) ? -1 : 1;
        });

        for (let iIndex = 0; iIndex < orderArray.length; iIndex++) {
            oEventCatsAndDogs[orderArray[iIndex]].sort(upperCaseSort);
            oStateDeviceEventControlStore.add(oEventCatsAndDogs[orderArray[iIndex]]);
            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oEventCatsAndDogs[orderArray[iIndex]][name]));

            if (iIndex < orderArray.length - 1) {
                let fSkip = false;
                if (oEventCatsAndDogs[orderArray[iIndex]].length === 1) {
                    if (oEventCatsAndDogs[orderArray[iIndex]][0].type === 'category') {
                        if (oEventCatsAndDogs[orderArray[iIndex + 1]].length === 1) {
                            if (oEventCatsAndDogs[orderArray[iIndex + 1]][0].type === 'category') {
                                fSkip = true;
                            }
                        }
                    }
                }
                if (!fSkip) {
                    oStateDeviceEventControlStore.add({
                        event: '-',
                        _dsuid: null,
                        name: '-',
                        type: 'spacer',
                        subentry: null
                    });
                }
            }

        }

        if (oStateDeviceEventControlStore.count() > 0 && oEventArray.length) {
            oStateDeviceEventControlStore.add({
                event: '-',
                _dsuid: null,
                name: '-',
                type: 'spacer',
                subentry: null
            });
        }
        oStateDeviceEventControlStore.add(oEventArray);

        var width = (this.getWidth() - (this.iDefaultWidth * 4)) - 2;
        if (wider) {
            width += this.iDefaultWidth;
        }
        if (oEventCategories.length) {        // condition never fulfilled - the array is created empty and nothing's added to it
            width = this.iDefaultWidth;
        }
        var me = this;
        this.grids.lvlDeviceEventEvent = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: Math.max(width, maxLabelWidth),
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oStateDeviceEventControlStore,
            selModel: {
                mode: 'MULTI'
            },
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ],
            listeners: {
                itemdblclick: function (grid, record) {
                    me.macPanel.remove(me.grids.lvlDeviceEventEventSub);
                    const type = record.get('type');
                    if (type === 'category') {
                        me.constructGridLvlDeviceEventEventSubcategories(record.get('_dsuid'), record.get('event'), record.get('fAdvanced'));
                    } else if (type === 'event') {
                        if (me.isValid()) {
                            me.fireEvent('valueDoubleClicked', me);
                        }
                    }
                },
                itemclick: {
                    fn: function (grid, record) {
                        me.macPanel.remove(me.grids.lvlDeviceEventEventSub);
                        const type = record.get('type');
                        if (type === 'event') {
                            me.fireEvent('valueClicked', me);
                        } else if (type === 'category') {
                            me.constructGridLvlDeviceEventEventSubcategories(record.get('_dsuid'), record.get('event'), record.get('fAdvanced'));
                        }
                    },
                    // single: true,
                    buffer: 300
                }
            }
        });
        addDataIdToElement(this.grids.lvlDeviceEventEvent, this.triggerSelectorName + 'GridLvlDeviceActionDeviceActionClassEvent');

        this.grids.lvlDeviceEventEvent.on('beforeitemclick', function (grid, record) {
            if (record.get('_id') == -1) {
                this.getSelectionModel().deselect(record);
                return false;
            }

            const checkable = record.get('checkable');
            if (checkable) {
                this.getSelectionModel().deselect(record);
                var fChecked = !record.get('checked');
                me.macPanel.remove(me.grids.lvlDeviceEventEventSub);
                me.macPanel.remove(me.grids.lvlDeviceEventEvent);
                me.constructGridLvlDeviceEventEvent(record.get('_dsuid'), fChecked);
                return false;
            }
        });

        this.grids.lvlDeviceEventEvent.parent = this;

        /*
         * this.grids.lvlDeviceEventEvent.on('added', function (grid, record) { for (var sKey in me.grids) { me.grids[sKey].setHeight(me.iDefaultHeight-17); } });
         * this.grids.lvlDeviceEventEvent.on('removed', function (grid, record) { for (var sKey in me.grids) { me.grids[sKey].setHeight(me.iDefaultHeight); } });
         */

        this.macPanel.add(this.grids.lvlDeviceEventEvent);

    },
    constructGridLvlDeviceActionActionSubcategories: function (dSUID, category) {
        const oActionStandardArray = [];
        const oActionCustomArray = [];
        const oActionDeviceArray = [];
        this.grids.lvlDeviceActionActionSub && this.macPanel.remove(this.grids.lvlDeviceActionActionSub);
        const oStateDeviceArrayControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'action', '_dsuid', {
                name: 'name',
                sortType: 'asUCText'
            } ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            },
            sorters: [ {
                property: 'name',
                transform: function (val) {
                    return val.toLowerCase();
                }
            } ]
        });

        let maxLabelWidth = 0;
        const calcWidth = this.calculateWidthHelper('x-grid-cell');

        const oStandard = dss.staticDataModel.getDevice(dSUID).deviceStandardActions;
        const oDeviceActions = dss.staticDataModel.getDevice(dSUID).deviceActions;
        const oDescriptions = dss.staticDataModel.getDevice(dSUID).deviceActionDescription;

        if (oStandard) {
            for (const sKey in oStandard) {
                if (Object.prototype.hasOwnProperty.call(oStandard, sKey)) {
                    const description = oDescriptions[oStandard[sKey].action];
                    if (description.category == category) {
                        oActionStandardArray.push({
                            action: sKey,
                            _dsuid: dSUID,
                            name: oStandard[sKey].title
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oStandard[sKey].title));
                    }
                }
            }
        }

        if (oDeviceActions) {
            for (const sKey in oDeviceActions) {
                if (Object.prototype.hasOwnProperty.call(oDeviceActions, sKey)) {
                    if (oDeviceActions[sKey].category == category) {
                        oActionDeviceArray.push({
                            action: sKey,
                            _dsuid: dSUID,
                            name: oDeviceActions[sKey].title
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oDeviceActions[sKey].title));
                    }
                }
            }
        }

        oStateDeviceArrayControlStore.add(oActionStandardArray);
        oStateDeviceArrayControlStore.add(oActionCustomArray);
        oStateDeviceArrayControlStore.add(oActionDeviceArray);
        var me = this;
        this.grids.lvlDeviceActionActionSub = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: Math.max(this.iDefaultWidth, maxLabelWidth),
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oStateDeviceArrayControlStore,
            selModel: {
                mode: 'MULTI'
            },
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {        // jshint ignore:line
                    const action = record.get('action');
                    const dsuid = record.get('_dsuid');
                    const name = record.get('name');
                    return '<table style="height:16px;width:100%" data-action="' + action + '" data-name="' + joinAndCapitalize(name) + '" data-dsuid="' + dsuid + '">' +
                        '<tr><td width="100%">' + name + '</td></tr>' +
                    '</table>';
                }
            } ],
            listeners: this.makeClickListeners
        });
        addDataIdToElement(this.grids.lvlDeviceActionActionSub, this.triggerSelectorName + 'GridLvlDeviceActionDeviceActionClassActionSub');
        this.grids.lvlDeviceActionActionSub.parent = this;

        this.grids.lvlDeviceActionActionSub.on('added', function (grid, record) {     // jshint ignore:line
            for (const sKey in me.grids) {
                if (Object.prototype.hasOwnProperty.call(me.grids, sKey)) {
                    me.grids[sKey].setHeight(me.iDefaultHeight - 17);
                }
            }
        });
        this.grids.lvlDeviceActionActionSub.on('removed', function (grid, record) {       // jshint ignore:line
            for (const sKey in me.grids) {
                if (Object.prototype.hasOwnProperty.call(me.grids, sKey)) {
                    me.grids[sKey].setHeight(me.iDefaultHeight);
                }
            }
        });

        this.macPanel.add(this.grids.lvlDeviceActionActionSub);
        this.macPanel.scrollBy(1000, 0, false);
    },

    constructGridLvlDeviceActionAction: function (dSUID, wider) {

        this.grids.lvlDeviceActionAction && this.macPanel.remove(this.grids.lvlDeviceActionAction);
        var oStateDeviceActionControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'action', '_dsuid', 'name', 'next', 'type', 'orderID' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oDevice = dss.staticDataModel.getDevice(dSUID);
        var oCustom = oDevice.deviceCustomActions;
        var oStandard = oDevice.deviceStandardActions;
        var oDeviceActions = oDevice.deviceActions;
        var oCategory = oDevice.deviceCategory;
        var oDescriptions = oDevice.deviceActionDescription;
        var oActionArray = [];
        var oStandardActionArray = [];
        var oCustomActionArray = [];

        var oActionCatsAndDogs = {};

        var maxLabelWidth = 0;
        var calcWidth = this.calculateWidthHelper('x-grid-cell');

        if (oStandard) {

            for (const sKey in oStandard) {
                if (Object.prototype.hasOwnProperty.call(oStandard, sKey)) {
                    const description = oDescriptions[oStandard[sKey].action];
                    let selectedCategory = null;
                    if (description && description.category && oCategory) {
                        selectedCategory = oCategory[description.category];
                    }
                    if (selectedCategory == null) {
                        oStandardActionArray.push({
                            type: 'action',
                            action: sKey,
                            _dsuid: dSUID,
                            name: oStandard[sKey].title,
                            next: false
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oStandard[sKey].title));
                    } else {
                        if (oActionCatsAndDogs[description.category] == null) {
                            oActionCatsAndDogs[description.category] = [];
                        }

                        if (!selectedCategory.alwaysVisible) {
                            oActionCatsAndDogs[description.category] = [ {
                                type: 'category',
                                action: description.category,
                                _dsuid: dSUID,
                                name: selectedCategory.title,
                                next: true,
                                orderID: selectedCategory.orderId
                            } ];
                            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(selectedCategory.title));
                        } else {
                            oActionCatsAndDogs[description.category].push({
                                type: 'action',
                                action: sKey,
                                _dsuid: dSUID,
                                name: oStandard[sKey].title,
                                next: false
                            });
                            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oStandard[sKey].title));
                        }
                    }
                }
            }
        }

        /*
        if (oCustom) {

            for (let sKey in oCustom) {
                const description = oDescriptions[oCustom[sKey].action];
                let selectedCategory = null;
                if (description && description.category && oCategory) {
                    selectedCategory = oCategory[description.category];
                }
                if (selectedCategory == null) {
                    oCustomActionArray.push({
                        type: 'action',
                        action: sKey,
                        _dsuid: dSUID,
                        name: oCustom[sKey].title,
                        next: false
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oCustom[sKey].title));
                } else {
                    if (oActionCatsAndDogs[description.category] == null)
                        oActionCatsAndDogs[description.category] = [];
                    if (!selectedCategory.alwaysVisible) {
                        oActionCatsAndDogs[description.category] = [{
                            type: 'category',
                            action: description.category,
                            _dsuid: dSUID,
                            name: selectedCategory.title,
                            next: true,
                            orderID: selectedCategory.orderId
                        }];
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(selectedCategory.title));
                    } else {
                        oActionCatsAndDogs[description.category].push({
                            type: 'action',
                            action: sKey,
                            _dsuid: dSUID,
                            name: oCustom[sKey].title,
                            next: false
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oCustom[sKey].title));
                    }
                }
            }
        }
        */
        if (oCustom) {

            for (const sKey in oCustom) {
                if (Object.prototype.hasOwnProperty.call(oCustom, sKey)) {
                    if (oActionCatsAndDogs.__custom == null) {
                        oActionCatsAndDogs.__custom = [];
                    }
                    oActionCatsAndDogs.__custom.push({
                        type: 'action',
                        action: sKey,
                        _dsuid: dSUID,
                        name: oCustom[sKey].title,
                        next: false
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oCustom[sKey].title));
                }
            }
        }

        if (oDeviceActions) {

            for (const sKey in oDeviceActions) {
                if (Object.prototype.hasOwnProperty.call(oDeviceActions, sKey)) {
                    let selectedCategory = null;
                    if (oDeviceActions[sKey].category && oCategory) {
                        selectedCategory = oCategory[oDeviceActions[sKey].category];
                    }
                    if (selectedCategory == null) {
                        oActionArray.push({
                            type: 'action',
                            action: sKey,
                            _dsuid: dSUID,
                            name: oDeviceActions[sKey].title,
                            next: false
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oDeviceActions[sKey].title));
                    } else {
                        if (oActionCatsAndDogs[oDeviceActions[sKey].category] == null) {
                            oActionCatsAndDogs[oDeviceActions[sKey].category] = [];
                        }
                        if (!selectedCategory.alwaysVisible) {
                            oActionCatsAndDogs[oDeviceActions[sKey].category] = [ {
                                type: 'category',
                                action: oDeviceActions[sKey].category,
                                _dsuid: dSUID,
                                name: selectedCategory.title,
                                next: true,
                                orderID: selectedCategory.orderId
                            } ];
                            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(selectedCategory.title));
                        } else {
                            oActionCatsAndDogs[oDeviceActions[sKey].category].push({
                                type: 'action',
                                action: sKey,
                                _dsuid: dSUID,
                                name: oDeviceActions[sKey].title,
                                next: false
                            });
                            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oDeviceActions[sKey].title));
                        }
                    }
                }
            }
        }
        var orderArray = [];
        for (const sKey in oActionCatsAndDogs) {
            if (Object.prototype.hasOwnProperty.call(oActionCatsAndDogs, sKey)) {
                orderArray.push(sKey);
            }
        }

        orderArray.sort(function (aKey, bKey) {
            if (aKey === '__custom') {
                return 1;
            }
            if (bKey === '__custom') {
                return -1;
            }
            var a = oCategory[aKey];
            var b = oCategory[bKey];
            return parseInt(a.orderId.toString(), 10) < parseInt(b.orderId.toString(), 10) ? -1 : 1;
        });

        for (let iIndex = 0; iIndex < orderArray.length; iIndex++) {
            oActionCatsAndDogs[orderArray[iIndex]].sort(upperCaseSort);
            oStateDeviceActionControlStore.add(oActionCatsAndDogs[orderArray[iIndex]]);
            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oActionCatsAndDogs[orderArray[iIndex]][name]));

            if (iIndex < orderArray.length - 1) {
                let fSkip = false;
                if (oActionCatsAndDogs[orderArray[iIndex]].length === 1) {
                    if (oActionCatsAndDogs[orderArray[iIndex]][0].type === 'category') {
                        if (oActionCatsAndDogs[orderArray[iIndex + 1]].length === 1) {
                            if (oActionCatsAndDogs[orderArray[iIndex + 1]][0].type === 'category') {
                                fSkip = true;
                            }
                        }
                    }
                }
                if (!fSkip) {
                    oStateDeviceActionControlStore.add({
                        event: '-',
                        _dsuid: null,
                        name: '-',
                        type: 'spacer',
                        subentry: null
                    });
                }
            }

        }

        if (oStateDeviceActionControlStore.count() > 0 && oStandardActionArray.length + oCustomActionArray.length > 0) {
            oStateDeviceActionControlStore.add({
                event: '-',
                _dsuid: null,
                name: '-',
                type: 'spacer',
                subentry: null
            });
        }
        oStateDeviceActionControlStore.add(oStandardActionArray);
        oStateDeviceActionControlStore.add(oCustomActionArray);
        oStateDeviceActionControlStore.add(oActionArray);

        var me = this;
        var width = this.getWidth() - this.iDefaultWidth * 4 - 2;
        if (wider) {
            width += this.iDefaultWidth;
        }
        this.grids.lvlDeviceActionAction = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: Math.max(width, maxLabelWidth),
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oStateDeviceActionControlStore,
            selModel: {
                mode: 'MULTI'
            },
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ],
            listeners: {
                itemdblclick: function (grid, record) {
                    me.macPanel.remove(me.grids.lvlDeviceActionActionSub);
                    if (record.get('type') === 'category') {
                        me.constructGridLvlDeviceActionActionSubcategories(record.get('_dsuid'), record.get('action'));
                    }
                    if (record.get('type') === 'action') {
                        if (me.isValid()) {
                            me.fireEvent('valueDoubleClicked', me);
                        }
                    }
                },
                itemclick: {
                    fn: function (grid, record) {
                        me.macPanel.remove(me.grids.lvlDeviceActionActionSub);
                        const type = record.get('type');
                        if (type === 'action') {
                            me.fireEvent('valueClicked', me);
                        } else if (type === 'category') {
                            me.constructGridLvlDeviceActionActionSubcategories(record.get('_dsuid'), record.get('action'));
                        }
                    },
                    // single: true,
                    buffer: 300
                }
            }
        });
        addDataIdToElement(this.grids.lvlDeviceActionAction, this.triggerSelectorName + 'GridLvlDeviceActionDeviceClassAction');
        this.grids.lvlDeviceActionAction.parent = this;
        this.macPanel.add(this.grids.lvlDeviceActionAction);
    },
    /*
     * constructGridLvlDeviceActionDevice: function (zoneID) {
     *
     * var oDeviceControlStore = Ext.create('Ext.data.Store', {
     *     fields: [ 'dSID', 'dSUID', 'name', 'icon', 'present'],
     *     proxy: { type: 'memory', reader: { type: 'json', root: 'items' } }
     * });
     *
     * var oDeviceArray = []; if (dss.staticDataModel.deviceCache) { for (var iIndex = 0; iIndex < dss.staticDataModel.deviceCache.length; iIndex++) { var fSkip = false; if
     * (dss.staticDataModel.deviceCache[iIndex].present) if (dss.staticDataModel.deviceCache[iIndex].deviceCustomActions != null) if
     * (Object.keys(dss.staticDataModel.deviceCache[iIndex].deviceCustomActions).length) if (dss.staticDataModel.deviceCache[iIndex].ZoneID == zoneID) {
     * oDeviceControlStore.add(dss.staticDataModel.deviceCache[iIndex]); fSkip = true; } if (!fSkip) if (dss.staticDataModel.deviceCache[iIndex].present) if
     * (dss.staticDataModel.deviceCache[iIndex].deviceStandardActions != null) if (Object.keys(dss.staticDataModel.deviceCache[iIndex].deviceStandardActions).length) if
     * (dss.staticDataModel.deviceCache[iIndex].ZoneID == zoneID) { oDeviceControlStore.add(dss.staticDataModel.deviceCache[iIndex]); } } }
     *
     *
     * oDeviceControlStore.sort('name'); this.grids.lvlDeviceActionDevice = Ext.create('Ext.grid.Panel', {
     *    hideHeaders: true,
     *    height: this.iDefaultHeight,
     *    width: this.iDefaultWidth,
     *    autoScroll: true,
     *    viewConfig: { stripeRows: true },
     *    store: oDeviceControlStore,
     *    columns: [
     *         { sortable: false, hideable: false, text: _('device'), dataIndex: 'name', flex: 1, renderer: this.defMacGridRenderer }
     *    ]
     * });
     * this.grids.lvlDeviceActionDevice.parent = this;
     * this.grids.lvlDeviceActionDevice.on('itemclick', function (grid, record) {
     *     this.parent.grids.lvlDeviceActionAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionAction);
     *     var dSUID = record.get('dSUID');
     *     this.parent.constructGridLvlDeviceActionAction(dSUID);
     * });
     * this.macPanel.add(this.grids.lvlDeviceActionDevice);
     *
     * },
     *//*
         * constructGridLvlDeviceActionRoom: function () { var oRoomArray = []; if (dss.staticDataModel.deviceCache) {
         *     for (var iIndex = 0; iIndex < dss.staticDataModel.deviceCache.length; iIndex++) {
         *        if (dss.staticDataModel.deviceCache[iIndex].present)
         *            if (dss.staticDataModel.deviceCache[iIndex].deviceCustomActions != null)
         *                if (Object.keys(dss.staticDataModel.deviceCache[iIndex].deviceCustomActions).length)
         *                    if (oRoomArray.indexOf(dss.staticDataModel.deviceCache[iIndex].ZoneID) < 0) {
         *                        oRoomArray.push(dss.staticDataModel.deviceCache[iIndex].ZoneID);
         *                    }
         *        if (dss.staticDataModel.deviceCache[iIndex].present)
         *            if (dss.staticDataModel.deviceCache[iIndex].deviceStandardActions != null)
         *                if (Object.keys(dss.staticDataModel.deviceCache[iIndex].deviceStandardActions).length)
         *                    if (oRoomArray.indexOf(dss.staticDataModel.deviceCache[iIndex].ZoneID) < 0) {
         *                        oRoomArray.push(dss.staticDataModel.deviceCache[iIndex].ZoneID);
         *                    }
         *      }
         * }
         * var oRoomDeviceActionControlStore = Ext.create('Ext.data.Store', {
         *     fields: [ 'ZoneID', 'name' ],
         *     proxy: { type: 'memory', reader: { type: 'json', root: 'items' } }
         * });
         * var oArray = [];
         * for (var iIndex = 0; iIndex < oRoomArray.length; iIndex++) {
         *     oArray.push({ ZoneID: oRoomArray[iIndex], name: dss.staticDataModel.getNameForZone(oRoomArray[iIndex]) });
         * }
         * oRoomDeviceActionControlStore.add(oArray);
         * oRoomDeviceActionControlStore.sort('name');
         *
         * this.grids.lvlDeviceActionRoom = Ext.create('Ext.grid.Panel', {
         *     hideHeaders: true,
         *     height: this.iDefaultHeight,
         *     width: this.iDefaultWidth,
         *     autoScroll: true,
         *     viewConfig: { stripeRows: true },
         *     store: oRoomDeviceActionControlStore, columns: [
         *        { sortable: false, hideable: false, text: _('room'), dataIndex: 'name', flex: 1, renderer: function (val, metadata, record, colIdx, store, view) {
         *            return '<table style="height: 16px;width: 100%"><tr><td width="100%">' + record.get('name') + '</td><td>&gt;</td></tr></table>';
         *        } }
         *     ]
         * });
         *
         * this.grids.lvlDeviceActionRoom.parent = this;
         * this.grids.lvlDeviceActionRoom.on('itemclick', function (grid, record) {
         *     this.parent.grids.lvlDeviceActionDevice && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionDevice);
         *     this.parent.grids.lvlDeviceActionAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionAction);
         *     var zoneID = record.get('ZoneID');
         *     this.parent.constructGridLvlDeviceActionDevice(zoneID);
         * });
         * this.macPanel.add(this.grids.lvlDeviceActionRoom);
         * },
         */
    constructGridLvlDeviceAKMValue: function (dSUID, type) {
        const oStore = Ext.create('Ext.data.Store', {
            fields: [ 'name', '_id', 'next', 'icon', 'tooltip', 'index' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        const oDevice = dss.staticDataModel.getDevice(dSUID);

        let fSkip = false;
        if (oDevice.specialActions) {
            if (oDevice.specialActions.triggers) {
                if (oDevice.specialActions.triggers.binaryInputs) {
                    for (const sKey in oDevice.specialActions.triggers.binaryInputs) {
                        if (Object.prototype.hasOwnProperty.call(oDevice.specialActions.triggers.binaryInputs, sKey)) {
                            if (oDevice.specialActions.triggers.binaryInputs[sKey].id == type) {
                                const oItem = {
                                    index: oDevice.specialActions.triggers.binaryInputs[sKey].id,
                                    name: oDevice.specialActions.triggers.binaryInputs[sKey].name,
                                    _id: oDevice.specialActions.triggers.binaryInputs[sKey].value === 'active',
                                    tooltip: '',
                                    icon: null
                                };
                                oStore.add(oItem);
                                fSkip = true;
                            }
                        }
                    }
                }
            }
        }
        if (!fSkip) {
            if (dss.staticDataModel.getDeviceType(oDevice) == 10) { // TNY
                if (oDevice.binaryInputs) {
                    for (let iIndex = 0; iIndex < oDevice.binaryInputs.length; iIndex++) {
                        if (oDevice.binaryInputs[iIndex].type == type) {
                            let oItem = {
                                index: oDevice.binaryInputs[iIndex].index,
                                name: dss.staticDataModel.getNameForAKMValue(oDevice.binaryInputs[iIndex].type, false),
                                _id: false,
                                tooltip: dss.staticDataModel.getTooltipForState(oDevice.binaryInputs[iIndex].type, false),
                                icon: dss.staticDataModel.getImagePathForAKMValue(oDevice.binaryInputs[iIndex].type, false)
                            };
                            oStore.add(oItem);
                            oItem = {
                                index: oDevice.binaryInputs[iIndex].index,
                                name: dss.staticDataModel.getNameForAKMValue(oDevice.binaryInputs[iIndex].type, true),
                                _id: true,
                                tooltip: dss.staticDataModel.getTooltipForState(oDevice.binaryInputs[iIndex].type, true),
                                icon: dss.staticDataModel.getImagePathForAKMValue(oDevice.binaryInputs[iIndex].type, true)
                            };
                            oStore.add(oItem);
                        }
                    }
                }
            } else {
                var bIsWindowHandle = dss.staticDataModel.IsWindowHandle(oDevice);
                if (oDevice.binaryInputs) {
                    for (let iIndex = 0; iIndex < oDevice.binaryInputs.length; iIndex++) {
                        if (!bIsWindowHandle) {
                            let oItem = {
                                index: oDevice.binaryInputs[iIndex].index,
                                name: dss.staticDataModel.getNameForAKMValue(oDevice.binaryInputs[iIndex].type, false),
                                _id: false,
                                tooltip: dss.staticDataModel.getTooltipForState(oDevice.binaryInputs[iIndex].type, false),
                                icon: dss.staticDataModel.getImagePathForAKMValue(oDevice.binaryInputs[iIndex].type, false)
                            };
                            oStore.add(oItem);

                            oItem = {
                                index: oDevice.binaryInputs[iIndex].index,
                                name: dss.staticDataModel.getNameForAKMValue(oDevice.binaryInputs[iIndex].type, true),
                                _id: true,
                                tooltip: dss.staticDataModel.getTooltipForState(oDevice.binaryInputs[iIndex].type, true),
                                icon: dss.staticDataModel.getImagePathForAKMValue(oDevice.binaryInputs[iIndex].type, true)
                            };
                            oStore.add(oItem);
                        } else if (oDevice.binaryInputs[iIndex].type == 15) { // BinaryInputIDWindowTilt
                            let oItem = {
                                name: dss.staticDataModel.getNameForAKMValue(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Closed),
                                _id: dss.whState.WH_Closed,
                                tooltip: dss.staticDataModel.getTooltipForState(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Closed),
                                icon: dss.staticDataModel.getImagePathForAKMValue(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Closed),
                                index: oDevice.binaryInputs[iIndex].index
                            };
                            oStore.add(oItem);
                            oItem = {
                                name: dss.staticDataModel.getNameForAKMValue(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Open),
                                _id: dss.whState.WH_Open,
                                tooltip: dss.staticDataModel.getTooltipForState(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Open),
                                icon: dss.staticDataModel.getImagePathForAKMValue(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Open),
                                index: oDevice.binaryInputs[iIndex].index
                            };
                            oStore.add(oItem);
                            oItem = {
                                name: dss.staticDataModel.getNameForAKMValue(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Tilted),
                                _id: dss.whState.WH_Tilted,
                                tooltip: dss.staticDataModel.getTooltipForState(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Tilted),
                                icon: dss.staticDataModel.getImagePathForAKMValue(oDevice.binaryInputs[iIndex].type, dss.whState.WH_Tilted),
                                index: oDevice.binaryInputs[iIndex].index
                            };
                            oStore.add(oItem);
                        }
                    }
                }
            }
        }

        var me = this;
        this.grids.lvlDeviceAKMValue = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.getWidth() - this.iDefaultWidth * 3 - 2, // 150 + 174,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('type'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlDeviceAKMValue, this.triggerSelectorName + 'GridLvlBinarySensorEventDeviceValue');
        this.grids.lvlDeviceAKMValue.parent = this;
        this.macPanel.add(this.grids.lvlDeviceAKMValue);
    },
    constructGridLvlDeviceAKMs: function (zoneID) {
        var oDeviceControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'dSID', 'dSUID', {
                name: 'name',
                sortType: 'asUCText'
            }, 'icon', 'present', 'type' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        var me = this;
        var oDeviceStore = this.getDeviceAKMStore(zoneID);
        oDeviceStore.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('present') || me.fIgnorePresence) {
                var oDevice = dss.staticDataModel.getDevice(oNewRecord.get('dSUID'));
                var fSkip = false;
                if (oDevice.specialActions) {
                    if (oDevice.specialActions.triggers) {
                        if (oDevice.specialActions.triggers.binaryInputs) {
                            var typeArray = [];
                            for (const sKey in oDevice.specialActions.triggers.binaryInputs) {
                                if (typeArray.indexOf(oDevice.specialActions.triggers.binaryInputs[sKey].id) === -1) {
                                    const oObject = {
                                        dSUID: oNewRecord.get('dSUID'),
                                        name: oNewRecord.get('name'),
                                        icon: oNewRecord.get('icon'),
                                        present: oNewRecord.get('present'),
                                        type: oDevice.specialActions.triggers.binaryInputs[sKey].id
                                    };
                                    typeArray.push(oDevice.specialActions.triggers.binaryInputs[sKey].id);
                                    oDeviceControlStore.add(oObject);
                                    fSkip = true;
                                }
                            }
                        }
                    }
                }
                if (!fSkip) {
                    if (dss.staticDataModel.getDeviceType(oDevice) == 10 /* TNY */) {
                        if (oDevice.binaryInputs) {
                            for (let kIndex = 0; kIndex < oDevice.binaryInputs.length; kIndex++) {
                                const type = oDevice.binaryInputs[kIndex].type;
                                if (type != 255) {
                                    const oInput = oNewRecord.copy();
                                    const typeName = dss.staticDataModel.getNameForAKMType(type);
                                    oInput.data.name = typeName;
                                    oInput.data.type = type;
                                    oDeviceControlStore.add(oInput);
                                }
                            }
                        }
                    } else {
                        oDeviceControlStore.add(oNewRecord);
                    }
                }
            }
        });
        oDeviceControlStore.sort('name');
        this.grids.lvlAKMDevice = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oDeviceControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('device'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlAKMDevice, this.triggerSelectorName + 'GridLvlBinarySensorEventDevice');
        this.grids.lvlAKMDevice.parent = this;
        this.grids.lvlAKMDevice.on('itemclick', function (grid, record) {
            this.parent.grids.lvlDeviceAKMValue && this.parent.macPanel.remove(this.parent.grids.lvlDeviceAKMValue);
            const dSUID = record.get('dSUID');
            const type = record.get('type');
            this.parent.constructGridLvlDeviceAKMValue(dSUID, type);
        });
        this.macPanel.add(this.grids.lvlAKMDevice);
    },
    constructGridLvlAKM: function () {
        var oZoneControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'ZoneID', {
                name: 'name',
                sortType: 'asUCText'
            } ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        const oZoneStore = Ext.data.StoreManager.lookup('triggerConfigurator_zoneAKMStore');
        oZoneStore.each(function (record) {
            const oNewRecord = record.copy();
            if (oNewRecord.get('ZoneID') != 0) {
                oZoneControlStore.add(oNewRecord);
            }
        });
        oZoneControlStore.sort('name');
        this.grids.lvlAKMRoom = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oZoneControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {           // jshint ignore:line
                    return '<table style="height:16px;width:100%" data-zone-id="' + record.get('ZoneID') + '">' +
                        '<tr><td width="100%">' + record.get('name') + '</td><td>&gt;</td></tr>' +
                    '</table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlAKMRoom, this.triggerSelectorName + 'GridLvlBinarySensorEvent');
        this.grids.lvlAKMRoom.parent = this;
        this.grids.lvlAKMRoom.on('itemclick', function (grid, record) {
            this.parent.grids.lvlDeviceScene && this.parent.macPanel.remove(this.parent.grids.lvlDeviceScene);
            this.parent.grids.lvlGroup && this.parent.macPanel.remove(this.parent.grids.lvlGroup);
            this.parent.grids.lvlScene && this.parent.macPanel.remove(this.parent.grids.lvlScene);
            this.parent.grids.lvlDevice && this.parent.macPanel.remove(this.parent.grids.lvlDevice);
            this.parent.grids.lvlDeviceAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceAction);
            this.parent.grids.lvlSensorDevice && this.parent.macPanel.remove(this.parent.grids.lvlSensorDevice);
            this.parent.grids.lvlSensorMessage && this.parent.macPanel.remove(this.parent.grids.lvlSensorMessage);
            this.parent.grids.lvlDeviceAKMValue && this.parent.macPanel.remove(this.parent.grids.lvlDeviceAKMValue);
            this.parent.grids.lvlAKMDevice && this.parent.macPanel.remove(this.parent.grids.lvlAKMDevice);
            this.parent.grids.lvlUserstate && this.parent.macPanel.remove(this.parent.grids.lvlUserstate);
            this.parent.grids.lvlUserstateValue && this.parent.macPanel.remove(this.parent.grids.lvlUserstateValue);
            this.parent.grids.lvlDeviceActionClass && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionClass);
            this.parent.grids.lvlDeviceEventEvent && this.parent.macPanel.remove(this.parent.grids.lvlDeviceEventEvent);
            this.parent.grids.lvlDeviceEventEventSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceEventEventSub);
            this.parent.grids.lvlDeviceActionAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionAction);
            this.parent.grids.lvlDeviceActionActionSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionActionSub);
            this.parent.grids.lvlGlobalApplicationState && this.parent.macPanel.remove(this.parent.grids.lvlGlobalApplicationState);
            this.parent.grids.lvlGlobalApplicationStateGroup && this.parent.macPanel.remove(this.parent.grids.lvlGlobalApplicationStateGroup);
            var zoneID = record.get('ZoneID');
            if (zoneID == -1) {
                zoneID = 0;
            }
            this.parent.constructGridLvlDeviceAKMs(zoneID);
        });
        this.macPanel.add(this.grids.lvlAKMRoom);
    },
    constructGridLvlDeviceActions: function (dSUID, wider) {
        var oDeviceActionControlStore = Ext.create('Ext.data.Store', {
            fields: [ '_id', 'name', 'icon', 'next', 'type' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var maxLabelWidth = 0;
        var calcWidth = this.calculateWidthHelper('x-grid-cell');

        var oDeviceObject = dss.staticDataModel.getDevice(dSUID);
        var fSpecial = false;
        const triggers = oDeviceObject.specialActions.triggers;
        if (triggers) {
            if (triggers.deviceMsg != null) {
                fSpecial = true;
                for (const sKey in triggers.deviceMsg) {
                    if (Object.prototype.hasOwnProperty.call(triggers.deviceMsg, sKey)) {
                        oDeviceActionControlStore.add({
                            _id: triggers.deviceMsg[sKey].id,
                            name: triggers.deviceMsg[sKey].name,
                            icon: null,
                            next: false,
                            type: 'deviceMSG'
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(triggers.deviceMsg[sKey].name));
                    }
                }
            }
            if (triggers.deviceAction != null) {
                fSpecial = true;
                for (const sKey in triggers.deviceAction) {
                    if (Object.prototype.hasOwnProperty.call(triggers.deviceAction, sKey)) {
                        oDeviceActionControlStore.add({
                            _id: triggers.deviceAction[sKey].id,
                            name: triggers.deviceAction[sKey].name,
                            icon: null,
                            next: false,
                            type: 'deviceMSG'
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(triggers.deviceAction[sKey].name));
                    }
                }
            }
            if (triggers.deviceActions != null) {
                fSpecial = true;
                for (const sKey in triggers.deviceActions) {
                    if (Object.prototype.hasOwnProperty.call(triggers.deviceActions, sKey)) {
                        oDeviceActionControlStore.add({
                            _id: triggers.deviceActions[sKey].action,
                            name: triggers.deviceActions[sKey].name,
                            icon: null,
                            next: false,
                            type: 'deviceMSG'
                        });
                        maxLabelWidth = Math.max(maxLabelWidth, calcWidth(triggers.deviceActions[sKey].name));
                    }
                }
            }
        }
        if (!fSpecial) {
            if (oDeviceObject.tasterMode == 15) {
                if (oDeviceObject.buttonType == 2) {
                    oDeviceActionControlStore.add({
                        _id: 1000,
                        name: _('switch down'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('switch down')));
                    oDeviceActionControlStore.add({
                        _id: 1001,
                        name: _('switch up'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('switch up')));
                } else if (oDeviceObject.buttonType >= 16 && oDeviceObject.buttonType <= 21) {
                    oDeviceActionControlStore.add({
                        _id: 1002,
                        name: _('switch down'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('switch down')));
                    oDeviceActionControlStore.add({
                        _id: 1003,
                        name: _('switch up'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('switch up')));
                } else if (oDeviceObject.buttonType == 13 && oDeviceObject.HWInfo.indexOf('F6-02-FF') >= 0) {
                    oDeviceActionControlStore.add({
                        _id: 1100,
                        name: _('1x click lower switch'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('1x click lower switch')));
                    oDeviceActionControlStore.add({
                        _id: 1101,
                        name: _('2x click lower switch'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('2x click lower switch')));
                    oDeviceActionControlStore.add({
                        _id: 1102,
                        name: _('3x click lower switch'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('3x click lower switch')));
                    oDeviceActionControlStore.add({
                        _id: 1103,
                        name: _('4x click lower switch'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('4x click lower switch')));
                    oDeviceActionControlStore.add({
                        _id: 1106,
                        name: _('hold lower switch'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('hold lower switch')));

                    oDeviceActionControlStore.add({
                        _id: 1200,
                        name: _('1x click upper switch'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('1x click upper switch')));
                    oDeviceActionControlStore.add({
                        _id: 1201,
                        name: _('2x click upper switch'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('2x click upper switch')));
                    oDeviceActionControlStore.add({
                        _id: 1202,
                        name: _('3x click upper switch'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('3x click upper switch')));
                    oDeviceActionControlStore.add({
                        _id: 1203,
                        name: _('4x click upper switch'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('4x click upper switch')));
                    oDeviceActionControlStore.add({
                        _id: 1206,
                        name: _('hold upper switch'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('hold upper switch')));
                } else {
                    oDeviceActionControlStore.add({
                        _id: 0,
                        name: _('1x click'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('1x click')));
                    oDeviceActionControlStore.add({
                        _id: 1,
                        name: _('2x click'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('2x click')));
                    oDeviceActionControlStore.add({
                        _id: 2,
                        name: _('3x click'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('3x click')));
                    oDeviceActionControlStore.add({
                        _id: 3,
                        name: _('4x click'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('4x click')));
                    oDeviceActionControlStore.add({
                        _id: 6,
                        name: _('hold'),
                        icon: null,
                        next: false,
                        type: 'deviceMSG'
                    });
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('hold')));
                    /*
                     * oDeviceActionControlStore.add({ _id: 10, name: 'Kurz-Lang', icon: null, next: false });
                     */
                }
            } else {
                oDeviceActionControlStore.add({
                    _id: 50,
                    name: _('local off'),
                    icon: null,
                    next: false,
                    type: 'deviceMSG'
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('local off')));
                oDeviceActionControlStore.add({
                    _id: 51,
                    name: _('local on'),
                    icon: null,
                    next: false,
                    type: 'deviceMSG'
                });
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('local on')));
            }
        }
        var width = this.getWidth() - this.iDefaultWidth * 4 - 2;
        if (wider) {
            width += this.iDefaultWidth;
        }

        var me = this;
        this.grids.lvlDeviceAction = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: Math.max(width, maxLabelWidth),       // 324,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            selModel: {
                mode: 'MULTI'
            },
            store: oDeviceActionControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('taster action'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlDeviceAction, this.triggerSelectorName + 'GridLvlDeviceActionDeviceActionClassAction');
        this.grids.lvlDeviceAction.parent = this;
        this.macPanel.add(this.grids.lvlDeviceAction);
    },
    constructGridLvlAccess: function () {

        var me = this;
        this.grids.lvlAccess = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.getWidth() - this.iDefaultWidth - 2, // 624,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            selModel: {
                mode: 'MULTI'
            },
            store: Ext.data.StoreManager.lookup('triggerConfigurator_accessStore'),
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('scene'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlAccess, this.triggerSelectorName + 'GridLvlAccess');
        this.grids.lvlAccess.parent = this;

        this.grids.lvlAccess.on('beforeitemclick', function (grid, record) {
            if (record.get('_id') == -1) {
                this.getSelectionModel().deselect(record);
                return false;
            }
        });

        this.macPanel.add(this.grids.lvlAccess);

    },
    constructGridLvlWeather: function () {
        var me = this;
        this.grids.lvlWeather = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            selModel: {
                mode: 'MULTI'
            },
            store: Ext.data.StoreManager.lookup('triggerConfigurator_weatherStore'),
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('State'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlWeather, this.triggerSelectorName + 'GridLvlWeather');
        this.grids.lvlWeather.parent = this;

        this.grids.lvlWeather.on('beforeitemclick', function (grid, record) {
            this.parent.grids.lvlSunProtection && this.parent.macPanel.remove(this.parent.grids.lvlSunProtection);
            if (record.get('_id') == -1) {
                this.getSelectionModel().deselect(record);
                return false;
            } else if (record.get('_id') === 'sun') {
                this.parent.constructGridLvlSunProtection();
            }
        });

        this.macPanel.add(this.grids.lvlWeather);
    },
    constructGridLvlSunProtection: function () {
        var me = this;
        this.grids.lvlSunProtection = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: (this.getWidth() - (this.iDefaultWidth * 2)) - 2, // 470,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            selModel: {
                mode: 'MULTI'
            },
            store: Ext.data.StoreManager.lookup('triggerConfigurator_weatherSunStore'),
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('State'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlSunProtection, this.triggerSelectorName + 'GridLvlWeatherSunProtection');
        this.grids.lvlSunProtection.parent = this;

        this.grids.lvlSunProtection.on('beforeitemclick', function (grid, record) {
            if (record.get('_id') == -1) {
                this.getSelectionModel().deselect(record);
                return false;
            }
        });

        this.macPanel.add(this.grids.lvlSunProtection);
    },
    constructGridLvlUserstateValue: function (stateID) {
        const oAddonStateStore = Ext.create('Ext.data.Store', {
            fields: [ '_id', 'name', 'passive', 'icon' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        let oState = null;
        if (dss.staticDataModel.userStates) {
            for (let iIndex = 0; iIndex < dss.staticDataModel.userStates.length; iIndex++) {
                if (dss.staticDataModel.userStates[iIndex].name == stateID) {
                    oState = dss.staticDataModel.userStates[iIndex];
                }
            }
        }
        if (oState != null) {
            let oItem = {
                _id: 'active',
                name: oState.setName,
                icon: dss.staticDataModel.baseImagePath + 'state/uds_state_active.png',
                passive: false
            };
            oAddonStateStore.add(oItem);
            oItem = {
                _id: 'inactive',
                name: oState.resetName,
                icon: dss.staticDataModel.baseImagePath + 'state/uds_state_inactive.png',
                passive: false
            };
            oAddonStateStore.add(oItem);
        }

        var me = this;
        this.grids.lvlUserstateValue = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.getWidth() - this.iDefaultWidth * 2 - 102, // 624,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            selModel: {
                mode: 'MULTI'
            },
            store: oAddonStateStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('State'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlUserstateValue, this.triggerSelectorName + 'GridLvlUserDefinedStateValue');
        this.grids.lvlUserstateValue.parent = this;

        this.grids.lvlUserstateValue.on('beforeitemclick', function (grid, record) {
            if (record.get('_id') == -1) {
                this.getSelectionModel().deselect(record);
                return false;
            }
        });

        this.macPanel.add(this.grids.lvlUserstateValue);
    },
    constructGridLvlUserstate: function () {

        this.grids.lvlUserstate = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth + 100, // 624,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            selModel: {
                mode: 'MULTI'
            },
            store: Ext.data.StoreManager.lookup('triggerConfigurator_userStateStore'),
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('State'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlUserstate, this.triggerSelectorName + 'GridLvlUserDefinedState');
        this.grids.lvlUserstate.parent = this;

        this.grids.lvlUserstate.on('itemclick', function (grid, record) {
            this.parent.grids.lvlUserstateValue && this.parent.macPanel.remove(this.parent.grids.lvlUserstateValue);
            this.parent.constructGridLvlUserstateValue(record.get('_id'));
        });

        this.macPanel.add(this.grids.lvlUserstate);
    },
    constructGridLvlPanic: function () {

        var me = this;
        this.grids.lvlPanic = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: (this.getWidth() - (this.iDefaultWidth * 1)) - 2, // 624,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            selModel: {
                mode: 'MULTI'
            },
            store: Ext.data.StoreManager.lookup('triggerConfigurator_securityStore'),
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('scene'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlPanic, this.triggerSelectorName + 'GridLvlSecurity');
        this.grids.lvlPanic.parent = this;

        this.grids.lvlPanic.on('beforeitemclick', function (grid, record) {
            if (record.get('_id') == -1) {
                this.getSelectionModel().deselect(record);
                return false;
            }
        });

        this.macPanel.add(this.grids.lvlPanic);
    },
    constructGridLvlDeviceActionClass: function (dSUID) {
        var oDeviceControlStore = Ext.create('Ext.data.Store', {
            fields: [ '_id', 'name', 'dsuid', 'type' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var maxLabelWidth = 0;
        var calcWidth = this.calculateWidthHelper('x-grid-cell');

        var oDevice = dss.staticDataModel.getDevice(dSUID);

        var dsidActions = false;
        if (((oDevice.tasterMode == 0) && (oDevice.buttonType != 65)) || (oDevice.tasterMode == 15)) {
            dsidActions = true;
        }

        if (oDevice.deviceEvents && Object.keys(oDevice.deviceEvents).length) {
            oDeviceControlStore.add({
                _id: 1,
                name: _('device events'),
                dsuid: dSUID
            });
            dsidActions = false;
            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('device events')));
        }

        var actions = false;
        if (oDevice.deviceCustomActions && Object.keys(oDevice.deviceCustomActions).length) {
            actions = true;
        }
        if (oDevice.deviceStandardActions && Object.keys(oDevice.deviceStandardActions).length) {
            actions = true;
        }
        if (oDevice.deviceActions && Object.keys(oDevice.deviceActions).length) {
            actions = true;
        }
        if (actions) {
            oDeviceControlStore.add({
                _id: 2,
                name: _('device actions'),
                dsuid: dSUID
            });
            dsidActions = false;
            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('device actions')));
        }
        if (oDevice.hasStates) {
            const oStates = dss.staticDataModel.getDeviceStates(dSUID);
            for (var sKey in oStates) {
                if (Object.prototype.hasOwnProperty.call(oStates, sKey)) {
                    oDeviceControlStore.add({
                        _id: 3,
                        name: oStates[sKey].title,
                        dsuid: dSUID,
                        type: sKey
                    });
                    dsidActions = false;
                    maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oStates[sKey].title));
                }
            }
        }

        if (oDevice.specialActions) {
            if (oDevice.specialActions.triggers) {
                if (oDevice.specialActions.triggers.deviceMsg != null || oDevice.specialActions.triggers.deviceAction != null) {
                    dsidActions = true;
                }
            }
        }
        if (dsidActions) {
            oDeviceControlStore.add({
                _id: 4,
                name: _('taster action'),
                dsuid: dSUID
            });
            maxLabelWidth = Math.max(maxLabelWidth, calcWidth(_('taster action')));
        }

        var hideColumn = oDeviceControlStore.count() == 1 && oDeviceControlStore.getAt(0).get('_id') !== 3;
        this.grids.lvlDeviceActionClass = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: hideColumn ? 0 : Math.max(this.iDefaultWidth, maxLabelWidth),
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oDeviceControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('device'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {           // jshint ignore:line
                    const type = record.get('type');
                    const name = record.get('name');
                    return '<table style="height:16px;width:100%" data-dsuid="' + record.get('dsuid') + '" data-name="' + joinAndCapitalize(name) + '"' +
                        (type ? ' data-type="' + type + '"' : '') + '>' +
                        '<tr><td width="100%">' + name + '</td><td>&gt;</td></tr>' +
                    '</table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlDeviceActionClass, this.triggerSelectorName + 'GridLvlDeviceActionDeviceClass');
        this.grids.lvlDeviceActionClass.parent = this;
        this.grids.lvlDeviceActionClass.on('itemclick', function (grid, record) {
            this.parent.grids.lvlDeviceAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceAction);
            this.parent.grids.lvlDeviceEventEvent && this.parent.macPanel.remove(this.parent.grids.lvlDeviceEventEvent);
            this.parent.grids.lvlDeviceEventEventSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceEventEventSub);
            this.parent.grids.lvlDeviceActionAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionAction);
            this.parent.grids.lvlDeviceActionActionSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionActionSub);
            this.parent.grids.lvlDeviceScene && this.parent.macPanel.remove(this.parent.grids.lvlDeviceScene);
            this.parent.grids.lvlDeviceStateValue && this.parent.macPanel.remove(this.parent.grids.lvlDeviceStateValue);
            switch (record.get('_id')) {
            case 1: // device events
                this.parent.constructGridLvlDeviceEventEvent(dSUID);    // TODO: do all four
                break;
            case 2: // device actions
                this.parent.constructGridLvlDeviceActionAction(dSUID);
                break;
            case 3: // device states
                this.parent.constructGridLvlDeviceStateValue(dSUID, record.get('type'));
                break;
            case 4: // device messages
                this.parent.constructGridLvlDeviceActions(dSUID);
                break;
            }
            // this.parent.constructGridLvlDeviceActionClass(dSUID);
        });
        this.macPanel.add(this.grids.lvlDeviceActionClass);
        if (hideColumn) {
            var oRecordDeviceClass = oDeviceControlStore.getAt(0);
            switch (oRecordDeviceClass.get('_id')) {
            case 1: // device events
                this.constructGridLvlDeviceEventEvent(dSUID, null, hideColumn);
                this.grids.lvlDeviceActionClass.getSelectionModel().select(oRecordDeviceClass);
                return true;
            case 2: // device actions
                this.constructGridLvlDeviceActionAction(dSUID, hideColumn);
                this.grids.lvlDeviceActionClass.getSelectionModel().select(oRecordDeviceClass);
                return true;
            // case 3: // device states
            //     this.constructGridLvlDeviceStateValue(dSUID, oRecordDeviceClass.get('type'));
            //     return true;
            case 4: // device messages
                this.constructGridLvlDeviceActions(dSUID, hideColumn);
                this.grids.lvlDeviceActionClass.getSelectionModel().select(oRecordDeviceClass);
                return true;
            }
        }
        return false;
    },
    calculateWidthHelper: function (styleClass) {
        var el = document.getElementById('WidthHelper');
        if (!el) {
            el = document.createElement('DIV');
            el.id = 'WidthHelper';
            el.className = styleClass;
            el.style.position = 'fixed';
            el.style.top = '0px';
            el.style.left = '0px';
            el.style.marginLeft = '-99999px';
            document.body.appendChild(el);
        }
        return function (str) {
            el.innerHTML = str;
            return el.offsetWidth;
        };
    },
    deviceIsAKM: function (dSUID) {
        let isAKM = false;
        const oDevice = dss.staticDataModel.getDevice(dSUID);

        if (oDevice.binaryInputs.length) {
            for (let input of oDevice.binaryInputs) {
                if (!input.independent) {
                    if ([ 16, 17, 18, 19, 20, 21, 22, 23 ].indexOf(oDevice.buttonType) !== -1) {
                        isAKM = true;
                        break;
                    }
                }
            }
        }
        if (oDevice.deviceEvents && Object.keys(oDevice.deviceEvents).length) {
            isAKM = false;
        }
        if (oDevice.deviceCustomActions && Object.keys(oDevice.deviceCustomActions).length) {
            isAKM = false;
        }
        if (oDevice.deviceStandardActions && Object.keys(oDevice.deviceStandardActions).length) {
            isAKM = false;
        }
        if (oDevice.deviceActions && Object.keys(oDevice.deviceActions).length) {
            isAKM = false;
        }

        if (oDevice.hasStates) {
            var oStates = dss.staticDataModel.getDeviceStates(dSUID);
            if (oStates.length) {
                isAKM = false;
            }
        }

        if (oDevice.specialActions) {
            if (oDevice.specialActions.triggers) {
                if (oDevice.specialActions.triggers.deviceMsg != null || oDevice.specialActions.triggers.deviceAction != null) {
                    isAKM = false;
                }
            }
        }

        return isAKM;
    },
    constructGridLvlDevice: function (zoneID) {
        var me = this;
        var oDeviceControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'dSID', 'dSUID', {
                name: 'name',
                sortType: 'asUCText'
            }, 'icon', 'present' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var maxLabelWidth = 0;
        var calcWidth = this.calculateWidthHelper('x-grid-cell');

        var oDeviceStore = this.getDeviceStore(zoneID);
        oDeviceStore.each(function (record) {
            const oNewRecord = record.copy();
            if (!me.deviceIsAKM(oNewRecord.get('dSUID')) && (oNewRecord.get('present') || me.fIgnorePresence)) {
                oDeviceControlStore.add(oNewRecord);
                maxLabelWidth = Math.max(maxLabelWidth, calcWidth(oNewRecord.get('name')));
            }
        });
        oDeviceControlStore.sort('name');
        this.grids.lvlDevice = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: Math.max(this.iDefaultWidth, maxLabelWidth + 21),    // + 21 for icon width
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oDeviceControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('device'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlDevice, this.triggerSelectorName + 'GridLvlDeviceActionDevice');
        this.grids.lvlDevice.parent = this;
        this.grids.lvlDevice.on('itemclick', function (grid, record) {

            this.parent.grids.lvlDeviceActionClass && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionClass);
            this.parent.grids.lvlDeviceAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceAction);
            this.parent.grids.lvlDeviceEventEvent && this.parent.macPanel.remove(this.parent.grids.lvlDeviceEventEvent);
            this.parent.grids.lvlDeviceEventEventSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceEventEventSub);
            this.parent.grids.lvlDeviceActionAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionAction);
            this.parent.grids.lvlDeviceActionActionSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionActionSub);
            this.parent.grids.lvlDeviceScene && this.parent.macPanel.remove(this.parent.grids.lvlDeviceScene);
            this.parent.grids.lvlDeviceStateValue && this.parent.macPanel.remove(this.parent.grids.lvlDeviceStateValue);

            const dSUID = record.get('dSUID');
            this.parent.constructGridLvlDeviceActionClass(dSUID);
        });
        this.macPanel.add(this.grids.lvlDevice);
    },
    constructGridLvlGroupScene: function (zoneID, groupID, fAll, fForced) {
       var oSceneControlStore = Ext.create('Ext.data.Store', {
            fields: [ '_id', 'name', 'icon', 'reachable', 'checkable', 'checked' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        if (fAll == null) {
            fAll = false;
        }
        if (fForced == null) {
            fForced = false;
        }
        if (zoneID == -1) {
            zoneID = 0;
        }
        var oSceneStore = this.getSceneStore(zoneID, groupID);
        // if ((zoneID == 0) && (groupID > 63)) {  // || groupID == 10 || groupID == 12
        //
        // } else {
            oSceneControlStore.add({
                _id: SHOW_ALL_SCENES_ID,
                name: _('show all scenes'),
                icon: null,
                reachable: true,
                checkable: true,
                checked: fAll

            });
            oSceneControlStore.add({
                _id: -1,
                name: '-',
                passive: true,
                icon: null,
                reachable: true
            });
        // }
        oSceneStore.each(function (record) {
            const oNewRecord = record.copy();
            if (oNewRecord.get('reachable') || fAll) {
                if (oNewRecord.get('_id') < 64 && !oNewRecord.get('stateid')) {
                    oSceneControlStore.add(oNewRecord);
                }
            }
        });

        var me = this;
        this.grids.lvlScene = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.getWidth() - this.iDefaultWidth * 3 - 2, // 324,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            selModel: {
                mode: 'MULTI'
            },
            store: oSceneControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('scene'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlScene, this.triggerSelectorName + 'GridLvlZoneGroupScene');
        this.grids.lvlScene.parent = this;

        this.grids.lvlScene.on('beforeitemclick', function (grid, record) {
            if (record.get('_id') == -1) {
                this.getSelectionModel().deselect(record);
                return false;
            }

            if (record.get('checkable') != null) {
                if (record.get('checkable')) {
                    this.getSelectionModel().deselect(record);
                    const fChecked = !record.get('checked');
                    record.set('checked', fChecked);
                    grid.getSelectionModel().deselectAll();
                    if (record.get('_id') == SHOW_ALL_SCENES_ID) {
                        // alle Stimmungen
                        grid.store.removeAll();

                        grid.store.add({
                            _id: SHOW_ALL_SCENES_ID,
                            name: _('show all scenes'),
                            icon: null,
                            reachable: true,
                            checkable: true,
                            checked: record.get('checked')

                        });
                        grid.store.add({
                            _id: -1,
                            name: '-',
                            passive: true,
                            icon: null,
                            reachable: true
                        });
                        oSceneStore.each(function (record) {
                            var oNewRecord = record.copy();
                            if (oNewRecord.get('reachable') || fChecked) {
                                if (oNewRecord.get('_id') < 64) {
                                    grid.store.add(oNewRecord);
                                }
                            }
                        });

                    }
                }
            }
        });

        this.macPanel.add(this.grids.lvlScene);
    },
    constructGridLvlRoomRoom: function () {
        var me = this;
        var oZoneControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'ZoneID', {
                name: 'name',
                sortType: 'asUCText'
            } ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oZoneStore = Ext.data.StoreManager.lookup('triggerConfigurator_zoneZoneStore');
        oZoneStore.each(function (record) {
            var oNewRecord = record.copy();
            var fSkip = false;
            if (oNewRecord.get('ZoneID') != 0 && !fSkip) {
                oZoneControlStore.add(oNewRecord);
            }
        });
        oZoneControlStore.sort('name');
        oZoneControlStore.add({
            ZoneID: -1,
            name: _('any room')
        });

        this.grids.lvlRoomRoom = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oZoneControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {           // jshint ignore:line
                    return '<table style="height:16px;width:100%" data-zone-id="' + record.get('ZoneID') + '">' +
                        '<tr><td width="100%">' + record.get('name') + '</td><td>&gt;</td></tr>' +
                    '</table>';
                }
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlRoomRoom, this.triggerSelectorName + 'GridLvlZone');
        this.grids.lvlRoomRoom.parent = this;

        this.grids.lvlRoomRoom.on('beforeitemclick', function (grid, record) {
            this.parent.grids.lvlGroup && this.parent.macPanel.remove(this.parent.grids.lvlGroup);
            this.parent.grids.lvlScene && this.parent.macPanel.remove(this.parent.grids.lvlScene);
            var zoneID = record.get('ZoneID');
            if (zoneID == -1) {
                zoneID = 0;
            }
            this.parent.constructGridLvlGroup(zoneID);
        });

        this.macPanel.add(this.grids.lvlRoomRoom);
    },
    constructGridLvlDeviceRoom: function () {
        var me = this;
        var oZoneControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'ZoneID', {
                name: 'name',
                sortType: 'asUCText'
            } ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oZoneStore = Ext.data.StoreManager.lookup('triggerConfigurator_deviceZoneStore');
        oZoneStore.each(function (record) {
            if (record.get('ZoneID') == 0) {
                return;
            }
            const oDeviceStore = me.getDeviceStore(record.get('ZoneID'));
            oDeviceStore.filterBy(function (storeRecord) {
                return storeRecord && !me.deviceIsAKM(storeRecord.get('dSUID'));
            });
            if (oDeviceStore.count() == 0) {
                return;
            }
            oZoneControlStore.add(record.copy());
        });
        oZoneControlStore.sort('name');

        this.grids.lvlDeviceRoom = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oZoneControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {           // jshint ignore:line
                    return '<table style="height:16px;width:100%" data-zone-id="' + record.get('ZoneID') + '">' +
                        '<tr><td width="100%">' + record.get('name') + '</td><td>&gt;</td></tr>' +
                    '</table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlDeviceRoom, this.triggerSelectorName + 'GridLvlDeviceAction');
        this.grids.lvlDeviceRoom.parent = this;

        this.grids.lvlDeviceRoom.on('itemclick', function (grid, record) {
            this.parent.grids.lvlDevice && this.parent.macPanel.remove(this.parent.grids.lvlDevice);
            this.parent.grids.lvlDeviceActionClass && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionClass);
            this.parent.grids.lvlDeviceAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceAction);
            this.parent.grids.lvlDeviceEventEvent && this.parent.macPanel.remove(this.parent.grids.lvlDeviceEventEvent);
            this.parent.grids.lvlDeviceEventEventSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceEventEventSub);
            this.parent.grids.lvlDeviceActionAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionAction);
            this.parent.grids.lvlDeviceActionActionSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionActionSub);
            this.parent.grids.lvlDeviceScene && this.parent.macPanel.remove(this.parent.grids.lvlDeviceScene);
            this.parent.grids.lvlDeviceStateValue && this.parent.macPanel.remove(this.parent.grids.lvlDeviceStateValue);

            var zoneID = record.get('ZoneID');
            if (zoneID == -1) {
                zoneID = 0;
            }
            this.parent.constructGridLvlDevice(zoneID);
        });

        this.macPanel.add(this.grids.lvlDeviceRoom);
    },
    constructGridLvlHigh: function () {
        var oStore = Ext.create('Ext.data.Store', {
            fields: [ {
                name: 'name',
                sortType: 'asUCText'
            }, '_id' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        if (dss.staticDataModel.HLECache) {
            for (let iIndex = 0; iIndex < dss.staticDataModel.HLECache.length; iIndex++) {
                oStore.add({
                    name: dss.staticDataModel.HLECache[iIndex].name,
                    _id: dss.staticDataModel.HLECache[iIndex].id
                });
            }
        }
        oStore.sort('name', 'ASC');
        var me = this;
        this.grids.lvlHighLevelEvent = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: (this.getWidth() - (this.iDefaultWidth * 1)) - 2, // 624,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            selModel: {
                mode: 'MULTI'
            },
            store: oStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('user def. actions'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {       // jshint ignore:line
                    const name = record.get('name');
                    return '<table data-name="' + joinAndCapitalize(name) + '" data-id="' + record.get('_id') + '">' +
                        '<tr style="height:16px;"><td width="100%">' + record.get('name') + '</td></tr>' +
                    '</table>';
                }
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlHighLevelEvent, this.triggerSelectorName + 'GridLvlUserDefinedAction');
        this.grids.lvlHighLevelEvent.parent = this;

        this.macPanel.add(this.grids.lvlHighLevelEvent);
    },
    constructGridLvlGroup: function (zoneID) {
        var oGroupControlStore = Ext.create('Ext.data.Store', {
            fields: [ '_id', 'group', 'name', 'icon', 'reachable' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oGroupStore = this.getGroupStore(zoneID);
        oGroupStore.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('reachable')) {
                var fOK = true;
                switch (oNewRecord.get('group')) {
                    case 0:
                    case 6:
                    case 7:
                    case 8:
                    case 16:
                    case 17:
                    case 18:
                    case 19:
                    case 20:
                    case 21:
                    case 22:
                    case 23:
                        fOK = false;
                        break;
                }
                if (fOK) {
                    oGroupControlStore.add(oNewRecord);
                }
            }
        });
        oGroupControlStore.add({
            group: -1,
            name: '-',
            passive: '-',
            icon: null,
            reachable: true
        });
        var oSceneGroup0Store;
        if (zoneID == 0) {
            oSceneGroup0Store = this.getSceneStore(dss.staticDataModel.getZoneArray()[1].ZoneID, 0);
        } else {
            oSceneGroup0Store = this.getSceneStore(zoneID, 0);
        }
        oSceneGroup0Store.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('reachable')) {
                oGroupControlStore.add(oNewRecord);
            }
        });

        var me = this;
        this.grids.lvlGroup = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            selModel: {
                mode: 'MULTI'
            },
            store: oGroupControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('activity'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlGroup, this.triggerSelectorName + 'GridLvlZoneGroup');
        this.grids.lvlGroup.parent = this;

        this.grids.lvlGroup.on('beforeitemclick', function (grid, record) {
            if (record.get('group') == -1) {
                grid.getSelectionModel().deselect(record);
                return false;
            }

            this.parent.grids.lvlScene && this.parent.macPanel.remove(this.parent.grids.lvlScene);
            var selection = this.parent.grids.lvl1.getSelectionModel().getSelection()[0];
            if (selection && selection.get('_id') == 0) {
                selection = this.parent.grids.lvlRoomRoom && this.parent.grids.lvlRoomRoom.getSelectionModel().getSelection()[0];
                if (selection) {
                    var zoneID = selection.get('ZoneID');
                    var groupID = record.get('group');
                    if (zoneID != null && groupID != null) {
                        this.parent.constructGridLvlGroupScene(zoneID, groupID);
                        this.getSelectionModel().deselectAll();
                        this.getSelectionModel().select(record);
                    }
                    if (groupID == null) {
                        grid.store.each(function (record) {
                            if (record.get('group') != null) {
                                grid.getSelectionModel().deselect(record);
                            }
                        });
                    }
                }
            }
        });

        this.macPanel.add(this.grids.lvlGroup);
    },
    constructGridLvlSensorMessage: function (dSUID) {
        const me = this;
        var oStore = Ext.create('Ext.data.Store', {
            fields: [ 'name', '_id' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oSensorMessageStore = this.getSensorMessage(dSUID);
        oSensorMessageStore.each(function (record) {
            var oNewRecord = record.copy();
            oStore.add(oNewRecord);
        });

        this.grids.lvlSensorMessage = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.getWidth() - this.iDefaultWidth * 3 - 2, // 324,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            selModel: {
                mode: 'MULTI'
            },
            store: oStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('message'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {       // jshint ignore:line
                    const name = record.get('name');
                    return '<table data-name="' + joinAndCapitalize(name) + '" data-id="' + record.get('_id') + '">' +
                        '<tr style="height:16px;"><td width="100%">' + name + '</td></tr>' +
                    '</table>';
                }
            } ],
            listeners: me.makeClickListeners()
        });
        addDataIdToElement(this.grids.lvlSensorMessage, this.triggerSelectorName + 'GridLvlConsumptionEventDeviceMessage');
        this.grids.lvlSensorMessage.parent = this;

        this.macPanel.add(this.grids.lvlSensorMessage);
    },
    constructGridLvlSensorDevice: function (zoneID) {
        var oDeviceControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'dSID', 'dSUID', {
                name: 'name',
                sortType: 'asUCText'
            }, 'icon', 'present' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        var me = this;
        var oDeviceStore = this.getSensorDeviceStore(zoneID);
        oDeviceStore.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('present') || me.fIgnorePresence) {
                oDeviceControlStore.add(oNewRecord);
            }
        });
        oDeviceControlStore.sort('name');
        this.grids.lvlSensorDevice = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oDeviceControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('device'),
                dataIndex: 'name',
                flex: 1,
                renderer: this.defMacGridRenderer
            } ]
        });
        addDataIdToElement(this.grids.lvlSensorDevice, this.triggerSelectorName + 'GridLvlConsumptionEventDevice');
        this.grids.lvlSensorDevice.parent = this;
        this.grids.lvlSensorDevice.on('itemclick', function (grid, record) {
            this.parent.grids.lvlSensorMessage && this.parent.macPanel.remove(this.parent.grids.lvlSensorMessage);
            var dSUID = record.get('dSUID');
            this.parent.constructGridLvlSensorMessage(dSUID);
        });
        this.macPanel.add(this.grids.lvlSensorDevice);
    },
    constructGridLvlSensor: function () {
        var oZoneControlStore = Ext.create('Ext.data.Store', {
            fields: [ 'ZoneID', {
                name: 'name',
                sortType: 'asUCText'
            } ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });

        var oZoneStore = Ext.data.StoreManager.lookup('triggerConfigurator_zoneSensorStore');
        oZoneStore.each(function (record) {
            var oNewRecord = record.copy();
            if (oNewRecord.get('ZoneID') != 0) {
                oZoneControlStore.add(oNewRecord);
            }
        });
        oZoneControlStore.sort('name');
        this.grids.lvlSensorRoom = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oZoneControlStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('room'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {       // jshint ignore:line
                    return '<table style="height:16px;width:100%" data-zone-id="' + record.get('ZoneID') + '">' +
                        '<tr><td width="100%">' + record.get('name') + '</td><td>&gt;</td></tr>' +
                    '</table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlSensorRoom, this.triggerSelectorName + 'GridLvlConsumptionEvent');
        this.grids.lvlSensorRoom.parent = this;
        this.grids.lvlSensorRoom.on('itemclick', function (grid, record) {
            this.parent.grids.lvlDeviceScene && this.parent.macPanel.remove(this.parent.grids.lvlDeviceScene);
            this.parent.grids.lvlDeviceActionClass && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionClass);
            this.parent.grids.lvlGroup && this.parent.macPanel.remove(this.parent.grids.lvlGroup);
            this.parent.grids.lvlScene && this.parent.macPanel.remove(this.parent.grids.lvlScene);
            this.parent.grids.lvlDevice && this.parent.macPanel.remove(this.parent.grids.lvlDevice);
            this.parent.grids.lvlDeviceAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceAction);
            this.parent.grids.lvlSensorDevice && this.parent.macPanel.remove(this.parent.grids.lvlSensorDevice);
            this.parent.grids.lvlSensorMessage && this.parent.macPanel.remove(this.parent.grids.lvlSensorMessage);
            this.parent.grids.lvlUserstate && this.parent.macPanel.remove(this.parent.grids.lvlUserstate);
            this.parent.grids.lvlUserstateValue && this.parent.macPanel.remove(this.parent.grids.lvlUserstateValue);
            this.parent.grids.lvlGlobalApplicationState && this.parent.macPanel.remove(this.parent.grids.lvlGlobalApplicationState);
            this.parent.grids.lvlGlobalApplicationStateGroup && this.parent.macPanel.remove(this.parent.grids.lvlGlobalApplicationStateGroup);
            var zoneID = record.get('ZoneID');
            if (zoneID == -1) {
                zoneID = 0;
            }
            this.parent.constructGridLvlSensorDevice(zoneID);
        });
        this.macPanel.add(this.grids.lvlSensorRoom);
    },
    constructGridLvlGlobalApplicationStateGroup: function (groupId) {
        var oSourceStore = this.getGlobalAppStateStore(groupId);
        var oStore = Ext.create('Ext.data.Store', {
            fields: [ '_id', 'name', 'reachable', 'icon', 'outputMode', 'forced', 'value', 'stateid', 'valueandstate' ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        oSourceStore.each(function (record) {
            var oNewRecord = record.copy();
            // if (oNewRecord.get('GroupID') != 0) {     // TODO: remove
                oStore.add(oNewRecord);
            // }
        });

         var me = this;
         this.grids.lvlGlobalApplicationStateGroup = Ext.create('Ext.grid.Panel', {
             hideHeaders: true,
             height: this.iDefaultHeight,
             width: this.iDefaultWidth * 2 + 12,   // 324,
             autoScroll: true,
             viewConfig: {
                 stripeRows: true
             },
             selModel: {
                 mode: 'MULTI'
             },
             store: oStore,
             columns: [ {
                 sortable: false,
                 hideable: false,
                 text: _('State'),
                 dataIndex: 'name',
                 flex: 1,
                 renderer: this.defMacGridRenderer
             } ],
             listeners: me.makeClickListeners()
         });
         addDataIdToElement(this.grids.lvlGlobalApplicationStateGroup, this.triggerSelectorName + 'GridLvlApplicationStatus');
         this.grids.lvlGlobalApplicationStateGroup.parent = this;

         this.grids.lvlGlobalApplicationStateGroup.on('beforeitemclick', function (grid, record) {      // jshint ignore:line
             // TODO
         });

         this.macPanel.add(this.grids.lvlGlobalApplicationStateGroup);
    },
    constructGridLvlGlobalApplicationState: function () {
        var oStore = Ext.create('Ext.data.Store', {
            fields: [ 'GroupID', {
                name: 'name',
                sortType: 'asUCText'
            } ],
            proxy: {
                type: 'memory',
                reader: {
                    type: 'json',
                    root: 'items'
                }
            }
        });
        var groupIds = [ 64 ];
        var groups = dss.staticDataModel.getGroupForZone(0);
        groups.forEach(function (group) {
            if (groupIds.indexOf(group.group) !== -1) {
                oStore.add({
                    GroupID: group.group,
                    name: _(group.name)
                });
            }
        });
        this.grids.lvlGlobalApplicationState = Ext.create('Ext.grid.Panel', {
            hideHeaders: true,
            height: this.iDefaultHeight,
            width: this.iDefaultWidth * 2 + 12,
            autoScroll: true,
            viewConfig: {
                stripeRows: true
            },
            store: oStore,
            columns: [ {
                sortable: false,
                hideable: false,
                text: _('group'),
                dataIndex: 'name',
                flex: 1,
                renderer: function (val, metadata, record, colIdx, store, view) {   // jshint ignore:line
                    const name = record.get('name');
                    return '<table style="height:16px;width:100%" data-group-id="' + record.get('GroupID') + '" data-name="' + joinAndCapitalize(name) + '">' +
                        '<tr><td width="100%">' + name + '</td><td>&gt;</td></tr>' +
                    '</table>';
                }
            } ]
        });
        addDataIdToElement(this.grids.lvlGlobalApplicationState, this.triggerSelectorName + 'GridLvlApplication');
        this.grids.lvlGlobalApplicationState.parent = this;
        this.grids.lvlGlobalApplicationState.on('itemclick', function (grid, record) {
            this.parent.grids.lvlGlobalApplicationStateGroup && this.parent.macPanel.remove(this.parent.grids.lvlGlobalApplicationStateGroup);
            var groupId = record.get('GroupID');
            if (groupId == -1) {
                groupId = 0;
            }
            this.parent.constructGridLvlGlobalApplicationStateGroup(groupId);
        });
        this.macPanel.add(this.grids.lvlGlobalApplicationState);
    },
    constructMacGrids: function () {
        var lvl1Store;
        if (this.grids.lvl1 == null) {
            var lvl1TempStore = Ext.create('Ext.data.Store', {
                fields: [ '_id', 'name', 'next', 'icon' ],
                proxy: {
                    type: 'memory',
                    reader: {
                        type: 'json',
                        root: 'items'
                    }
                }
            });

            lvl1Store = Ext.data.StoreManager.lookup('triggerConfigurator_l1Store');
            lvl1Store.each(function (record) {
                var oNewRecord = record.copy();
                lvl1TempStore.add(oNewRecord);
            });
            this.grids.lvl1 = Ext.create('Ext.grid.Panel', {
                hideHeaders: true,
                height: this.iDefaultHeight,
                width: this.iDefaultWidth,
                autoScroll: true,
                viewConfig: {
                    stripeRows: true
                },
                selModel: {
                    mode: 'MULTI'
                },
                store: lvl1TempStore,
                columns: [ {
                    sortable: false,
                    hideable: false,
                    text: _('Action'),
                    dataIndex: 'name',
                    flex: 1,
                    renderer: this.defMacGridRenderer
                } ]
            });
            addDataIdToElement(this.grids.lvl1, this.triggerSelectorName + 'GridLvl1');
            this.grids.lvl1.getSelectionModel().store = this.grids.lvl1.store;
            this.grids.lvl1.parent = this;

            this.grids.lvl1.on('itemclick', function (grid, record) {
                this.parent.grids.lvlDeviceAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceAction);
                this.parent.grids.lvlRoomRoom && this.parent.macPanel.remove(this.parent.grids.lvlRoomRoom);
                this.parent.grids.lvlDeviceRoom && this.parent.macPanel.remove(this.parent.grids.lvlDeviceRoom);
                this.parent.grids.lvlHighLevelEvent && this.parent.macPanel.remove(this.parent.grids.lvlHighLevelEvent);
                this.parent.grids.lvlGroup && this.parent.macPanel.remove(this.parent.grids.lvlGroup);
                this.parent.grids.lvlScene && this.parent.macPanel.remove(this.parent.grids.lvlScene);
                this.parent.grids.lvlDevice && this.parent.macPanel.remove(this.parent.grids.lvlDevice);
                this.parent.grids.lvlSensorRoom && this.parent.macPanel.remove(this.parent.grids.lvlSensorRoom);
                this.parent.grids.lvlSensorDevice && this.parent.macPanel.remove(this.parent.grids.lvlSensorDevice);
                this.parent.grids.lvlSensorMessage && this.parent.macPanel.remove(this.parent.grids.lvlSensorMessage);
                this.parent.grids.lvlWeather && this.parent.macPanel.remove(this.parent.grids.lvlWeather);
                this.parent.grids.lvlPanic && this.parent.macPanel.remove(this.parent.grids.lvlPanic);
                this.parent.grids.lvlAccess && this.parent.macPanel.remove(this.parent.grids.lvlAccess);
                this.parent.grids.lvlDeviceAKMValue && this.parent.macPanel.remove(this.parent.grids.lvlDeviceAKMValue);
                this.parent.grids.lvlAKMDevice && this.parent.macPanel.remove(this.parent.grids.lvlAKMDevice);
                this.parent.grids.lvlAKMRoom && this.parent.macPanel.remove(this.parent.grids.lvlAKMRoom);
                this.parent.grids.lvlUserstate && this.parent.macPanel.remove(this.parent.grids.lvlUserstate);
                this.parent.grids.lvlUserstateValue && this.parent.macPanel.remove(this.parent.grids.lvlUserstateValue);
                this.parent.grids.lvlRoomStateState && this.parent.macPanel.remove(this.parent.grids.lvlRoomStateState);
                this.parent.grids.lvlRoomStateRoom && this.parent.macPanel.remove(this.parent.grids.lvlRoomStateRoom);
                this.parent.grids.lvlSunProtection && this.parent.macPanel.remove(this.parent.grids.lvlSunProtection);
                this.parent.grids.lvlDeviceStateRoom && this.parent.macPanel.remove(this.parent.grids.lvlDeviceStateRoom);
                this.parent.grids.lvlDeviceStateDevice && this.parent.macPanel.remove(this.parent.grids.lvlDeviceStateDevice);
                this.parent.grids.lvlDeviceStateState && this.parent.macPanel.remove(this.parent.grids.lvlDeviceStateState);
                this.parent.grids.lvlDeviceStateValue && this.parent.macPanel.remove(this.parent.grids.lvlDeviceStateValue);
                this.parent.grids.lvlDeviceActionClass && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionClass);
                this.parent.grids.lvlDeviceEventEvent && this.parent.macPanel.remove(this.parent.grids.lvlDeviceEventEvent);
                this.parent.grids.lvlDeviceEventEventSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceEventEventSub);
                this.parent.grids.lvlDeviceActionAction && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionAction);
                this.parent.grids.lvlDeviceActionActionSub && this.parent.macPanel.remove(this.parent.grids.lvlDeviceActionActionSub);
                this.parent.grids.lvlGlobalApplicationState && this.parent.macPanel.remove(this.parent.grids.lvlGlobalApplicationState);
                this.parent.grids.lvlGlobalApplicationStateGroup && this.parent.macPanel.remove(this.parent.grids.lvlGlobalApplicationStateGroup);

                var fIsPassive = false;
                if (record.get('passive') != null) {
                    if (record.get('passive') == true) {
                        this.getSelectionModel().deselect(record);
                        fIsPassive = true;
                    }
                }
                if (!fIsPassive) {
                    if (record.get('_id') == 0) {
                        this.getSelectionModel().deselectAll();
                        this.getSelectionModel().select(record);
                        this.parent.constructGridLvlRoomRoom();
                    } else if (record.get('_id') == 1) {
                        this.getSelectionModel().deselectAll();
                        this.getSelectionModel().select(record);
                        this.parent.constructGridLvlDeviceRoom();
                    } else if (record.get('_id') == 2) {
                        this.getSelectionModel().deselectAll();
                        this.getSelectionModel().select(record);
                        this.parent.constructGridLvlHigh();
                    } else if (record.get('_id') == 3) {
                        this.getSelectionModel().deselectAll();
                        this.getSelectionModel().select(record);
                        this.parent.constructGridLvlSensor();
                    } else if (record.get('_id') == 4) {
                        this.getSelectionModel().deselectAll();
                        this.getSelectionModel().select(record);
                        this.parent.constructGridLvlAccess();
                    } else if (record.get('_id') == 5) {
                        this.getSelectionModel().deselectAll();
                        this.getSelectionModel().select(record);
                        this.parent.constructGridLvlWeather();
                    } else if (record.get('_id') == 6) {
                        this.getSelectionModel().deselectAll();
                        this.getSelectionModel().select(record);
                        this.parent.constructGridLvlPanic();
                    } else if (record.get('_id') == 7) {
                        this.getSelectionModel().deselectAll();
                        this.getSelectionModel().select(record);
                        this.parent.constructGridLvlAKM();
                    } else if (record.get('_id') == 8) {
                        this.getSelectionModel().deselectAll();
                        this.getSelectionModel().select(record);
                        this.parent.constructGridLvlUserstate();
                    } else if (record.get('_id') == 9) {
                        this.getSelectionModel().deselectAll();
                        this.getSelectionModel().select(record);
                        this.parent.constructGridLvlRoomStateRoom();
                    // } else if (record.get('_id') == 10) {
                    //     this.getSelectionModel().deselectAll();
                    //     this.getSelectionModel().select(record);
                    //     this.parent.constructGridLvlDeviceStateRoom();         // there's no such method in the code!!!
                    } else if (record.get('_id') == 12) {
                        this.getSelectionModel().deselectAll();
                        this.getSelectionModel().select(record);
                        this.parent.constructGridLvlGlobalApplicationState();
                    } else {
                        grid.store.each(function (record) {
                            if (record.get('_id') < 7) {
                                grid.getSelectionModel().deselect(record);
                            }
                        });
                    }
                }
                this.parent.macPanel.doLayout();
            });
        } else {
            this.grids.lvl1.getStore().removeAll();
            var mThis = this;
            lvl1Store = Ext.data.StoreManager.lookup('triggerConfigurator_l1Store');
            lvl1Store.each(function (record) {
                var oNewRecord = record.copy();
                mThis.grids.lvl1.getStore().add(oNewRecord);
            });
        }

    },
    constructor: function (config) {
        this.addEvents('valueDoubleClicked');
        var mThis = this;
        this.grids = {};
        this.fEnableUsergroup = false;
        if (config.enableUsergroup != null) {
            this.fEnableUsergroup = config.enableUsergroup;
        }

        this.ShowUserstates = true;
        if (config.userStates) {
            this.ShowUserstates = config.userStates;
        }

        this.fHeatingOperationMode = false;
        if (config.heatingOperationMode != null) {
            this.fHeatingOperationMode = config.heatingOperationMode;
        }

        this.fIgnorePresence = false;
        if (config.ignorePresence != null) {
            this.fIgnorePresence = config.ignorePresence;
        }

        this.fShowEventService = false;
        if (config.ignorePresence != null) {
            this.fShowEventService = config.ignorePresence;
        }

        this.triggerSelectorName = config.myID
            ? config.myID
            : config.autoEl && config.autoEl['data-id']
                ? config.autoEl['data-id']
                : '';

        Ext.data.StoreManager.on('clear', function () {
            mThis.initStores(mThis.ShowUserstates);
            mThis.constructMacGrids();
        });

        this.initStores(this.ShowUserstates);
        this.constructMacGrids();

        this.macPanel = Ext.create('Ext.panel.Panel', {
            border: '0',
            height: 320,
            layout: {
                type: 'hbox'
            },
            items: [ this.grids.lvl1 ],
            overflowX: 'scroll',
            overflowY: 'auto'
        });
        Ext.override(this.macPanel, {
            add: function () {
                this.callParent(arguments);
                this.body.dom.scrollLeft = 9999;    // scroll to the maximum right
            }
        });

        Ext.apply(this, {
            items: [ this.macPanel ]
        });

        this.callParent(arguments);
    },
    reset: function () {
        this.setValue([]);
    },
    setValue: function (values) {
        this.grids.lvlDeviceAction && this.macPanel.remove(this.grids.lvlDeviceAction);
        this.grids.lvlRoomRoom && this.macPanel.remove(this.grids.lvlRoomRoom);
        this.grids.lvlDeviceRoom && this.macPanel.remove(this.grids.lvlDeviceRoom);
        this.grids.lvlHighLevelEvent && this.macPanel.remove(this.grids.lvlHighLevelEvent);
        this.grids.lvlGroup && this.macPanel.remove(this.grids.lvlGroup);
        this.grids.lvlScene && this.macPanel.remove(this.grids.lvlScene);
        this.grids.lvlDevice && this.macPanel.remove(this.grids.lvlDevice);
        this.grids.lvlSensorRoom && this.macPanel.remove(this.grids.lvlSensorRoom);
        this.grids.lvlSensorDevice && this.macPanel.remove(this.grids.lvlSensorDevice);
        this.grids.lvlSensorMessage && this.macPanel.remove(this.grids.lvlSensorMessage);
        this.grids.lvlWeather && this.macPanel.remove(this.grids.lvlWeather);
        this.grids.lvlPanic && this.macPanel.remove(this.grids.lvlPanic);
        this.grids.lvlAccess && this.macPanel.remove(this.grids.lvlAccess);
        this.grids.lvlDeviceAKMValue && this.macPanel.remove(this.grids.lvlDeviceAKMValue);
        this.grids.lvlAKMDevice && this.macPanel.remove(this.grids.lvlAKMDevice);
        this.grids.lvlAKMRoom && this.macPanel.remove(this.grids.lvlAKMRoom);
        this.grids.lvlUserstate && this.macPanel.remove(this.grids.lvlUserstate);
        this.grids.lvlUserstateValue && this.macPanel.remove(this.grids.lvlUserstateValue);
        this.grids.lvlRoomStateState && this.macPanel.remove(this.grids.lvlRoomStateState);
        this.grids.lvlRoomStateRoom && this.macPanel.remove(this.grids.lvlRoomStateRoom);
        this.grids.lvlSunProtection && this.macPanel.remove(this.grids.lvlSunProtection);
        this.grids.lvlDeviceStateRoom && this.macPanel.remove(this.grids.lvlDeviceStateRoom);
        this.grids.lvlDeviceStateDevice && this.macPanel.remove(this.grids.lvlDeviceStateDevice);
        this.grids.lvlDeviceStateState && this.macPanel.remove(this.grids.lvlDeviceStateState);
        this.grids.lvlDeviceStateValue && this.macPanel.remove(this.grids.lvlDeviceStateValue);
        this.grids.lvlDeviceActionClass && this.macPanel.remove(this.grids.lvlDeviceActionClass);
        this.grids.lvlDeviceEventEvent && this.macPanel.remove(this.grids.lvlDeviceEventEvent);
        this.grids.lvlDeviceEventEventSub && this.macPanel.remove(this.grids.lvlDeviceEventEventSub);
        this.grids.lvlDeviceActionAction && this.macPanel.remove(this.grids.lvlDeviceActionAction);
        this.grids.lvlDeviceActionActionSub && this.macPanel.remove(this.grids.lvlDeviceActionActionSub);
        this.grids.lvlGlobalApplicationState && this.macPanel.remove(this.grids.lvlGlobalApplicationState);
        this.grids.lvlGlobalApplicationStateGroup && this.macPanel.remove(this.grids.lvlGlobalApplicationStateGroup);

        let fGridConstructed = false;

        this.grids.lvl1.getSelectionModel().deselectAll();
        if (values == null || values && !values.length) {
            return;
        }

        for (let vIndex = 0; vIndex < values.length; vIndex++) {
            const oValue = values[vIndex];
            switch (oValue.type) {
            case 'device-named-event': {
                const oDevice = dss.staticDataModel.getDevice(oValue.dsuid);
                const zoneID = oDevice.ZoneID;
                const fAdvanced = Boolean(oDevice.deviceEvents[oValue.id] && oDevice.deviceEvents[oValue.id].accessLevel === 'advanced');
                if (!fGridConstructed) {
                    this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 1, false, false, true), true);
                    this.constructGridLvlDeviceRoom();
                    const oRecordZone = this.grids.lvlDeviceRoom.store.find('ZoneID', zoneID, 0, false, false, true);
                    if (oRecordZone != -1) {
                        this.grids.lvlDeviceRoom.getSelectionModel().select(oRecordZone);
                    } else {
                        break;
                    }
                    this.constructGridLvlDevice(zoneID);
                    const oRecordDevice = this.grids.lvlDevice.store.find('dSUID', oValue.dsuid, 0, false, false, true);
                    if (oRecordDevice != -1) {
                        this.grids.lvlDevice.getSelectionModel().select(oRecordDevice);
                    } else {
                        break;
                    }
                    const fConstructed = this.constructGridLvlDeviceActionClass(oValue.dsuid);
                    const oRecordDeviceClass = this.grids.lvlDeviceActionClass.store.find('_id', 1, 0, false, false, true);
                    if (oRecordDeviceClass != -1) {
                        this.grids.lvlDeviceActionClass.getSelectionModel().select(oRecordDeviceClass);
                    } else {
                        break;
                    }
                    if (!fConstructed) {
                        this.constructGridLvlDeviceEventEvent(oValue.dsuid, fAdvanced);
                    }
                }
                const iIndex = this.grids.lvlDeviceEventEvent.store.find('event', oValue.id, 0, false, false, true);
                if (iIndex != -1) {
                    this.grids.lvlDeviceEventEvent.getSelectionModel().select(iIndex, true);
                } else if (oDevice.deviceEvents[oValue.id]) {
                    if (oDevice.deviceEvents[oValue.id].category != null) {
                        const jIndex = this.grids.lvlDeviceEventEvent.store.find('event', oDevice.deviceEvents[oValue.id].category, 0, false, false, true);
                        if (jIndex != -1) {
                            this.grids.lvlDeviceEventEvent.getSelectionModel().select(jIndex, true);
                            this.constructGridLvlDeviceEventEventSubcategories(oValue.dsuid, oDevice.deviceEvents[oValue.id].category, fAdvanced);
                            const kIndex = this.grids.lvlDeviceEventEventSub.store.find('event', oValue.id, 0, false, false, true);
                            if (kIndex != -1) {
                                this.grids.lvlDeviceEventEventSub.getSelectionModel().select(kIndex, true);
                            }
                        }
                    }
                }
                break;
            }
            case 'device-named-action': {
                const zoneID = dss.staticDataModel.getDevice(oValue.dsuid).ZoneID;
                if (!fGridConstructed) {
                    this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 1, false, false, true), true);
                    this.constructGridLvlDeviceRoom();
                    const oRecordZone = this.grids.lvlDeviceRoom.store.find('ZoneID', zoneID, 0, false, false, true);
                    if (oRecordZone != -1) {
                        this.grids.lvlDeviceRoom.getSelectionModel().select(oRecordZone);
                    } else {
                        break;
                    }
                    this.constructGridLvlDevice(zoneID);
                    let oRecordDevice = this.grids.lvlDevice.store.find('dSUID', oValue.dsuid, 0, false, false, true);
                    if (oRecordDevice != -1) {
                        this.grids.lvlDevice.getSelectionModel().select(oRecordDevice);
                    } else {
                        break;
                    }
                    const fConstructed = this.constructGridLvlDeviceActionClass(oValue.dsuid);

                    oRecordDevice = this.grids.lvlDeviceActionClass.store.find('_id', 2, 0, false, false, true);
                    if (oRecordDevice != -1) {
                        this.grids.lvlDeviceActionClass.getSelectionModel().select(oRecordDevice);
                    } else {
                        break;
                    }
                    if (!fConstructed) {
                        this.constructGridLvlDeviceActionAction(oValue.dsuid);
                    }
                }
                const iIndex = this.grids.lvlDeviceActionAction.store.find('action', oValue.id, 0, false, false, true);
                if (iIndex != -1) {
                    this.grids.lvlDeviceActionAction.getSelectionModel().select(iIndex, true);
                } else {
                    const oDevice = dss.staticDataModel.getDevice(oValue.dsuid);
                    let category = null;

                    const oCustom = oDevice.deviceCustomActions;
                    const oStandard = oDevice.deviceStandardActions;
                    const oDeviceActions = oDevice.deviceActions;
                    const oDescriptions = oDevice.deviceActionDescription;

                    if (oStandard) {
                        if (oStandard[oValue.id]) {
                            const description = oDescriptions[oStandard[oValue.id].action];
                            if (description.category != null) {
                                category = description.category;
                            }
                        }
                    }

                    if (oCustom) {
                        if (oCustom[oValue.id]) {
                            const description = oDescriptions[oCustom[oValue.id].action];
                            if (description.category != null) {
                                category = description.category;
                            }
                        }
                    }

                    if (oDeviceActions) {
                        if (oDeviceActions[oValue.id]) {
                            const description = oDescriptions[oDeviceActions[oValue.id].action];
                            if (description.category != null) {
                                category = description.category;
                            }
                        }
                    }

                    if (category != null) {
                        const jIndex = this.grids.lvlDeviceActionAction.store.find('action', category, 0, false, false, true);
                        if (jIndex != -1) {
                            this.grids.lvlDeviceActionAction.getSelectionModel().select(jIndex, true);
                            this.constructGridLvlDeviceActionActionSubcategories(oValue.dsuid, category);

                            const kIndex = this.grids.lvlDeviceActionActionSub.store.find('action', oValue.id, 0, false, false, true);
                            if (kIndex != -1) {
                                this.grids.lvlDeviceActionActionSub.getSelectionModel().select(kIndex, true);
                            }
                        }
                    }

                }
                break;
            }
            case 'device-sensor': {
                const zoneID = dss.staticDataModel.getDevice(oValue.dsuid).ZoneID;
                if (!fGridConstructed) {
                    this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 3, false, false, true), true);
                    this.constructGridLvlSensor();
                    const oRecordZone = this.grids.lvlSensorRoom.store.find('ZoneID', zoneID, 0, false, false, true);
                    if (oRecordZone != -1) {
                        this.grids.lvlSensorRoom.getSelectionModel().select(oRecordZone);
                    } else {
                        break;
                    }
                    this.constructGridLvlSensorDevice(zoneID);
                    const oRecordDevice = this.grids.lvlSensorDevice.store.find('dSUID', oValue.dsuid, 0, false, false, true);
                    if (oRecordDevice != -1) {
                        this.grids.lvlSensorDevice.getSelectionModel().select(oRecordDevice);
                    } else {
                        break;
                    }
                    this.constructGridLvlSensorMessage(oValue.dsuid);
                }
                const iIndex = this.grids.lvlSensorMessage.store.find('_id', oValue.eventid, 0, false, false, true);
                if (iIndex != -1) {
                    this.grids.lvlSensorMessage.getSelectionModel().select(iIndex, true);
                }
                break;
            }
            case 'device-binary-input': {
                const zoneID = dss.staticDataModel.getDevice(oValue.dsuid).ZoneID;
                if (!fGridConstructed) {
                    this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 1, false, false, true), true);
                    this.constructGridLvlRoom();
                    const oRecordZone = this.grids.lvlDeviceRoom.store.find('ZoneID', zoneID, 0, false, false, true);
                    if (oRecordZone != -1) {
                        this.grids.lvlDeviceRoom.getSelectionModel().select(oRecordZone);
                    } else {
                        break;
                    }
                    this.constructGridLvlDevice(zoneID);
                    const oRecordDevice = this.grids.lvlDevice.store.find('dSUID', oValue.dsuid, 0, false, false, true);
                    if (oRecordDevice != -1) {
                        this.grids.lvlDevice.getSelectionModel().select(oRecordDevice);
                    } else {
                        break;
                    }
                    this.constructGridLvlDeviceActions(oValue.dsuid);
                }
                if (oValue.state == 1) {
                    const iIndex = this.grids.lvlDeviceAction.store.find('_id', 1002, 0, false, false, true);
                    if (iIndex != -1) {
                        this.grids.lvlDeviceAction.getSelectionModel().select(iIndex, true);
                    }
                }
                if (oValue.state == 0) {
                    const iIndex = this.grids.lvlDeviceAction.store.find('_id', 1003, 0, false, false, true);
                    if (iIndex != -1) {
                        this.grids.lvlDeviceAction.getSelectionModel().select(iIndex, true);
                    }
                }

                break;
            }
            case 'device-action':
            case 'device-msg': {
                const zoneID = dss.staticDataModel.getDevice(oValue.dsuid).ZoneID;
                if (!fGridConstructed) {
                    this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 1, false, false, true), true);

                    this.constructGridLvlDeviceRoom();
                    const oRecordZone = this.grids.lvlDeviceRoom.store.find('ZoneID', zoneID, 0, false, false, true);
                    if (oRecordZone != -1) {
                        this.grids.lvlDeviceRoom.getSelectionModel().select(oRecordZone);
                    } else {
                        break;
                    }
                    this.constructGridLvlDevice(zoneID);
                    const oRecordDevice = this.grids.lvlDevice.store.find('dSUID', oValue.dsuid, 0, false, false, true);
                    if (oRecordDevice != -1) {
                        this.grids.lvlDevice.getSelectionModel().select(oRecordDevice);
                    } else {
                        break;
                    }
                    const fConstructed = this.constructGridLvlDeviceActionClass(oValue.dsuid);
                    const oRecordDeviceClass = this.grids.lvlDeviceActionClass.store.find('_id', 4, 0, false, false, true);
                    if (oRecordDeviceClass != -1) {
                        this.grids.lvlDeviceActionClass.getSelectionModel().select(oRecordDeviceClass);
                    } else {
                        break;
                    }
                    if (!fConstructed) {
                        this.constructGridLvlDeviceActions(oValue.dsuid);
                    }
                }
                if (oValue.action != null) {
                    const iIndex = this.grids.lvlDeviceAction.store.find('_id', oValue.action, 0, false, false, true);
                    if (iIndex != -1) {
                        this.grids.lvlDeviceAction.getSelectionModel().select(iIndex, true);
                    }
                } else if ((oValue.msg == 0) && (oValue.buttonIndex != null)) {
                    if (oValue.buttonIndex == 2) {
                        const iIndex = this.grids.lvlDeviceAction.store.find('_id', 1000, 0, false, false, true);
                        if (iIndex != -1) {
                            this.grids.lvlDeviceAction.getSelectionModel().select(iIndex, true);
                        }
                    }
                    if (oValue.buttonIndex == 1) {
                        const iIndex = this.grids.lvlDeviceAction.store.find('_id', 1001, 0, false, false, true);
                        if (iIndex != -1) {
                            this.grids.lvlDeviceAction.getSelectionModel().select(iIndex, true);
                        }
                    }
                } else if (oValue.msg == 7) {
                    var msg = 0;
                    if (oValue.buttonIndex != null) {
                        msg += 1000 + oValue.buttonIndex * 100;
                    }
                    const iIndex = this.grids.lvlDeviceAction.store.find('_id', msg, 0, false, false, true);
                    if (iIndex != -1) {
                        this.grids.lvlDeviceAction.getSelectionModel().select(iIndex, true);
                    }
                } else if (oValue.msg == 8) {
                    var msg = 1;
                    if (oValue.buttonIndex != null) {
                        msg += 1000 + oValue.buttonIndex * 100;
                    }
                    const iIndex = this.grids.lvlDeviceAction.store.find('_id', msg, 0, false, false, true);
                    if (iIndex != -1) {
                        this.grids.lvlDeviceAction.getSelectionModel().select(iIndex, true);
                    }
                } else if (oValue.msg == 9) {
                    var msg = 2;
                    if (oValue.buttonIndex != null) {
                        msg += 1000 + oValue.buttonIndex * 100;
                    }
                    const iIndex = this.grids.lvlDeviceAction.store.find('_id', msg, 0, false, false, true);
                    if (iIndex != -1) {
                        this.grids.lvlDeviceAction.getSelectionModel().select(iIndex, true);
                    }
                } else {
                    var msg = oValue.msg;
                    if (oValue.buttonIndex != null) {
                        msg += 1000 + oValue.buttonIndex * 100;
                    }
                    const iIndex = this.grids.lvlDeviceAction.store.find('_id', msg, 0, false, false, true);
                    if (iIndex != -1) {
                        this.grids.lvlDeviceAction.getSelectionModel().select(iIndex, true);
                    }
                }

                break;
            }
            case 'undo-zone-scene': {
                if (oValue.zone == 0) {
                    switch (oValue.scene) {
                        case 65:
                        case 74:
                        case 83:
                        case 84:
                        case 85: {
                            if (!fGridConstructed) {
                                this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 6, false, false, true), true);
                                this.constructGridLvlPanic();
                            }
                            const oRecordScene = this.grids.lvlPanic.store.find('_id', 1000 + oValue.scene, 0, false, false, true);
                            if (oRecordScene != -1) {
                                this.grids.lvlPanic.getSelectionModel().select(oRecordScene, true);
                            }
                            break;
                        }
                    }
                }
                break;
            }
            case 'addon-state-change': {
                if (oValue['addon-id'] === 'system-addon-user-defined-states') {
                    const stateName = oValue.name;
                    if (!fGridConstructed) {
                        this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 8, false, false, true), true);
                        this.constructGridLvlUserstate();
                    }
                    const oRecordScene = this.grids.lvlUserstate.store.find('_id', stateName, 0, false, false, true);
                    if (oRecordScene != -1) {
                        this.grids.lvlUserstate.getSelectionModel().select(oRecordScene, true);
                        this.constructGridLvlUserstateValue(stateName);
                        const oRecordValue = this.grids.lvlUserstateValue.store.find('_id', oValue.state, 0, false, false, true);
                        if (oRecordValue != -1) {
                            this.grids.lvlUserstateValue.getSelectionModel().select(oRecordValue, true);
                        }
                    }
                    break;
                }
                break;
            }
            case 'state-change': {

                const idString = oValue.name;
                const valueString = oValue.state;
                const oUserStates = dss.staticDataModel.getUserStates();
                let fSkipAfterUserstateFound = false;
                if (oUserStates) {
                    for (let jIndex = 0; jIndex < oUserStates.length; jIndex++) {
                        if (oUserStates[jIndex].name == idString) {
                            if (!fGridConstructed) {
                                this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 8, false, false, true), true);
                                this.constructGridLvlUserstate();
                            }
                            const oRecordScene = this.grids.lvlUserstate.store.find('_id', idString, 0, false, false, true);
                            if (oRecordScene != -1) {
                                this.grids.lvlUserstate.getSelectionModel().select(oRecordScene, true);
                                this.constructGridLvlUserstateValue(idString);
                                const oRecordValue = this.grids.lvlUserstateValue.store.find('_id', valueString, 0, false, false, true);
                                if (oRecordValue != -1) {
                                    this.grids.lvlUserstateValue.getSelectionModel().select(oRecordValue, true);
                                }
                            }
                            fSkipAfterUserstateFound = true;
                        }
                    }
                }
                if (fSkipAfterUserstateFound) {
                    break;
                }

                var oArray = oValue.name.split('.');
                if (idString.indexOf('zone.0.group') > -1) {    // global application states
                    const groupId = oArray[3];
                    if (groupId) {
                        this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 12, false, false, true), true);
                        if (!fGridConstructed) {
                            this.constructGridLvlGlobalApplicationState();
                            var oRecordGroup = this.grids.lvlGlobalApplicationState.store.find('GroupID', groupId, 0, false, false, true);
                            if (oRecordGroup != -1) {
                                this.grids.lvlGlobalApplicationState.getSelectionModel().select(oRecordGroup);
                            } else {
                                break;
                            }
                            this.constructGridLvlGlobalApplicationStateGroup(groupId);
                        }
                        const oState = this.grids.lvlGlobalApplicationStateGroup.store.findBy(function (record) {
                            return record.get('stateid') === idString && record.get('value') === oValue.state;
                        });
                        if (oState != -1) {
                            this.grids.lvlGlobalApplicationStateGroup.getSelectionModel().select(oState);
                        } else {
                            break;
                        }
                        break;
                    }
                }
                // name, state: 'active', 'inactive'
                if (oArray.length === 3) {
                    if (oArray[0] === 'dev') {
                        const oDevice = dss.staticDataModel.getDevice(oArray[1]);
                        if (oDevice) {
                            if (oDevice.hasStates && dss.staticDataModel.getDeviceStates(oArray[1])) {
                                if (!fGridConstructed) {
                                    this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 1, false, false, true), true);
                                    this.constructGridLvlDeviceRoom();
                                    const oRecordZone = this.grids.lvlDeviceRoom.store.find('ZoneID', oDevice.ZoneID, 0, false, false, true);
                                    if (oRecordZone != -1) {
                                        this.grids.lvlDeviceRoom.getSelectionModel().select(oRecordZone);
                                    } else {
                                        break;
                                    }
                                    this.constructGridLvlDevice(oDevice.ZoneID);
                                    const oRecordDevice = this.grids.lvlDevice.store.find('dSUID', oDevice.dSUID, 0, false, false, true);
                                    if (oRecordDevice != -1) {
                                        this.grids.lvlDevice.getSelectionModel().select(oRecordDevice);
                                    } else {
                                        break;
                                    }
                                    const fConstructed = this.constructGridLvlDeviceActionClass(oDevice.dSUID);
                                    let oRecordDeviceClass = -1;
                                    for (let iIndex = 0; iIndex < this.grids.lvlDeviceActionClass.store.count(); iIndex++) {
                                        if (this.grids.lvlDeviceActionClass.store.getAt(iIndex).get('_id') == 3) {
                                            if (this.grids.lvlDeviceActionClass.store.getAt(iIndex).get('type') == oArray[2]) {
                                                oRecordDeviceClass = iIndex;
                                            }
                                        }
                                    }
                                    if (oRecordDeviceClass != -1) {
                                        this.grids.lvlDeviceActionClass.getSelectionModel().select(oRecordDeviceClass);
                                    } else {
                                        break;
                                    }
                                    if (!fConstructed) {
                                        this.constructGridLvlDeviceStateValue(oArray[1], oArray[2]);
                                    }
                                }
                                const oRecordValue = this.grids.lvlDeviceStateValue.store.find('type', oValue.state, 0, false, false, true);
                                if (oRecordValue != -1) {
                                    this.grids.lvlDeviceStateValue.getSelectionModel().select(oRecordValue, true);
                                }
                                break;
                            } else {
                                if (!fGridConstructed) {
                                    this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 7, false, false, true), true);
                                    this.constructGridLvlAKM();
                                    const oRecordZone = this.grids.lvlAKMRoom.store.find('ZoneID', oDevice.ZoneID, 0, false, false, true);
                                    if (oRecordZone != -1) {
                                        this.grids.lvlAKMRoom.getSelectionModel().select(oRecordZone, true);
                                        this.constructGridLvlDeviceAKMs(oDevice.ZoneID);
                                        let oRecordDevice = -1;
                                        let type = null;
                                        for (let iIndex = 0; iIndex < this.grids.lvlAKMDevice.store.count(); iIndex++) {
                                            if (this.grids.lvlAKMDevice.store.getAt(iIndex).get('dSUID') == oArray[1]) {
                                                oRecordDevice = iIndex;
                                                type = this.grids.lvlAKMDevice.store.getAt(iIndex).get('type');
                                                break;
                                            }
                                        }
                                        if (oRecordDevice != -1) {
                                            oRecordDevice = this.grids.lvlAKMDevice.store.find('dSUID', oArray[1], 0, false, false, true);
                                        }
                                        if (oRecordDevice != -1) {
                                            this.grids.lvlAKMDevice.getSelectionModel().select(oRecordDevice, true);
                                            this.constructGridLvlDeviceAKMValue(oArray[1], type != null ? type : oArray[2]);
                                        } else {
                                            break;
                                        }
                                    } else {
                                        break;
                                    }
                                }

                                const oRecordValue = this.grids.lvlDeviceAKMValue.store.findBy(function (record) {
                                    if (record.get('index').toString() === oArray[2].toString()) {
                                        const id = record.get('_id');
                                        if (id === true && oValue.state === 'active') {
                                            return true;
                                        }
                                        if (id === false && oValue.state === 'inactive') {
                                            return true;
                                        }
                                        if (id === 3 && oValue.state === 'tilted') {
                                            return true;
                                        }
                                        if (id === 2 && oValue.state === 'active') {
                                            return true;
                                        }
                                        if (id === 1 && oValue.state === 'inactive') {
                                            return true;
                                        }
                                    }
                                    return false;
                                });

                                if (oRecordValue != -1) {
                                    this.grids.lvlDeviceAKMValue.getSelectionModel().select(oRecordValue, true);
                                }
                                break;
                            }
                        }
                    }
                    if (oArray[0] === 'zone') {
                        if (!fGridConstructed) {
                            this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 9, false, false, true), true);
                            this.constructGridLvlRoomStateRoom();
                            const oRecordZone = this.grids.lvlRoomStateRoom.store.find('ZoneID', parseInt(oArray[1], 10), 0, false, false, true);
                            if (oRecordZone != -1) {
                                this.grids.lvlRoomStateRoom.getSelectionModel().select(oRecordZone, true);
                                this.constructGridLvlRoomStateState(parseInt(oArray[1], 10));
                                const oRecordState = this.grids.lvlRoomStateState.getStore().find('valueandstate', oValue.name + '_' + oValue.state);
                                if (oRecordState != null) {
                                    this.grids.lvlRoomStateState.getSelectionModel().select(oRecordState);
                                }
                            }
                            break;
                        }
                    }
                } else if (oArray.length === 6) {
                    if (oArray[0] === 'zone') {
                        if (!fGridConstructed) {
                            this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 9, false, false, true), true);
                            this.constructGridLvlRoomStateRoom();
                            const oRecordZone = this.grids.lvlRoomStateRoom.store.find('ZoneID', parseInt(oArray[1], 10), 0, false, false, true);
                            if (oRecordZone != -1) {
                                this.grids.lvlRoomStateRoom.getSelectionModel().select(oRecordZone, true);
                                this.constructGridLvlRoomStateState(parseInt(oArray[1], 10));
                                const oRecordState = this.grids.lvlRoomStateState.getStore().find('valueandstate', oValue.name + '_' + oValue.state);
                                if (oRecordState != null) {
                                    this.grids.lvlRoomStateState.getSelectionModel().select(oRecordState);
                                }
                            }
                            break;
                        }
                    }
                }
                if (oValue.name === 'fire') {
                    if (!fGridConstructed) {
                        this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 6, false, false, true), true);
                        this.constructGridLvlPanic();
                    }
                    const oRecordScene = this.grids.lvlPanic.store.find('_id', oValue.name + '_' + oValue.state, 0, false, false, true);
                    if (oRecordScene != -1) {
                        this.grids.lvlPanic.getSelectionModel().select(oRecordScene, true);
                    }
                } else {
                    if (!fGridConstructed) {
                        this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 5, false, false, true), true);
                        this.constructGridLvlWeather();
                    }
                    const oRecordScene = this.grids.lvlWeather.store.find('_id', oValue.name + '_' + oValue.state, 0, false, false, true);
                    if (oRecordScene != -1) {
                        this.grids.lvlWeather.getSelectionModel().select(oRecordScene, true);
                    }
                }

                break;
            }
            case 'zone-scene': {
                if (oValue.zone == 0) {

                    switch (oValue.scene) {
                        case 65:
                        case 74:
                        case 83:
                        case 84:
                        case 85: {
                            if (!fGridConstructed) {
                                this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 6, false, false, true), true);
                                this.constructGridLvlPanic();
                            }
                            const oRecordScene = this.grids.lvlPanic.store.find('_id', oValue.scene, 0, false, false, true);
                            if (oRecordScene != -1) {
                                this.grids.lvlPanic.getSelectionModel().select(oRecordScene, true);
                            }
                            break;
                        }
                        case 73:
                        case 72:
                        case 71: {
                            if (!fGridConstructed) {
                                this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 4, false, false, true), true);
                                this.constructGridLvlAccess();
                            }
                            const oRecordScene = this.grids.lvlAccess.store.find('_id', oValue.scene, 0, false, false, true);
                            if (oRecordScene != -1) {
                                this.grids.lvlAccess.getSelectionModel().select(oRecordScene, true);
                            }
                            break;
                        }
                    }

                } else {
                    this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 0, false, false, true), true);
                    if (!fGridConstructed) {
                        this.constructGridLvlRoomRoom();
                        const oRecordZone = this.grids.lvlRoomRoom.store.find('ZoneID', oValue.zone, 0, false, false, true);
                        if (oRecordZone != -1) {
                            this.grids.lvlRoomRoom.getSelectionModel().select(oRecordZone);
                        } else {
                            break;
                        }

                        if (oValue.zone == -1) {
                            this.constructGridLvlGroup(0);
                        } else {
                            this.constructGridLvlGroup(oValue.zone);
                        }
                    }
                    if (oValue.group == 0) {
                        const oRecordGroup = this.grids.lvlGroup.store.find('_id', oValue.scene, 0, false, false, true);
                        if (oRecordGroup != -1) {
                            this.grids.lvlGroup.getSelectionModel().select(oRecordGroup, true);
                        } else {
                            break;
                        }
                    } else {
                        const oRecordGroup = this.grids.lvlGroup.store.find('group', oValue.group, 0, false, false, true);
                        if (oRecordGroup != -1) {
                            this.grids.lvlGroup.getSelectionModel().select(oRecordGroup);
                        } else {
                            break;
                        }
                        if (!fGridConstructed) {
                            this.constructGridLvlGroupScene(oValue.zone, oValue.group, false, false);
                        }
                        let oRecordScene = this.grids.lvlScene.store.find('_id', oValue.scene, 0, false, false, true);
                        if (oRecordScene != -1) {
                            if (oValue.forced) {
                                oRecordScene++;
                            }
                            this.grids.lvlScene.getSelectionModel().select(oRecordScene, true);
                        } else {
                            this.grids.lvlScene && this.macPanel.remove(this.grids.lvlScene);
                            if (!fGridConstructed) {
                                this.constructGridLvlGroupScene(oValue.zone, oValue.group, true, false);
                            }
                            oRecordScene = this.grids.lvlScene.store.find('_id', oValue.scene, 0, false, false, true);
                            if (oRecordScene != -1) {
                                if (oValue.forced) {
                                    oRecordScene++;
                                }
                                this.grids.lvlScene.getSelectionModel().select(oRecordScene, true);
                            } else {
                                break;
                            }
                        }

                    }
                }
                break;
            }
            case 'device-scene': {
                const zoneID = dss.staticDataModel.getDevice(oValue.dsuid).ZoneID;
                if (!fGridConstructed) {
                    this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 1, false, false, true), true);
                    this.constructGridLvlDeviceRoom();
                    const oRecordZone = this.grids.lvlDeviceRoom.store.find('ZoneID', zoneID, 0, false, false, true);
                    if (oRecordZone != -1) {
                        this.grids.lvlDeviceRoom.getSelectionModel().select(oRecordZone);
                    } else {
                        break;
                    }
                    this.constructGridLvlDevice(zoneID);
                    const oRecordDevice = this.grids.lvlDevice.store.find('dSUID', oValue.dsuid, 0, false, false, true);
                    if (oRecordDevice != -1) {
                        this.grids.lvlDevice.getSelectionModel().select(oRecordDevice);
                    } else {
                        break;
                    }
                    const fConstructed = this.constructGridLvlDeviceActionClass(oValue.dsuid);
                    const oRecordDeviceClass = this.grids.lvlDeviceActionClass.store.find('_id', 4, 0, false, false, true);
                    if (oRecordDeviceClass != -1) {
                        this.grids.lvlDeviceActionClass.getSelectionModel().select(oRecordDeviceClass);
                    } else {
                        break;
                    }
                    if (!fConstructed) {
                        this.constructGridLvlDeviceActions(oValue.dsuid);
                    }
                }
                const jIndex = this.grids.lvlDeviceAction.store.find('_id', oValue.scene, 0, false, false, true);
                if (jIndex != -1) {
                    this.grids.lvlDeviceAction.getSelectionModel().select(jIndex, true);
                } else {
                    const oDevice = dss.staticDataModel.getDevice(oValue.dsuid);
                    if (oDevice.tasterMode == 0) {
                        if (oDevice.buttonType == 65) {
                            let kIndex;
                            switch (oValue.scene) {
                            case 15:
                                kIndex = this.grids.lvlDeviceAction.store.find('_id', 1004, 0, false, false, true);
                                if (kIndex != -1) {
                                    this.grids.lvlDeviceAction.getSelectionModel().select(kIndex, true);
                                }
                                break;
                            case 50:
                                kIndex = this.grids.lvlDeviceAction.store.find('_id', 1006, 0, false, false, true);
                                if (kIndex != -1) {
                                    this.grids.lvlDeviceAction.getSelectionModel().select(kIndex, true);
                                }
                                break;
                            case 51:
                                kIndex = this.grids.lvlDeviceAction.store.find('_id', 1005, 0, false, false, true);
                                if (kIndex != -1) {
                                    this.grids.lvlDeviceAction.getSelectionModel().select(kIndex, true);
                                }
                                break;
                            }
                        }
                    }
                }
                break;
            }
            case 'custom-event': {
                if (!fGridConstructed) {
                    this.constructGridLvlHigh();
                }
                this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 2, false, false, true), true);
                var oRecord = this.grids.lvlHighLevelEvent.store.find('_id', oValue.event, 0, false, false, true);
                if (oRecord != -1) {
                    this.grids.lvlHighLevelEvent.getSelectionModel().select(oRecord, true);
                }
                break;
            }
            case 'event':
                switch (oValue.name) {
                    case 'sunshine': {
                        if (!fGridConstructed) {
                            this.grids.lvl1.getSelectionModel().select(this.grids.lvl1.store.find('_id', 5, false, false, true), true);
                            this.constructGridLvlWeather();
                            this.grids.lvlWeather.getSelectionModel().select(this.grids.lvlWeather.store.find('_id', 'sun', 0, false, false, true), true);
                            this.constructGridLvlSunProtection();
                        }
                        const jIndex = this.grids.lvlSunProtection.store.find(
                            '_id', 'sun_' + oValue.parameter.direction + (oValue.parameter.value == 1 ? '_active' : '_inactive'), 0, false, false, true
                        );
                        if (jIndex != -1) {
                            this.grids.lvlSunProtection.getSelectionModel().select(jIndex, true);
                        }
                        break;
                    }
                }
                break;
            }
            fGridConstructed = true;
            this.macPanel.scrollBy(1000, 0, false);
        }
    },
    isValid: function () {
        const oValue = this.getValue();
        return (oValue.length);
    },
    getValue: function () {
        var oReturnValue = [];
        try {
            const oSelectionLVL1 = this.grids.lvl1.getSelectionModel().getSelection();
            for (let iIndex = 0; oSelectionLVL1 && iIndex < oSelectionLVL1.length; iIndex++) {
                const selectionLvl1Id = oSelectionLVL1[iIndex].get('_id');
                if (selectionLvl1Id == 4) {
                    // access trigger
                    const oSelectionScene = this.grids.lvlAccess && this.grids.lvlAccess.getSelectionModel().getSelection();
                    if (oSelectionScene) {
                        for (let kIndex = 0; kIndex < oSelectionScene.length; kIndex++) {

                            if (oSelectionScene[kIndex].get('_id') < 1000) {
                                const oObject = {
                                    type: 'zone-scene',
                                    zone: 0,
                                    group: 0,
                                    scene: oSelectionScene[kIndex].get('_id')
                                };
                                oReturnValue.push(oObject);
                            } else {
                                const oObject = {
                                    type: 'undo-zone-scene',
                                    zone: 0,
                                    group: 0,
                                    scene: oSelectionScene[kIndex].get('_id') - 1000
                                };
                                oReturnValue.push(oObject);
                            }
                        }
                    }
                }
                if (selectionLvl1Id == 9) {
                    const oSelectionScene = this.grids.lvlRoomStateState && this.grids.lvlRoomStateState.getSelectionModel().getSelection();
                    // zonestate
                    // name, state: 'active', 'inactive'
                    if (oSelectionScene) {
                        for (let kIndex = 0; kIndex < oSelectionScene.length; kIndex++) {
                            const sValue = oSelectionScene[kIndex].get('value');
                            const stateName = oSelectionScene[kIndex].get('stateid');
                            const oObject = {
                                type: 'state-change',
                                name: stateName,
                                state: sValue
                            };
                            oReturnValue.push(oObject);
                        }
                    }
                }
                if (selectionLvl1Id == 8) {
                    var userStates = this.grids.lvlUserstate && this.grids.lvlUserstate.getSelectionModel().getSelection();
                    var userStateValues = this.grids.lvlUserstateValue && this.grids.lvlUserstateValue.getSelectionModel().getSelection();
                    if (userStates && userStates[0] && userStateValues && userStateValues[0]) {
                        const stateName = userStates[0].get('_id');
                        const stateType = userStates[0].get('type');
                        const stateValue = userStateValues[0].get('_id');

                        // usertate
                        // name, state: 'active', 'inactive'
                        if (stateType === 'zone-sensor-states' || stateType === 'device-sensor-states') {
                            const oObject = {
                                type: 'state-change',
                                name: stateName,
                                state: stateValue
                            };
                            oReturnValue.push(oObject);
                        } else {
                            const oObject = {
                                type: 'addon-state-change',
                                name: stateName,
                                state: stateValue,
                                'addon-id': 'system-addon-user-defined-states',
                                addonId: 'system-addon-user-defined-states'
                            };
                            oReturnValue.push(oObject);
                        }
                    }
                }
                if (selectionLvl1Id == 5) {
                    const oSelectionScene = this.grids.lvlWeather && this.grids.lvlWeather.getSelectionModel().getSelection();
                    // weather
                    // name, state: 'active', 'inactive'
                    if (oSelectionScene) {
                        for (let kIndex = 0; kIndex < oSelectionScene.length; kIndex++) {
                            const sValue = oSelectionScene[kIndex].get('_id');
                            if (sValue !== 'sun') {
                                const stateName = sValue.substr(0, sValue.lastIndexOf('_'));
                                const stateValue = sValue.substr(sValue.lastIndexOf('_') + 1);
                                const oObject = {
                                    type: 'state-change',
                                    name: stateName,
                                    state: stateValue
                                };
                                oReturnValue.push(oObject);
                            } else {
                                const oSelectionSunProtection = this.grids.lvlSunProtection && this.grids.lvlSunProtection.getSelectionModel().getSelection();
                                for (let pIndex = 0; pIndex < oSelectionSunProtection.length; pIndex++) {
                                    const eventName = oSelectionSunProtection[pIndex].get('eventName');
                                    const direction = oSelectionSunProtection[pIndex].get('direction');
                                    const value = oSelectionSunProtection[pIndex].get('value');
                                    const oObject = {
                                        type: 'event',
                                        name: eventName,
                                        parameter: {
                                            value: value,
                                            direction: direction
                                        }
                                    };
                                    oReturnValue.push(oObject);
                                }
                            }
                        }
                    }
                }
                if (selectionLvl1Id == 6) {
                    const oSelectionScene = this.grids.lvlPanic && this.grids.lvlPanic.getSelectionModel().getSelection();
                    // panic
                    if (oSelectionScene) {
                        for (let kIndex = 0; kIndex < oSelectionScene.length; kIndex++) {
                            if (oSelectionScene[kIndex].get('_id').toString().indexOf('_') >= 0) {
                                const sValue = oSelectionScene[kIndex].get('_id');
                                const stateName = sValue.substr(0, sValue.lastIndexOf('_'));
                                const stateValue = sValue.substr(sValue.lastIndexOf('_') + 1);
                                const oObject = {
                                    type: 'state-change',
                                    name: stateName,
                                    state: stateValue
                                };
                                oReturnValue.push(oObject);
                            } else if (oSelectionScene[kIndex].get('_id') < 1000) {
                                const oObject = {
                                    type: 'zone-scene',
                                    zone: 0,
                                    group: 0,
                                    scene: oSelectionScene[kIndex].get('_id')
                                };
                                oReturnValue.push(oObject);
                            } else {
                                const oObject = {
                                    type: 'undo-zone-scene',
                                    zone: 0,
                                    group: 0,
                                    scene: oSelectionScene[kIndex].get('_id') - 1000
                                };
                                oReturnValue.push(oObject);
                            }
                        }
                    }
                }
                if (selectionLvl1Id == 7) {
                    const oSelectionDevice = this.grids.lvlAKMDevice && this.grids.lvlAKMDevice.getSelectionModel().getSelection()[0];
                    const dSUID = oSelectionDevice && oSelectionDevice.get('dSUID');
                    const oSelectionIndex = this.grids.lvlDeviceAKMValue && this.grids.lvlDeviceAKMValue.getSelectionModel().getSelection()[0];
                    const index = oSelectionIndex && oSelectionIndex.get('index');
                    if (dSUID && index != null) {
                        const oDevice = dss.staticDataModel.getDevice(dSUID);
                        const bIsWindowHandle = dss.staticDataModel.IsWindowHandle(oDevice);

                        const oSelectionValue = oSelectionIndex.get('_id');
                        if (!bIsWindowHandle) {
                            const oObject = {
                                type: 'state-change',
                                name: 'dev.' + dSUID + '.' + index,
                                state: (oSelectionValue ? 'active' : 'inactive')
                            };
                            oReturnValue.push(oObject);
                        } else {
                            const oObject = {
                                type: 'state-change',
                                name: 'dev.' + dSUID + '.' + index,
                                state: 'invalid'
                            };
                            if (oSelectionValue == dss.whState.WH_Closed) {
                                oObject.state = 'closed';
                            } else if (oSelectionValue == dss.whState.WH_Open) {
                                oObject.state = 'open';
                            } else if (oSelectionValue == dss.whState.WH_Tilted) {
                                oObject.state = 'tilted';
                            }
                            oReturnValue.push(oObject);
                        }
                    }
                }
                if (selectionLvl1Id == 0) { // zone scene
                    const roomSelection = this.grids.lvlRoomRoom && this.grids.lvlRoomRoom.getSelectionModel().getSelection()[0];
                    const oSelectionGroup = this.grids.lvlGroup && this.grids.lvlGroup.getSelectionModel().getSelection();
                    if (oSelectionGroup && roomSelection) {
                        const iZoneID = roomSelection.get('ZoneID');
                        for (let jIndex = 0, _id; jIndex < oSelectionGroup.length; jIndex++) {
                            _id = oSelectionGroup[jIndex].get('_id');
                            if (_id === SHOW_ALL_SCENES_ID) {
                                continue;
                            }
                            if (oSelectionGroup[jIndex].get('group') == null) {
                                const oObject = {
                                    type: 'zone-scene',
                                    zone: iZoneID,
                                    group: 0,
                                    scene: _id
                                };
                                oReturnValue.push(oObject);
                            } else {
                                const iGroupID = oSelectionGroup[jIndex].get('group');
                                const oSelectionScene = this.grids.lvlScene && this.grids.lvlScene.getSelectionModel().getSelection();
                                if (oSelectionScene) {
                                    for (let kIndex = 0; kIndex < oSelectionScene.length; kIndex++) {
                                        _id = oSelectionScene[kIndex].get('_id');
                                        if (_id === SHOW_ALL_SCENES_ID) {
                                            continue;
                                        }
                                        if (oSelectionScene[kIndex].get('stateid')) {
                                            const oObject = {
                                                type: 'state-change',
                                                name: oSelectionScene[kIndex].get('stateid'),
                                                state: oSelectionScene[kIndex].get('value')
                                            };
                                            oReturnValue.push(oObject);
                                        } else {
                                            const oObject = {
                                                type: 'zone-scene',
                                                zone: iZoneID,
                                                group: iGroupID,
                                                scene: _id
                                            };
                                            if (oSelectionScene[kIndex].get('forced')) {
                                                oObject.forced = oSelectionScene[kIndex].get('forced');
                                            }

                                            // else
                                            // oObject.forced = false;
                                            oReturnValue.push(oObject);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                if (selectionLvl1Id == 1) { // taster
                    const deviceSelection = this.grids.lvlDevice && this.grids.lvlDevice.getSelectionModel().getSelection()[0];
                    if (deviceSelection == null) {
                        return oReturnValue;
                    }
                    var oDSID = deviceSelection.get('dSID');
                    var oDSUID = deviceSelection.get('dSUID');

                    const actionSelection = this.grids.lvlDeviceActionClass && this.grids.lvlDeviceActionClass.getSelectionModel().getSelection()[0];
                    const actionType = actionSelection && actionSelection.get('_id');
                    if (actionType == 1) {
                        // device events
                        const selection = this.grids.lvlDeviceEventEvent && this.grids.lvlDeviceEventEvent.getSelectionModel().getSelection();
                        if (selection) {
                            for (let jIndex = 0; jIndex < selection.length; jIndex++) {
                                if (selection[jIndex].get('type') === 'category') {
                                    const selection2 = this.grids.lvlDeviceEventEventSub && this.grids.lvlDeviceEventEventSub.getSelectionModel().getSelection();
                                    if (selection2) {
                                        for (let kIndex = 0; kIndex < selection2.length; kIndex++) {
                                            const event = selection2[kIndex].get('event');
                                            const oObject = {
                                                type: 'device-named-event',
                                                dsuid: oDSUID,
                                                id: event
                                            };
                                            oReturnValue.push(oObject);
                                        }
                                    }
                                } else if (selection[jIndex].get('type') === 'event') {
                                    const event = selection[jIndex].get('event');
                                    const oObject = {
                                        type: 'device-named-event',
                                        dsuid: oDSUID,
                                        id: event
                                    };
                                    oReturnValue.push(oObject);
                                }
                            }
                        }
                    }
                    if (actionType == 2) {
                        // device actions
                        const selection = this.grids.lvlDeviceActionAction && this.grids.lvlDeviceActionAction.getSelectionModel().getSelection();
                        if (selection) {

                            for (let jIndex = 0; jIndex < selection.length; jIndex++) {
                                if (selection[jIndex].get('type') === 'category') {
                                    const selection2 = this.grids.lvlDeviceActionActionSub && this.grids.lvlDeviceActionActionSub.getSelectionModel().getSelection();
                                    if (selection2) {
                                        for (let kIndex = 0; kIndex < selection2.length; kIndex++) {
                                            const action = selection2[kIndex].get('action');
                                            const oObject = {
                                                type: 'device-named-action',
                                                dsuid: oDSUID,
                                                id: action
                                            };
                                            oReturnValue.push(oObject);
                                        }
                                    }
                                } else if (selection[jIndex].get('type') === 'action') {
                                    const action = selection[jIndex].get('action');
                                    const oObject = {
                                        type: 'device-named-action',
                                        dsuid: oDSUID,
                                        id: action
                                    };
                                    oReturnValue.push(oObject);
                                }
                            }
                        }
                    }
                    if (actionType == 3) {
                        // device states
                        const selection = this.grids.lvlDeviceStateValue && this.grids.lvlDeviceStateValue.getSelectionModel().getSelection();
                        if (selection) {
                            for (let jIndex = 0; jIndex < selection.length; jIndex++) {
                                const value = selection[jIndex].get('type');
                                const state = selection[jIndex].get('selector');
                                const oObject = {
                                    type: 'state-change',
                                    name: state,
                                    state: value
                                };
                                oReturnValue.push(oObject);
                            }
                        }
                    }
                    if (actionType == 4) {
                        var oSelectionDeviceAction = this.grids.lvlDeviceAction && this.grids.lvlDeviceAction.getSelectionModel().getSelection();

                        var oDevice = dss.staticDataModel.getDevice(oDSUID);

                        for (let jIndex = 0; oSelectionDeviceAction && jIndex < oSelectionDeviceAction.length; jIndex++) {
                            const selectionId = oSelectionDeviceAction[jIndex].get('_id');
                            var fSpecial = false;
                            if (oDevice.specialActions.triggers) {
                                if (oDevice.specialActions.triggers.deviceMsg) {
                                    for (const sKey in oDevice.specialActions.triggers.deviceMsg) {
                                        if (selectionId == oDevice.specialActions.triggers.deviceMsg[sKey].id) {
                                            fSpecial = true;
                                            const oObject = {
                                                dsid: oDSID,
                                                dsuid: oDSUID,
                                                type: 'device-msg',
                                                msg: oDevice.specialActions.triggers.deviceMsg[sKey].id
                                            };
                                            oReturnValue.push(oObject);
                                        }
                                    }
                                }
                                if (oDevice.specialActions.triggers.deviceAction) {
                                    for (const sKey in oDevice.specialActions.triggers.deviceAction) {
                                        if (selectionId == oDevice.specialActions.triggers.deviceAction[sKey].id) {
                                            fSpecial = true;
                                            const oObject = {
                                                dsid: oDSID,
                                                dsuid: oDSUID,
                                                type: 'device-action',
                                                action: oDevice.specialActions.triggers.deviceAction[sKey].id
                                            };
                                            oReturnValue.push(oObject);
                                        }
                                    }
                                }
                                if (oDevice.specialActions.triggers.deviceActions) {
                                    for (const sKey in oDevice.specialActions.triggers.deviceActions) {
                                        if (selectionId == oDevice.specialActions.triggers.deviceActions[sKey].action) {
                                            fSpecial = true;
                                            const oObject = {
                                                dsid: oDSID,
                                                dsuid: oDSUID,
                                                type: 'device-action',
                                                action: oDevice.specialActions.triggers.deviceActions[sKey].action
                                            };
                                            oReturnValue.push(oObject);
                                        }
                                    }
                                }

                            }
                            if (!fSpecial) {
                                if (selectionId > 15 && selectionId < 1000) {
                                    const oObject = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-scene',
                                        scene: selectionId
                                    };
                                    oReturnValue.push(oObject);
                                } else if (selectionId == 1100) {
                                    const oObject1 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        buttonIndex: 1,
                                        type: 'device-msg',
                                        msg: 0
                                    };
                                    oReturnValue.push(oObject1);
                                    const oObject2 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        buttonIndex: 1,
                                        type: 'device-msg',
                                        msg: 7
                                    };
                                    oReturnValue.push(oObject2);
                                } else if (selectionId == 1200) {
                                    const oObject1 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        buttonIndex: 2,
                                        type: 'device-msg',
                                        msg: 0
                                    };
                                    oReturnValue.push(oObject1);
                                    const oObject2 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        buttonIndex: 2,
                                        type: 'device-msg',
                                        msg: 7
                                    };
                                    oReturnValue.push(oObject2);
                                } else if (selectionId == 0) {
                                    const oObject1 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        msg: 0
                                    };
                                    oReturnValue.push(oObject1);
                                    const oObject2 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        msg: 7
                                    };
                                    oReturnValue.push(oObject2);
                                } else if (selectionId == 1) {
                                    const oObject1 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        msg: 1
                                    };
                                    oReturnValue.push(oObject1);
                                    const oObject2 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        msg: 8
                                    };
                                    oReturnValue.push(oObject2);
                                } else if (selectionId == 1101) {
                                    const oObject1 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        buttonIndex: 1,
                                        msg: 1
                                    };
                                    oReturnValue.push(oObject1);
                                    const oObject2 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        buttonIndex: 1,
                                        msg: 8
                                    };
                                    oReturnValue.push(oObject2);
                                } else if (selectionId == 1201) {
                                    const oObject1 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        buttonIndex: 2,
                                        msg: 1
                                    };
                                    oReturnValue.push(oObject1);
                                    const oObject2 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        buttonIndex: 2,
                                        msg: 8
                                    };
                                    oReturnValue.push(oObject2);
                                } else if (selectionId == 2) {
                                    const oObject1 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        msg: 2
                                    };
                                    oReturnValue.push(oObject1);
                                    const oObject2 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        msg: 9
                                    };
                                    oReturnValue.push(oObject2);
                                } else if (selectionId == 1102) {
                                    const oObject1 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        buttonIndex: 1,
                                        msg: 2
                                    };
                                    oReturnValue.push(oObject1);
                                    const oObject2 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        buttonIndex: 1,
                                        msg: 9
                                    };
                                    oReturnValue.push(oObject2);
                                } else if (selectionId == 1202) {
                                    const oObject1 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        buttonIndex: 2,
                                        msg: 2
                                    };
                                    oReturnValue.push(oObject1);
                                    const oObject2 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        buttonIndex: 2,
                                        msg: 9
                                    };
                                    oReturnValue.push(oObject2);
                                } else if (selectionId == 1103) {
                                    const oObject1 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        buttonIndex: 1,
                                        msg: 3
                                    };
                                    oReturnValue.push(oObject1);
                                } else if (selectionId == 1203) {
                                    const oObject1 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        buttonIndex: 2,
                                        msg: 3
                                    };
                                    oReturnValue.push(oObject1);

                                } else if (selectionId == 1106) {
                                    const oObject1 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        buttonIndex: 1,
                                        msg: 6
                                    };
                                    oReturnValue.push(oObject1);
                                } else if (selectionId == 1206) {
                                    const oObject1 = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        buttonIndex: 2,
                                        msg: 6
                                    };
                                    oReturnValue.push(oObject1);
                                } else if (selectionId == 1000) {
                                    const oObject1 = { // switch down
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        msg: 0,
                                        buttonIndex: 2
                                    };
                                    oReturnValue.push(oObject1);
                                } else if (selectionId == 1001) {
                                    const oObject1 = { // switch up
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        msg: 0,
                                        buttonIndex: 1
                                    };
                                    oReturnValue.push(oObject1);
                                } else if (selectionId == 1002) {
                                    const oObject1 = { // switch down
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-binary-input',
                                        index: 0,
                                        state: 1
                                    };
                                    oReturnValue.push(oObject1);
                                } else if (selectionId == 1003) {
                                    const oObject1 = { // switch up
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-binary-input',
                                        index: 0,
                                        state: 0
                                    };
                                    oReturnValue.push(oObject1);
                                } else if (selectionId == 1004) {
                                    const oObject1 = { // controlled mode
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-scene',
                                        scene: 15
                                    };
                                    oReturnValue.push(oObject1);
                                } else if (selectionId == 1005) {
                                    const oObject1 = { // forced on
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-scene',
                                        scene: 51
                                    };
                                    oReturnValue.push(oObject1);
                                } else if (selectionId == 1006) {
                                    const oObject1 = { // forced off
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-scene',
                                        scene: 50
                                    };
                                    oReturnValue.push(oObject1);
                                } else {
                                    const oObject = {
                                        dsid: oDSID,
                                        dsuid: oDSUID,
                                        type: 'device-msg',
                                        msg: selectionId
                                    };
                                    oReturnValue.push(oObject);
                                }
                            }
                        }
                    }
                }
                if (selectionLvl1Id == 2) { // highlvl
                    const oSelectionHighLVL = this.grids.lvlHighLevelEvent && this.grids.lvlHighLevelEvent.getSelectionModel().getSelection();
                    if (oSelectionHighLVL) {
                        for (let jIndex = 0; jIndex < oSelectionHighLVL.length; jIndex++) {
                            const oObject = {
                                type: 'custom-event',
                                event: oSelectionHighLVL[jIndex].get('_id')
                            };
                            oReturnValue.push(oObject);
                        }
                    }
                }
                if (selectionLvl1Id == 3) { // sensor
                    const oSensorDeviceSelection = this.grids.lvlSensorDevice && this.grids.lvlSensorDevice.getSelectionModel().getSelection()[0];
                    const oDSID = oSensorDeviceSelection && oSensorDeviceSelection.get('dSID');
                    const oDSUID = oSensorDeviceSelection && oSensorDeviceSelection.get('dSUID');
                    const oSelectionMSG = this.grids.lvlSensorMessage && this.grids.lvlSensorMessage.getSelectionModel().getSelection();
                    if (oSelectionMSG) {
                        for (let jIndex = 0; jIndex < oSelectionMSG.length; jIndex++) {
                            const oObject = {
                                type: 'device-sensor',
                                dsid: oDSID,
                                dsuid: oDSUID,
                                eventid: oSelectionMSG[jIndex].get('_id')
                            };
                            oReturnValue.push(oObject);
                        }
                    }
                }
                if (selectionLvl1Id == 12) { // global application state
                    const oGroupIdSelection = this.grids.lvlGlobalApplicationState && this.grids.lvlGlobalApplicationState.getSelectionModel().getSelection()[0];
                    const groupId = oGroupIdSelection && oGroupIdSelection.get('GroupID');
                    const oStateGroupSelection = this.grids.lvlGlobalApplicationStateGroup && this.grids.lvlGlobalApplicationStateGroup.getSelectionModel().getSelection()[0];
                    const state = oStateGroupSelection && oStateGroupSelection.get('stateid');
                    const value = oStateGroupSelection && oStateGroupSelection.get('value');
                    if (groupId != null && state != null && value != null) {
                        const oObject = {
                            type: 'state-change',
                            name: state,
                            state: value
                        };
                        oReturnValue.push(oObject);
                    }
                }
            }
        } catch (e) {
            devLog && devLog(e);
        }
        return oReturnValue;
    }
}); // DSS.component.Trigger
